@@ -6,6 +6,7 @@
 #include "Emu/Cell/PPUModule.h"
 #include "Emu/Cell/lv2/sys_event.h"
 #include "Emu/IdManager.h"
+#include "Emu/Cell/timers.hpp"
 
 #include <cmath>
 
@@ -60,11 +61,6 @@ void fmt_class_string<CellCameraFormat>::format(std::string& out, u64 arg)
 	});
 }
 
-// Temporarily
-#ifndef _MSC_VER
-#pragma GCC diagnostic ignored "-Wunused-parameter"
-#endif
-
 // **************
 // * Prototypes *
 // **************
@@ -401,7 +397,7 @@ error_code check_init_and_open(s32 dev_num)
 }
 
 // This represents a recurring subfunction throughout libCamera
-error_code check_resolution(s32 dev_num)
+error_code check_resolution(s32 /*dev_num*/)
 {
 	// TODO: Some sort of connection check maybe?
 	//if (error == CELL_CAMERA_ERROR_RESOLUTION_UNKNOWN)
@@ -412,7 +408,7 @@ error_code check_resolution(s32 dev_num)
 	return CELL_OK;
 }
 
-// This represents a oftenly used sequence in libCamera (usually the beginning of a subfunction).
+// This represents an often used sequence in libCamera (usually the beginning of a subfunction).
 // There also exist common sequences for mutex lock/unlock by the way.
 error_code check_resolution_ex(s32 dev_num)
 {
@@ -1143,8 +1139,10 @@ error_code cellCameraGetBufferInfo(s32 dev_num, vm::ptr<CellCameraInfo> info)
 	return CELL_OK;
 }
 
-error_code cellCameraGetBufferInfoEx(s32 dev_num, vm::ptr<CellCameraInfoEx> info)
+error_code cellCameraGetBufferInfoEx(ppu_thread& ppu, s32 dev_num, vm::ptr<CellCameraInfoEx> info)
 {
+	ppu.state += cpu_flag::wait;
+
 	cellCamera.notice("cellCameraGetBufferInfoEx(dev_num=%d, info=0x%x)", dev_num, info);
 
 	// calls cellCameraGetBufferInfo
@@ -1155,10 +1153,16 @@ error_code cellCameraGetBufferInfoEx(s32 dev_num, vm::ptr<CellCameraInfoEx> info
 	}
 
 	auto& g_camera = g_fxo->get<camera_thread>();
-	std::lock_guard lock(g_camera.mutex);
 
-	*info = g_camera.info;
+	CellCameraInfoEx info_out;
 
+	{
+		std::lock_guard lock(g_camera.mutex);
+
+		info_out = g_camera.info;
+	}
+
+	*info = info_out;
 	return CELL_OK;
 }
 
