@@ -3,7 +3,6 @@
 #include "Emu/system_config.h"
 #include "Emu/Cell/PPUModule.h"
 #include "Emu/Cell/lv2/sys_process.h"
-#include "Emu/Cell/lv2/sys_sync.h"
 #include "Emu/Io/pad_types.h"
 #include "Emu/RSX/Overlays/overlay_debug_overlay.h"
 #include "Input/pad_thread.h"
@@ -77,6 +76,9 @@ void pad_info::save(utils::serial& ar)
 void show_debug_overlay(const CellPadData& data, const Pad& pad, const pad_info& config)
 {
 	const u32 setting = config.port_setting[pad.m_player_id];
+	const CellPadData& raw = pad.data;
+	const u16 raw_d1 = raw.button[CELL_PAD_BTN_OFFSET_DIGITAL1];
+	const u16 raw_d2 = raw.button[CELL_PAD_BTN_OFFSET_DIGITAL2];
 	const u16 d1 = data.button[CELL_PAD_BTN_OFFSET_DIGITAL1];
 	const u16 d2 = data.button[CELL_PAD_BTN_OFFSET_DIGITAL2];
 
@@ -87,6 +89,8 @@ void show_debug_overlay(const CellPadData& data, const Pad& pad, const pad_info&
 		">     Digital:   %5s   %5s\n"
 		">       Press:   %5s   %5s\n"
 		">      Sensor:   %5s   %5s\n"
+		"> Large Motor:   %5d   %5d\n"
+		"> Small Motor:   %5d   %5d\n"
 		">\n"
 		">   Digital 1:  0x%04x  0x%04x\n"
 		">   Digital 2:  0x%04x  0x%04x\n"
@@ -127,33 +131,35 @@ void show_debug_overlay(const CellPadData& data, const Pad& pad, const pad_info&
 		"on", data.len >= CELL_PAD_LEN_CHANGE_DEFAULT ? "on" : "off",
 		(setting & CELL_PAD_SETTING_PRESS_ON) ? "on" : "off", data.len >= CELL_PAD_LEN_CHANGE_PRESS_ON ? "on" : "off",
 		(setting & CELL_PAD_SETTING_SENSOR_ON) ? "on" : "off", data.len >= CELL_PAD_LEN_CHANGE_SENSOR_ON ? "on" : "off",
-		pad.m_digital_1, d1,
-		pad.m_digital_2, d2,
-		pad.m_press_up, !!(d1 & CELL_PAD_CTRL_UP), data.button[CELL_PAD_BTN_OFFSET_PRESS_UP],
-		pad.m_press_down, !!(d1 & CELL_PAD_CTRL_DOWN), data.button[CELL_PAD_BTN_OFFSET_PRESS_DOWN],
-		pad.m_press_left, !!(d1 & CELL_PAD_CTRL_LEFT), data.button[CELL_PAD_BTN_OFFSET_PRESS_LEFT],
-		pad.m_press_right, !!(d1 & CELL_PAD_CTRL_RIGHT), data.button[CELL_PAD_BTN_OFFSET_PRESS_RIGHT],
-		pad.m_press_cross, !!(d2 & CELL_PAD_CTRL_CROSS), data.button[CELL_PAD_BTN_OFFSET_PRESS_CROSS],
-		pad.m_press_square, !!(d2 & CELL_PAD_CTRL_SQUARE), data.button[CELL_PAD_BTN_OFFSET_PRESS_SQUARE],
-		pad.m_press_circle, !!(d2 & CELL_PAD_CTRL_CIRCLE), data.button[CELL_PAD_BTN_OFFSET_PRESS_CIRCLE],
-		pad.m_press_triangle, !!(d2 & CELL_PAD_CTRL_TRIANGLE), data.button[CELL_PAD_BTN_OFFSET_PRESS_TRIANGLE],
-		!!(pad.m_digital_1 & CELL_PAD_CTRL_START), !!(d1 & CELL_PAD_CTRL_START),
-		!!(pad.m_digital_1 & CELL_PAD_CTRL_SELECT), !!(d1 & CELL_PAD_CTRL_SELECT),
-		!!(pad.m_digital_1 & CELL_PAD_CTRL_PS), !!(d1 & CELL_PAD_CTRL_PS),
-		pad.m_press_L1, !!(d2 & CELL_PAD_CTRL_L1), data.button[CELL_PAD_BTN_OFFSET_PRESS_L1],
-		pad.m_press_L2, !!(d2 & CELL_PAD_CTRL_L2), data.button[CELL_PAD_BTN_OFFSET_PRESS_L2],
-		!!(pad.m_digital_1 & CELL_PAD_CTRL_L3), !!(d1 & CELL_PAD_CTRL_L3),
-		pad.m_press_R1, !!(d2 & CELL_PAD_CTRL_R1), data.button[CELL_PAD_BTN_OFFSET_PRESS_R1],
-		pad.m_press_R2, !!(d2 & CELL_PAD_CTRL_R2), data.button[CELL_PAD_BTN_OFFSET_PRESS_R2],
-		!!(pad.m_digital_1 & CELL_PAD_CTRL_R3), !!(d1 & CELL_PAD_CTRL_R3),
-		pad.m_analog_left_x, data.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X],
-		pad.m_analog_left_y, data.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y],
-		pad.m_analog_right_x, data.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X],
-		pad.m_analog_right_y, data.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y],
-		pad.m_sensor_x, data.button[CELL_PAD_BTN_OFFSET_SENSOR_X],
-		pad.m_sensor_y, data.button[CELL_PAD_BTN_OFFSET_SENSOR_Y],
-		pad.m_sensor_z, data.button[CELL_PAD_BTN_OFFSET_SENSOR_Z],
-		pad.m_sensor_g, data.button[CELL_PAD_BTN_OFFSET_SENSOR_G],
+		pad.m_vibrate_motors[0].value, pad.m_vibrate_motors[0].adjusted_value,
+		pad.m_vibrate_motors[1].value, pad.m_vibrate_motors[1].adjusted_value,
+		raw_d1, d1,
+		raw_d2, d2,
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_UP],       !!(d1 & CELL_PAD_CTRL_UP),       data.button[CELL_PAD_BTN_OFFSET_PRESS_UP],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_DOWN],     !!(d1 & CELL_PAD_CTRL_DOWN),     data.button[CELL_PAD_BTN_OFFSET_PRESS_DOWN],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_LEFT],     !!(d1 & CELL_PAD_CTRL_LEFT),     data.button[CELL_PAD_BTN_OFFSET_PRESS_LEFT],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_RIGHT],    !!(d1 & CELL_PAD_CTRL_RIGHT),    data.button[CELL_PAD_BTN_OFFSET_PRESS_RIGHT],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_CROSS],    !!(d2 & CELL_PAD_CTRL_CROSS),    data.button[CELL_PAD_BTN_OFFSET_PRESS_CROSS],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_SQUARE],   !!(d2 & CELL_PAD_CTRL_SQUARE),   data.button[CELL_PAD_BTN_OFFSET_PRESS_SQUARE],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_CIRCLE],   !!(d2 & CELL_PAD_CTRL_CIRCLE),   data.button[CELL_PAD_BTN_OFFSET_PRESS_CIRCLE],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_TRIANGLE], !!(d2 & CELL_PAD_CTRL_TRIANGLE), data.button[CELL_PAD_BTN_OFFSET_PRESS_TRIANGLE],
+		!!(raw_d1 & CELL_PAD_CTRL_START),               !!(d1 & CELL_PAD_CTRL_START),
+		!!(raw_d1 & CELL_PAD_CTRL_SELECT),              !!(d1 & CELL_PAD_CTRL_SELECT),
+		!!(raw_d1 & CELL_PAD_CTRL_PS),                  !!(d1 & CELL_PAD_CTRL_PS),
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_L1],       !!(d2 & CELL_PAD_CTRL_L1),       data.button[CELL_PAD_BTN_OFFSET_PRESS_L1],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_L2],       !!(d2 & CELL_PAD_CTRL_L2),       data.button[CELL_PAD_BTN_OFFSET_PRESS_L2],
+		!!(raw_d1 & CELL_PAD_CTRL_L3),                  !!(d1 & CELL_PAD_CTRL_L3),
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_R1],       !!(d2 & CELL_PAD_CTRL_R1),       data.button[CELL_PAD_BTN_OFFSET_PRESS_R1],
+		raw.button[CELL_PAD_BTN_OFFSET_PRESS_R2],       !!(d2 & CELL_PAD_CTRL_R2),       data.button[CELL_PAD_BTN_OFFSET_PRESS_R2],
+		!!(raw_d1 & CELL_PAD_CTRL_R3),                  !!(d1 & CELL_PAD_CTRL_R3),
+		raw.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X],  data.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X],
+		raw.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y],  data.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y],
+		raw.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X], data.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X],
+		raw.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y], data.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y],
+		raw.button[CELL_PAD_BTN_OFFSET_SENSOR_X],       data.button[CELL_PAD_BTN_OFFSET_SENSOR_X],
+		raw.button[CELL_PAD_BTN_OFFSET_SENSOR_Y],       data.button[CELL_PAD_BTN_OFFSET_SENSOR_Y],
+		raw.button[CELL_PAD_BTN_OFFSET_SENSOR_Z],       data.button[CELL_PAD_BTN_OFFSET_SENSOR_Z],
+		raw.button[CELL_PAD_BTN_OFFSET_SENSOR_G],       data.button[CELL_PAD_BTN_OFFSET_SENSOR_G],
 		pad.m_product_id,
 		pad.m_vendor_id,
 		pad.m_device_type,
@@ -273,7 +279,7 @@ error_code cellPadInit(ppu_thread& ppu, u32 max_connect)
 
 	for (usz i = 0; i < config.get_max_connect(); ++i)
 	{
-		if (!pads[i]->is_fake_pad && (pads[i]->m_port_status & CELL_PAD_STATUS_CONNECTED))
+		if (!pads[i]->is_fake_pad && pads[i]->is_connected())
 		{
 			send_sys_io_connect_event(i, CELL_PAD_STATUS_CONNECTED);
 		}
@@ -306,17 +312,15 @@ void clear_pad_buffer(const std::shared_ptr<Pad>& pad)
 	// might as well also reset everything in our pad 'buffer' to nothing as well
 
 	pad->m_buffer_cleared = true;
-	pad->m_analog_left_x = pad->m_analog_left_y = pad->m_analog_right_x = pad->m_analog_right_y = 128;
-
-	pad->m_digital_1 = pad->m_digital_2 = 0;
-	pad->m_press_right = pad->m_press_left = pad->m_press_up = pad->m_press_down = 0;
-	pad->m_press_triangle = pad->m_press_circle = pad->m_press_cross = pad->m_press_square = 0;
-	pad->m_press_L1 = pad->m_press_L2 = pad->m_press_R1 = pad->m_press_R2 = 0;
-
-	pad->m_sensor_x = DEFAULT_MOTION_X;
-	pad->m_sensor_y = DEFAULT_MOTION_Y;
-	pad->m_sensor_z = DEFAULT_MOTION_Z;
-	pad->m_sensor_g = DEFAULT_MOTION_G;
+	pad->data = {};
+	pad->data.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X] = 128;
+	pad->data.button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y] = 128;
+	pad->data.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X] = 128;
+	pad->data.button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y] = 128;
+	pad->data.button[CELL_PAD_BTN_OFFSET_SENSOR_X] = DEFAULT_MOTION_X;
+	pad->data.button[CELL_PAD_BTN_OFFSET_SENSOR_Y] = DEFAULT_MOTION_Y;
+	pad->data.button[CELL_PAD_BTN_OFFSET_SENSOR_Z] = DEFAULT_MOTION_Z;
+	pad->data.button[CELL_PAD_BTN_OFFSET_SENSOR_G] = DEFAULT_MOTION_G;
 }
 
 error_code cellPadClearBuf(u32 port_no)
@@ -340,7 +344,7 @@ error_code cellPadClearBuf(u32 port_no)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	clear_pad_buffer(pad);
@@ -364,6 +368,8 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 	const u32 setting = config.port_setting[port_no];
 	bool btnChanged = false;
 
+	CellPadData& output = pad->data;
+
 	if (rinfo.ignore_input || !is_input_allowed())
 	{
 		// Needed for Hotline Miami and Ninja Gaiden Sigma after dialogs were closed and buttons are still pressed.
@@ -381,14 +387,14 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 	}
 	else
 	{
-		const u16 d1Initial = pad->m_digital_1;
-		const u16 d2Initial = pad->m_digital_2;
+		const u16 d1_initial = output.button[CELL_PAD_BTN_OFFSET_DIGITAL1];
+		const u16 d2_initial = output.button[CELL_PAD_BTN_OFFSET_DIGITAL2];
 
 		// Check if this pad is configured as a skateboard which ignores sticks and pressure button values.
 		// Curiously it maps infrared on the press value of the face buttons for some reason.
 		const bool use_piggyback = pad->m_class_type == CELL_PAD_PCLASS_TYPE_SKATEBOARD;
 
-		const auto set_value = [&btnChanged, use_piggyback, &pad](u16& value, u16 new_value, bool force_processing = false, u16 old_max_value = 255, u16 new_max_value = 255)
+		const auto set_value = [&btnChanged, use_piggyback, &pad](auto& value, u16 new_value, bool force_processing = false, u16 old_max_value = 255, u16 new_max_value = 255)
 		{
 			if (use_piggyback)
 			{
@@ -412,7 +418,7 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 			}
 		};
 
-		for (Button& button : pad->m_buttons)
+		for (const Button& button : pad->m_buttons_external)
 		{
 			// here we check btns, and set pad accordingly,
 			// if something changed, set btnChanged
@@ -422,17 +428,17 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 			case CELL_PAD_BTN_OFFSET_DIGITAL1:
 			{
 				if (button.m_pressed)
-					pad->m_digital_1 |= button.m_outKeyCode;
+					output.button[CELL_PAD_BTN_OFFSET_DIGITAL1] |= button.m_outKeyCode;
 				else
-					pad->m_digital_1 &= ~button.m_outKeyCode;
+					output.button[CELL_PAD_BTN_OFFSET_DIGITAL1] &= ~button.m_outKeyCode;
 
 				switch (button.m_outKeyCode)
 				{
-				case CELL_PAD_CTRL_LEFT: set_value(pad->m_press_left, button.m_value); break;
-				case CELL_PAD_CTRL_DOWN: set_value(pad->m_press_down, button.m_value); break;
-				case CELL_PAD_CTRL_RIGHT: set_value(pad->m_press_right, button.m_value); break;
-				case CELL_PAD_CTRL_UP: set_value(pad->m_press_up, button.m_value); break;
-				// These arent pressure btns
+				case CELL_PAD_CTRL_LEFT:  set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_LEFT],  button.m_value); break;
+				case CELL_PAD_CTRL_DOWN:  set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_DOWN],  button.m_value); break;
+				case CELL_PAD_CTRL_RIGHT: set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_RIGHT], button.m_value); break;
+				case CELL_PAD_CTRL_UP:    set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_UP],    button.m_value); break;
+				// These aren't pressure btns
 				case CELL_PAD_CTRL_R3:
 				case CELL_PAD_CTRL_L3:
 				case CELL_PAD_CTRL_START:
@@ -444,20 +450,20 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 			case CELL_PAD_BTN_OFFSET_DIGITAL2:
 			{
 				if (button.m_pressed)
-					pad->m_digital_2 |= button.m_outKeyCode;
+					output.button[CELL_PAD_BTN_OFFSET_DIGITAL2] |= button.m_outKeyCode;
 				else
-					pad->m_digital_2 &= ~button.m_outKeyCode;
+					output.button[CELL_PAD_BTN_OFFSET_DIGITAL2] &= ~button.m_outKeyCode;
 
 				switch (button.m_outKeyCode)
 				{
-				case CELL_PAD_CTRL_SQUARE: set_value(pad->m_press_square, button.m_value); break;
-				case CELL_PAD_CTRL_CROSS: set_value(pad->m_press_cross, button.m_value); break;
-				case CELL_PAD_CTRL_CIRCLE: set_value(pad->m_press_circle, button.m_value); break;
-				case CELL_PAD_CTRL_TRIANGLE: set_value(pad->m_press_triangle, button.m_value); break;
-				case CELL_PAD_CTRL_R1: set_value(pad->m_press_R1, button.m_value); break;
-				case CELL_PAD_CTRL_L1: set_value(pad->m_press_L1, button.m_value); break;
-				case CELL_PAD_CTRL_R2: set_value(pad->m_press_R2, button.m_value); break;
-				case CELL_PAD_CTRL_L2: set_value(pad->m_press_L2, button.m_value); break;
+				case CELL_PAD_CTRL_SQUARE:   set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_SQUARE], button.m_value); break;
+				case CELL_PAD_CTRL_CROSS:    set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_CROSS], button.m_value); break;
+				case CELL_PAD_CTRL_CIRCLE:   set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_CIRCLE], button.m_value); break;
+				case CELL_PAD_CTRL_TRIANGLE: set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_TRIANGLE], button.m_value); break;
+				case CELL_PAD_CTRL_R1:       set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_R1], button.m_value); break;
+				case CELL_PAD_CTRL_L1:       set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_L1], button.m_value); break;
+				case CELL_PAD_CTRL_R2:       set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_R2], button.m_value); break;
+				case CELL_PAD_CTRL_L2:       set_value(output.button[CELL_PAD_BTN_OFFSET_PRESS_L2], button.m_value); break;
 				default: break;
 				}
 				break;
@@ -466,18 +472,22 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 			{
 				switch (button.m_outKeyCode)
 				{
-				case CELL_PAD_CTRL_PRESS_RIGHT:    set_value(pad->m_press_right,    button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_LEFT:     set_value(pad->m_press_left,     button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_UP:       set_value(pad->m_press_up,       button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_DOWN:     set_value(pad->m_press_down,     button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_TRIANGLE: set_value(pad->m_press_triangle, button.m_value, true, 255, 63); break; // Infrared on RIDE Skateboard
-				case CELL_PAD_CTRL_PRESS_CIRCLE:   set_value(pad->m_press_circle,   button.m_value, true, 255, 63); break; // Infrared on RIDE Skateboard
-				case CELL_PAD_CTRL_PRESS_CROSS:    set_value(pad->m_press_cross,    button.m_value, true, 255, 63); break; // Infrared on RIDE Skateboard
-				case CELL_PAD_CTRL_PRESS_SQUARE:   set_value(pad->m_press_square,   button.m_value, true, 255, 63); break; // Infrared on RIDE Skateboard
-				case CELL_PAD_CTRL_PRESS_L1:       set_value(pad->m_press_L1,       button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_R1:       set_value(pad->m_press_R1,       button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_L2:       set_value(pad->m_press_L2,       button.m_value, true); break;
-				case CELL_PAD_CTRL_PRESS_R2:       set_value(pad->m_press_R2,       button.m_value, true); break;
+				case CELL_PAD_CTRL_PRESS_RIGHT:
+				case CELL_PAD_CTRL_PRESS_LEFT:
+				case CELL_PAD_CTRL_PRESS_UP:
+				case CELL_PAD_CTRL_PRESS_DOWN:
+				case CELL_PAD_CTRL_PRESS_L1:
+				case CELL_PAD_CTRL_PRESS_R1:
+				case CELL_PAD_CTRL_PRESS_L2:
+				case CELL_PAD_CTRL_PRESS_R2:
+					set_value(output.button[button.m_outKeyCode], button.m_value, true);
+					break;
+				case CELL_PAD_CTRL_PRESS_TRIANGLE:
+				case CELL_PAD_CTRL_PRESS_CIRCLE:
+				case CELL_PAD_CTRL_PRESS_CROSS:
+				case CELL_PAD_CTRL_PRESS_SQUARE:
+					set_value(output.button[button.m_outKeyCode], button.m_value, true, 255, 63); // Infrared on RIDE Skateboard
+					break;
 				default: break;
 				}
 				break;
@@ -487,15 +497,18 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 			}
 		}
 
-		for (const AnalogStick& stick : pad->m_sticks)
+		for (const AnalogStick& stick : pad->m_sticks_external)
 		{
 			switch (stick.m_offset)
 			{
-			case CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X: set_value(pad->m_analog_left_x, stick.m_value); break;
-			case CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y: set_value(pad->m_analog_left_y, stick.m_value); break;
-			case CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X: set_value(pad->m_analog_right_x, stick.m_value); break;
-			case CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y: set_value(pad->m_analog_right_y, stick.m_value); break;
-			default: break;
+			case CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X:
+			case CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y:
+			case CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X:
+			case CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y:
+				set_value(output.button[stick.m_offset], stick.m_value);
+				break;
+			default:
+				break;
 			}
 		}
 
@@ -505,16 +518,19 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 			{
 				switch (sensor.m_offset)
 				{
-				case CELL_PAD_BTN_OFFSET_SENSOR_X: set_value(pad->m_sensor_x, sensor.m_value, true); break;
-				case CELL_PAD_BTN_OFFSET_SENSOR_Y: set_value(pad->m_sensor_y, sensor.m_value, true); break;
-				case CELL_PAD_BTN_OFFSET_SENSOR_Z: set_value(pad->m_sensor_z, sensor.m_value, true); break;
-				case CELL_PAD_BTN_OFFSET_SENSOR_G: set_value(pad->m_sensor_g, sensor.m_value, true); break;
-				default: break;
+				case CELL_PAD_BTN_OFFSET_SENSOR_X:
+				case CELL_PAD_BTN_OFFSET_SENSOR_Y:
+				case CELL_PAD_BTN_OFFSET_SENSOR_Z:
+				case CELL_PAD_BTN_OFFSET_SENSOR_G:
+					set_value(output.button[sensor.m_offset], sensor.m_value, true);
+					break;
+				default:
+					break;
 				}
 			}
 		}
 
-		if (d1Initial != pad->m_digital_1 || d2Initial != pad->m_digital_2)
+		if (d1_initial != output.button[CELL_PAD_BTN_OFFSET_DIGITAL1] || d2_initial != output.button[CELL_PAD_BTN_OFFSET_DIGITAL2])
 		{
 			btnChanged = true;
 		}
@@ -557,41 +573,25 @@ void pad_get_data(u32 port_no, CellPadData* data, bool get_periph_data = false)
 		// bits 15-8 reserved, 7-4 = 0x7, 3-0: data->len/2;
 		data->button[1] = (0x7 << 4) | std::min(data->len / 2, 15);
 
-		data->button[CELL_PAD_BTN_OFFSET_DIGITAL1] = pad->m_digital_1;
-		data->button[CELL_PAD_BTN_OFFSET_DIGITAL2] = pad->m_digital_2;
-		data->button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_X] = pad->m_analog_right_x;
-		data->button[CELL_PAD_BTN_OFFSET_ANALOG_RIGHT_Y] = pad->m_analog_right_y;
-		data->button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_X] = pad->m_analog_left_x;
-		data->button[CELL_PAD_BTN_OFFSET_ANALOG_LEFT_Y] = pad->m_analog_left_y;
-
 		if (setting & CELL_PAD_SETTING_PRESS_ON)
 		{
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_RIGHT] = pad->m_press_right;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_LEFT] = pad->m_press_left;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_UP] = pad->m_press_up;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_DOWN] = pad->m_press_down;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_TRIANGLE] = pad->m_press_triangle;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_CIRCLE] = pad->m_press_circle;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_CROSS] = pad->m_press_cross;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_SQUARE] = pad->m_press_square;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_L1] = pad->m_press_L1;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_R1] = pad->m_press_R1;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_L2] = pad->m_press_L2;
-			data->button[CELL_PAD_BTN_OFFSET_PRESS_R2] = pad->m_press_R2;
+			constexpr u32 copy_size = (static_cast<u32>(CELL_PAD_LEN_CHANGE_PRESS_ON) - static_cast<u32>(CELL_PAD_BTN_OFFSET_DIGITAL1)) * sizeof(u16);
+			std::memcpy(&data->button[CELL_PAD_BTN_OFFSET_DIGITAL1], &output.button[CELL_PAD_BTN_OFFSET_DIGITAL1], copy_size);
 		}
 		else
 		{
+			constexpr u32 copy_size = (static_cast<u32>(CELL_PAD_LEN_CHANGE_DEFAULT) - static_cast<u32>(CELL_PAD_BTN_OFFSET_DIGITAL1)) * sizeof(u16);
+			std::memcpy(&data->button[CELL_PAD_BTN_OFFSET_DIGITAL1], &output.button[CELL_PAD_BTN_OFFSET_DIGITAL1], copy_size);
+
 			// Clear area if setting is not used
-			constexpr u32 area_lengh = (CELL_PAD_LEN_CHANGE_PRESS_ON - CELL_PAD_LEN_CHANGE_DEFAULT) * sizeof(u16);
-			std::memset(&data->button[CELL_PAD_LEN_CHANGE_DEFAULT], 0, area_lengh);
+			constexpr u32 area_size = (CELL_PAD_LEN_CHANGE_PRESS_ON - CELL_PAD_LEN_CHANGE_DEFAULT) * sizeof(u16);
+			std::memset(&data->button[CELL_PAD_LEN_CHANGE_DEFAULT], 0, area_size);
 		}
 
 		if (data->len == CELL_PAD_LEN_CHANGE_SENSOR_ON)
 		{
-			data->button[CELL_PAD_BTN_OFFSET_SENSOR_X] = pad->m_sensor_x;
-			data->button[CELL_PAD_BTN_OFFSET_SENSOR_Y] = pad->m_sensor_y;
-			data->button[CELL_PAD_BTN_OFFSET_SENSOR_Z] = pad->m_sensor_z;
-			data->button[CELL_PAD_BTN_OFFSET_SENSOR_G] = pad->m_sensor_g;
+			constexpr u32 copy_size = (static_cast<u32>(CELL_PAD_LEN_CHANGE_SENSOR_ON) - static_cast<u32>(CELL_PAD_BTN_OFFSET_SENSOR_X)) * sizeof(u16);
+			std::memcpy(&data->button[CELL_PAD_BTN_OFFSET_SENSOR_X], &output.button[CELL_PAD_BTN_OFFSET_SENSOR_X], copy_size);
 		}
 	}
 
@@ -713,12 +713,12 @@ error_code cellPadGetData(u32 port_no, vm::ptr<CellPadData> data)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	pad_get_data(port_no, data.get_ptr());
 
-	if (g_cfg.io.debug_overlay && !g_cfg.video.overlay && port_no == 0)
+	if (g_cfg.io.pad_debug_overlay && !g_cfg.video.debug_overlay && port_no == 0)
 	{
 		show_debug_overlay(*data, *pad, config);
 	}
@@ -799,7 +799,7 @@ error_code cellPadPeriphGetData(u32 port_no, vm::ptr<CellPadPeriphData> data)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	pad_get_data(port_no, &data->cellpad_data, true);
@@ -831,10 +831,11 @@ error_code cellPadGetRawData(u32 port_no, vm::ptr<CellPadData> data)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	// ?
+	// std::memcpy(data.get_ptr(), &pad->data, sizeof(CellPadData));
 
 	return CELL_OK;
 }
@@ -844,7 +845,7 @@ error_code cellPadGetDataExtra(u32 port_no, vm::ptr<u32> device_type, vm::ptr<Ce
 	cellPad.trace("cellPadGetDataExtra(port_no=%d, device_type=*0x%x, data=*0x%x)", port_no, device_type, data);
 
 	// TODO: This is used just to get data from a BD/CEC remote,
-	// but if the port isnt a remote, device type is set to CELL_PAD_DEV_TYPE_STANDARD and just regular cellPadGetData is returned
+	// but if the port isn't a remote, device type is set to CELL_PAD_DEV_TYPE_STANDARD and just regular cellPadGetData is returned
 
 	if (auto err = cellPadGetData(port_no, data))
 	{
@@ -895,14 +896,14 @@ error_code cellPadSetActDirect(u32 port_no, vm::ptr<CellPadActParam> param)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	// TODO: find out if this is checked here or later or at all
 	if (!(pad->m_device_capability & CELL_PAD_CAPABILITY_ACTUATOR))
 		return CELL_PAD_ERROR_UNSUPPORTED_GAMEPAD;
 
-	handler->SetRumble(port_no, param->motor[1], param->motor[0] > 0);
+	handler->SetRumble(port_no, param->motor[1], param->motor[0]);
 
 	return CELL_OK;
 }
@@ -1022,7 +1023,7 @@ error_code cellPadGetCapabilityInfo(u32 port_no, vm::ptr<CellPadCapabilityInfo>
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	// Should return the same as device capability mask, psl1ght has it backwards in pad->h
@@ -1050,7 +1051,15 @@ error_code cellPadSetPortSetting(u32 port_no, u32 port_setting)
 	if (port_no >= CELL_PAD_MAX_PORT_NUM)
 		return CELL_OK;
 
-	config.port_setting[port_no] = port_setting;
+	if (port_setting & CELL_PAD_SETTING_PRESS_ON)
+		config.port_setting[port_no] |= CELL_PAD_SETTING_PRESS_ON;
+	else
+		config.port_setting[port_no] &= ~CELL_PAD_SETTING_PRESS_ON;
+
+	if (port_setting & CELL_PAD_SETTING_SENSOR_ON)
+		config.port_setting[port_no] |= CELL_PAD_SETTING_SENSOR_ON;
+	else
+		config.port_setting[port_no] &= ~CELL_PAD_SETTING_SENSOR_ON;
 
 	// can also return CELL_PAD_ERROR_UNSUPPORTED_GAMEPAD <- Update: seems to be just internal and ignored
 
@@ -1078,7 +1087,7 @@ error_code cellPadInfoPressMode(u32 port_no)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	return not_an_error((pad->m_device_capability & CELL_PAD_CAPABILITY_PRESS_MODE) ? 1 : 0);
@@ -1105,7 +1114,7 @@ error_code cellPadInfoSensorMode(u32 port_no)
 	const auto& pads = handler->GetPads();
 	const auto& pad = pads[port_no];
 
-	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+	if (pad->is_fake_pad || !config.is_reportedly_connected(port_no) || !pad->is_connected())
 		return not_an_error(CELL_PAD_ERROR_NO_DEVICE);
 
 	return not_an_error((pad->m_device_capability & CELL_PAD_CAPABILITY_SENSOR_MODE) ? 1 : 0);
@@ -1122,7 +1131,7 @@ error_code cellPadSetPressMode(u32 port_no, u32 mode)
 	if (!config.max_connect)
 		return CELL_PAD_ERROR_UNINITIALIZED;
 
-	if (port_no >= CELL_PAD_MAX_PORT_NUM)
+	if (port_no >= CELL_MAX_PADS || mode > 1)
 		return CELL_PAD_ERROR_INVALID_PARAMETER;
 
 	// CELL_PAD_ERROR_NO_DEVICE is not returned in this case.
@@ -1156,7 +1165,7 @@ error_code cellPadSetSensorMode(u32 port_no, u32 mode)
 	if (!config.max_connect)
 		return CELL_PAD_ERROR_UNINITIALIZED;
 
-	if (port_no >= CELL_MAX_PADS)
+	if (port_no >= CELL_MAX_PADS || mode > 1)
 		return CELL_PAD_ERROR_INVALID_PARAMETER;
 
 	// CELL_PAD_ERROR_NO_DEVICE is not returned in this case.
@@ -1241,7 +1250,7 @@ error_code cellPadLddGetPortNo(s32 handle)
 		return CELL_PAD_ERROR_UNINITIALIZED;
 
 	const auto handler = pad::get_pad_thread();
-	auto& pads = handler->GetPads();
+	const auto& pads = handler->GetPads();
 
 	if (handle < 0 || static_cast<u32>(handle) >= pads.size())
 		return CELL_PAD_ERROR_INVALID_PARAMETER;
