@@ -87,9 +87,9 @@ s32 lv2_socket_native::create_socket()
 	return CELL_OK;
 }
 
-void lv2_socket_native::set_socket(socket_type socket, lv2_socket_family family, lv2_socket_type type, lv2_ip_protocol protocol)
+void lv2_socket_native::set_socket(socket_type native_socket, lv2_socket_family family, lv2_socket_type type, lv2_ip_protocol protocol)
 {
-	this->socket   = socket;
+	this->native_socket = native_socket;
 	this->family   = family;
 	this->type     = type;
 	this->protocol = protocol;
@@ -115,12 +115,12 @@ std::tuple<bool, s32, shared_ptr<lv2_socket>, sys_net_sockaddr> lv2_socket_nativ
 		sys_net.error("Calling socket::accept() from a previously connected socket!");
 	}
 
-	socket_type native_socket = ::accept(socket, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen);
+	socket_type client_socket = ::accept(native_socket, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen);
 
-	if (native_socket != invalid_socket)
+	if (client_socket != invalid_socket)
 	{
 		auto newsock = make_single<lv2_socket_native>(family, type, protocol);
-		newsock->set_socket(native_socket, family, type, protocol);
+		newsock->set_socket(client_socket, family, type, protocol);
 
 		// Sockets inherit non blocking behaviour from their parent
 		newsock->so_nbio = so_nbio;
@@ -173,7 +173,7 @@ s32 lv2_socket_native::bind(const sys_net_sockaddr& addr)
 
 	sys_net.warning("[Native] Trying to bind %s:%d", native_addr.sin_addr, std::bit_cast<be_t<u16>, u16>(native_addr.sin_port));
 
-	if (::bind(socket, reinterpret_cast<struct sockaddr*>(&native_addr), native_addr_len) == 0)
+	if (::bind(native_socket, reinterpret_cast<struct sockaddr*>(&native_addr), native_addr_len) == 0)
 	{
 		// Only UPNP port forward binds to 0.0.0.0
 		if (saddr == 0)
@@ -182,7 +182,7 @@ s32 lv2_socket_native::bind(const sys_net_sockaddr& addr)
 			{
 				sockaddr_in client_addr;
 				socklen_t client_addr_size = sizeof(client_addr);
-				ensure(::getsockname(socket, reinterpret_cast<struct sockaddr*>(&client_addr), &client_addr_size) == 0);
+				ensure(::getsockname(native_socket, reinterpret_cast<struct sockaddr*>(&client_addr), &client_addr_size) == 0);
 				bound_port = std::bit_cast<u16, be_t<u16>>(client_addr.sin_port);
 			}
 			else
@@ -245,7 +245,7 @@ std::optional<s32> lv2_socket_native::connect(const sys_net_sockaddr& addr)
 		return -SYS_NET_EALREADY;
 	}
 
-	if (::connect(socket, reinterpret_cast<struct sockaddr*>(&native_addr), native_addr_len) == 0)
+	if (::connect(native_socket, reinterpret_cast<struct sockaddr*>(&native_addr), native_addr_len) == 0)
 	{
 		return CELL_OK;
 	}
@@ -272,7 +272,7 @@ std::optional<s32> lv2_socket_native::connect(const sys_net_sockaddr& addr)
 					{
 						int native_error;
 						::socklen_t size = sizeof(native_error);
-						if (::getsockopt(socket, SOL_SOCKET, SO_ERROR, reinterpret_cast<char*>(&native_error), &size) != 0 || size != sizeof(int))
+						if (::getsockopt(native_socket, SOL_SOCKET, SO_ERROR, reinterpret_cast<char*>(&native_error), &size) != 0 || size != sizeof(int))
 						{
 							so_error = 1;
 						}
@@ -304,7 +304,7 @@ s32 lv2_socket_native::connect_followup()
 {
 	int native_error;
 	::socklen_t size = sizeof(native_error);
-	if (::getsockopt(socket, SOL_SOCKET, SO_ERROR, reinterpret_cast<char*>(&native_error), &size) != 0 || size != sizeof(int))
+	if (::getsockopt(native_socket, SOL_SOCKET, SO_ERROR, reinterpret_cast<char*>(&native_error), &size) != 0 || size != sizeof(int))
 	{
 		return -1;
 	}
@@ -320,7 +320,7 @@ std::pair<s32, sys_net_sockaddr> lv2_socket_native::getpeername()
 	::sockaddr_storage native_addr;
 	::socklen_t native_addrlen = sizeof(native_addr);
 
-	if (::getpeername(socket, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen) == 0)
+	if (::getpeername(native_socket, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen) == 0)
 	{
 		ensure(native_addr.ss_family == AF_INET);
 
@@ -339,7 +339,7 @@ std::pair<s32, sys_net_sockaddr> lv2_socket_native::getsockname()
 	::sockaddr_storage native_addr;
 	::socklen_t native_addrlen = sizeof(native_addr);
 
-	if (::getsockname(socket, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen) == 0)
+	if (::getsockname(native_socket, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen) == 0)
 	{
 		ensure(native_addr.ss_family == AF_INET);
 
@@ -551,12 +551,14 @@ std::tuple<s32, lv2_socket::sockopt_data, u32> lv2_socket_native::getsockopt(s32
 		}
 		case SYS_NET_IP_TTLCHK:
 		{
-			sys_net.error("sys_net_bnet_getsockopt(IPPROTO_IP, SYS_NET_IP_TTLCHK): stubbed option");
+			out_val._int = min_ttl;
+			out_len = sizeof(s32);
 			return {CELL_OK, out_val, out_len};
 		}
 		case SYS_NET_IP_MAXTTL:
 		{
-			sys_net.error("sys_net_bnet_getsockopt(IPPROTO_IP, SYS_NET_IP_MAXTTL): stubbed option");
+			out_val._int = max_ttl;
+			out_len = sizeof(s32);
 			return {CELL_OK, out_val, out_len};
 		}
 		case SYS_NET_IP_DONTFRAG:
@@ -581,7 +583,7 @@ std::tuple<s32, lv2_socket::sockopt_data, u32> lv2_socket_native::getsockopt(s32
 		return {-SYS_NET_EINVAL, {}, {}};
 	}
 
-	if (::getsockopt(socket, native_level, native_opt, native_val.ch, &native_len) != 0)
+	if (::getsockopt(native_socket, native_level, native_opt, native_val.ch, &native_len) != 0)
 	{
 		return {-get_last_error(false), {}, {}};
 	}
@@ -834,13 +836,13 @@ s32 lv2_socket_native::setsockopt(s32 level, s32 optname, const std::vector<u8>&
 		}
 		case SYS_NET_IP_TTLCHK:
 		{
-			sys_net.error("sys_net_bnet_setsockopt(s=%d, IPPROTO_IP): Stubbed option (0x%x) (SYS_NET_IP_TTLCHK)", lv2_id, optname);
-			break;
+			min_ttl = native_int;
+			return {};
 		}
 		case SYS_NET_IP_MAXTTL:
 		{
-			sys_net.error("sys_net_bnet_setsockopt(s=%d, IPPROTO_IP): Stubbed option (0x%x) (SYS_NET_IP_MAXTTL)", lv2_id, optname);
-			break;
+			max_ttl = native_int;
+			return {};
 		}
 		case SYS_NET_IP_DONTFRAG:
 		{
@@ -864,7 +866,7 @@ s32 lv2_socket_native::setsockopt(s32 level, s32 optname, const std::vector<u8>&
 		return -SYS_NET_EINVAL;
 	}
 
-	if (::setsockopt(socket, native_level, native_opt, static_cast<const char*>(native_val), native_len) == 0)
+	if (::setsockopt(native_socket, native_level, native_opt, static_cast<const char*>(native_val), native_len) == 0)
 	{
 		return {};
 	}
@@ -876,7 +878,7 @@ s32 lv2_socket_native::listen(s32 backlog)
 {
 	std::lock_guard lock(mutex);
 
-	if (::listen(socket, backlog) == 0)
+	if (::listen(native_socket, backlog) == 0)
 	{
 		return CELL_OK;
 	}
@@ -910,7 +912,7 @@ std::optional<std::tuple<s32, std::vector<u8>, sys_net_sockaddr>> lv2_socket_nat
 	{
 		auto& nph         = g_fxo->get<named_thread<np::np_handler>>();
 		const auto packet = dnshook.get_dns_packet(lv2_id);
-		ensure(packet.size() < len);
+		ensure(packet.size() <= len);
 		memcpy(res_buf.data(), packet.data(), packet.size());
 		native_addr.ss_family                                             = AF_INET;
 		(reinterpret_cast<::sockaddr_in*>(&native_addr))->sin_port        = std::bit_cast<u16, be_t<u16>>(53); // htons(53)
@@ -930,7 +932,7 @@ std::optional<std::tuple<s32, std::vector<u8>, sys_net_sockaddr>> lv2_socket_nat
 		native_flags |= MSG_WAITALL;
 	}
 
-	auto native_result = ::recvfrom(socket, reinterpret_cast<char*>(res_buf.data()), len, native_flags, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen);
+	auto native_result = ::recvfrom(native_socket, reinterpret_cast<char*>(res_buf.data()), len, native_flags, reinterpret_cast<struct sockaddr*>(&native_addr), &native_addrlen);
 
 	if (native_result >= 0)
 	{
@@ -1021,7 +1023,7 @@ std::optional<s32> lv2_socket_native::sendto(s32 flags, const std::vector<u8>& b
 		}
 	}
 
-	native_result = ::sendto(socket, reinterpret_cast<const char*>(buf.data()), ::narrow<int>(buf.size()), native_flags, native_addr ? reinterpret_cast<struct sockaddr*>(&native_addr.value()) : nullptr, native_addr ? sizeof(sockaddr_in) : 0);
+	native_result = ::sendto(native_socket, reinterpret_cast<const char*>(buf.data()), ::narrow<int>(buf.size()), native_flags, native_addr ? reinterpret_cast<struct sockaddr*>(&native_addr.value()) : nullptr, native_addr ? sizeof(sockaddr_in) : 0);
 
 	if (native_result >= 0)
 	{
@@ -1069,18 +1071,20 @@ std::optional<s32> lv2_socket_native::sendmsg(s32 flags, const sys_net_msghdr& m
 		return {-SYS_NET_ECONNRESET};
 	}
 
+	std::vector<u8> buf_copy;
 	for (int i = 0; i < msg.msg_iovlen; i++)
 	{
 		auto iov_base = msg.msg_iov[i].iov_base;
 		const u32 len = msg.msg_iov[i].iov_len;
-		const std::vector<u8> buf_copy(vm::_ptr<const char>(iov_base.addr()), vm::_ptr<const char>(iov_base.addr()) + len);
+		const auto* src = vm::_ptr<const char>(iov_base.addr());
+		buf_copy.insert(buf_copy.end(), src, src + len);
+	}
 
-		native_result = ::send(socket, reinterpret_cast<const char*>(buf_copy.data()), ::narrow<int>(buf_copy.size()), native_flags);
+	native_result = ::send(native_socket, reinterpret_cast<const char*>(buf_copy.data()), ::narrow<int>(buf_copy.size()), native_flags);
 
-		if (native_result >= 0)
-		{
-			return {native_result};
-		}
+	if (native_result >= 0)
+	{
+		return {native_result};
 	}
 
 	result = get_last_error(!so_nbio && (flags & SYS_NET_MSG_DONTWAIT) == 0);
@@ -1096,15 +1100,9 @@ std::optional<s32> lv2_socket_native::sendmsg(s32 flags, const sys_net_msghdr& m
 void lv2_socket_native::close()
 {
 	std::lock_guard lock(mutex);
-	if (socket)
-	{
-#ifdef _WIN32
-		::closesocket(socket);
-#else
-		::close(socket);
-#endif
-		socket = {};
-	}
+
+	np::close_socket(native_socket);
+	native_socket = {};
 
 	if (auto dnshook = g_fxo->try_get<np::dnshook>())
 	{
@@ -1141,7 +1139,7 @@ s32 lv2_socket_native::shutdown(s32 how)
                                  SHUT_RDWR;
 #endif
 
-	if (::shutdown(socket, native_how) == 0)
+	if (::shutdown(native_socket, native_how) == 0)
 	{
 		return CELL_OK;
 	}
@@ -1149,21 +1147,21 @@ s32 lv2_socket_native::shutdown(s32 how)
 	return -get_last_error(false);
 }
 
-s32 lv2_socket_native::poll(sys_net_pollfd& sn_pfd, pollfd& native_pfd)
+void lv2_socket_native::poll(sys_net_pollfd& sn_pfd, pollfd& native_pfd)
 {
 	// Check for fake packet for dns interceptions
 	auto& dnshook = g_fxo->get<np::dnshook>();
 	if (sn_pfd.events & SYS_NET_POLLIN && dnshook.is_dns(sn_pfd.fd) && dnshook.is_dns_queue(sn_pfd.fd))
 	{
 		sn_pfd.revents |= SYS_NET_POLLIN;
-		return 1;
+		return;
 	}
 	if (sn_pfd.events & ~(SYS_NET_POLLIN | SYS_NET_POLLOUT | SYS_NET_POLLERR))
 	{
 		sys_net.warning("sys_net_bnet_poll(fd=%d): events=0x%x", sn_pfd.fd, sn_pfd.events);
 	}
 
-	native_pfd.fd = socket;
+	native_pfd.fd = native_socket;
 
 	if (sn_pfd.events & SYS_NET_POLLIN)
 	{
@@ -1173,13 +1171,11 @@ s32 lv2_socket_native::poll(sys_net_pollfd& sn_pfd, pollfd& native_pfd)
 	{
 		native_pfd.events |= POLLOUT;
 	}
-
-	return 0;
 }
 
 std::tuple<bool, bool, bool> lv2_socket_native::select(bs_t<lv2_socket::poll_t> selected, pollfd& native_pfd)
 {
-	native_pfd.fd = socket;
+	native_pfd.fd = native_socket;
 	if (selected & lv2_socket::poll_t::read)
 	{
 		native_pfd.events |= POLLIN;
@@ -1196,12 +1192,12 @@ void lv2_socket_native::set_default_buffers()
 {
 	// Those are the default PS3 values
 	u32 default_RCVBUF = (type == SYS_NET_SOCK_STREAM) ? 65535 : 9216;
-	if (::setsockopt(socket, SOL_SOCKET, SO_RCVBUF, reinterpret_cast<const char*>(&default_RCVBUF), sizeof(default_RCVBUF)) != 0)
+	if (::setsockopt(native_socket, SOL_SOCKET, SO_RCVBUF, reinterpret_cast<const char*>(&default_RCVBUF), sizeof(default_RCVBUF)) != 0)
 	{
 		sys_net.error("Error setting default SO_RCVBUF on sys_net_bnet_socket socket");
 	}
 	u32 default_SNDBUF = 131072;
-	if (::setsockopt(socket, SOL_SOCKET, SO_SNDBUF, reinterpret_cast<const char*>(&default_SNDBUF), sizeof(default_SNDBUF)) != 0)
+	if (::setsockopt(native_socket, SOL_SOCKET, SO_SNDBUF, reinterpret_cast<const char*>(&default_SNDBUF), sizeof(default_SNDBUF)) != 0)
 	{
 		sys_net.error("Error setting default SO_SNDBUF on sys_net_bnet_socket socket");
 	}
@@ -1212,12 +1208,7 @@ void lv2_socket_native::set_non_blocking()
 	// Set non-blocking
 	// This is done to avoid having threads stuck on blocking socket functions
 	// Blocking functions just put the thread to sleep and delegate the waking up to network_thread which polls the sockets
-#ifdef _WIN32
-	u_long _true = 1;
-	::ioctlsocket(socket, FIONBIO, &_true);
-#else
-	::fcntl(socket, F_SETFL, ::fcntl(socket, F_GETFL, 0) | O_NONBLOCK);
-#endif
+	np::set_socket_non_blocking(native_socket);
 }
 
 bool lv2_socket_native::is_socket_connected()
@@ -1232,7 +1223,7 @@ bool lv2_socket_native::is_socket_connected()
 	int listening = 0;
 	socklen_t len = sizeof(listening);
 
-	if (::getsockopt(socket, SOL_SOCKET, SO_ACCEPTCONN, reinterpret_cast<char*>(&listening), &len) == -1)
+	if (::getsockopt(native_socket, SOL_SOCKET, SO_ACCEPTCONN, reinterpret_cast<char*>(&listening), &len) == -1)
 	{
 		return false;
 	}
@@ -1243,16 +1234,16 @@ bool lv2_socket_native::is_socket_connected()
 		return false;
 	}
 
-	fd_set readfds, writefds;
-	struct timeval timeout{0, 0}; // Zero timeout
+	pollfd pfd{};
+	pfd.fd     = native_socket;
+	pfd.events = POLLIN | POLLOUT;
 
-	FD_ZERO(&readfds);
-	FD_ZERO(&writefds);
-	FD_SET(socket, &readfds);
-	FD_SET(socket, &writefds);
-
-	// Use select to check for readability and writability
-	const int result = ::select(1, &readfds, &writefds, NULL, &timeout);
+	// Use poll to check for readability and writability
+#ifdef _WIN32
+	const int result = WSAPoll(&pfd, 1, 0);
+#else
+	const int result = ::poll(&pfd, 1, 0);
+#endif
 
 	if (result < 0)
 	{
@@ -1261,5 +1252,5 @@ bool lv2_socket_native::is_socket_connected()
 	}
 
 	// Socket is connected if it's readable or writable
-	return FD_ISSET(socket, &readfds) || FD_ISSET(socket, &writefds);
+	return (pfd.revents & (POLLIN | POLLOUT)) != 0;
 }
