@@ -1,7 +1,6 @@
 #include "stdafx.h"
 
 #include "Utilities/Thread.h"
-#include "util/asm.hpp"
 #include "util/atomic.hpp"
 #include "lv2_socket_p2ps.h"
 #include "Emu/NP/np_helpers.h"
@@ -113,7 +112,6 @@ public:
 
 				// reply is late, increases rtt
 				auto& msg       = it->second;
-				const auto addr = msg.dst_addr.sin_addr.s_addr;
 				rtt_info rtt    = rtts[msg.sock_id];
 				// Only increases rtt once per loop(in case a big number of packets are sent at once)
 				if (!rtt_increased.count(msg.sock_id))
@@ -121,7 +119,7 @@ public:
 					rtt.num_retries += 1;
 					// Increases current rtt by 10%
 					rtt.rtt_time += (rtt.rtt_time / 10);
-					rtts[addr] = rtt;
+					rtts[msg.sock_id] = rtt;
 
 					rtt_increased.emplace(msg.sock_id);
 				}
@@ -146,7 +144,7 @@ public:
 						ensure(sock.get_type() == SYS_NET_SOCK_STREAM_P2P);
 						auto& sock_p2ps = reinterpret_cast<lv2_socket_p2ps&>(sock);
 
-						while (::sendto(sock_p2ps.get_socket(), reinterpret_cast<const char*>(msg.data.data()), ::size32(msg.data), 0, reinterpret_cast<const sockaddr*>(&msg.dst_addr), sizeof(msg.dst_addr)) == -1)
+						while (np::sendto_possibly_ipv6(sock_p2ps.get_socket(), reinterpret_cast<const char*>(msg.data.data()), ::size32(msg.data), &msg.dst_addr, 0) == -1)
 						{
 							const sys_net_error err = get_last_error(false);
 							// concurrency on the socket(from a sendto for example) can result in EAGAIN error in which case we try again
@@ -275,10 +273,10 @@ lv2_socket_p2ps::lv2_socket_p2ps(lv2_socket_family family, lv2_socket_type type,
 	sockopts[(static_cast<u64>(SYS_NET_SOL_SOCKET) << 32ull) | SYS_NET_SO_TYPE] = cache_type;
 }
 
-lv2_socket_p2ps::lv2_socket_p2ps(socket_type socket, u16 port, u16 vport, u32 op_addr, u16 op_port, u16 op_vport, u64 cur_seq, u64 data_beg_seq, s32 so_nbio)
+lv2_socket_p2ps::lv2_socket_p2ps(socket_type native_socket, u16 port, u16 vport, u32 op_addr, u16 op_port, u16 op_vport, u64 cur_seq, u64 data_beg_seq, s32 so_nbio)
 	: lv2_socket_p2p(SYS_NET_AF_INET, SYS_NET_SOCK_STREAM_P2P, SYS_NET_IPPROTO_IP)
 {
-	this->socket       = socket;
+	this->native_socket = native_socket;
 	this->port         = port;
 	this->vport        = vport;
 	this->op_addr      = op_addr;
@@ -467,7 +465,7 @@ bool lv2_socket_p2ps::handle_listening(p2ps_encapsulated_tcp* tcp_header, [[mayb
 		const u16 new_op_vport     = tcp_header->src_port;
 		const u64 new_cur_seq      = send_hdr.seq + 1;
 		const u64 new_data_beg_seq = send_hdr.ack;
-		auto sock_lv2              = make_shared<lv2_socket_p2ps>(socket, port, vport, new_op_addr, new_op_port, new_op_vport, new_cur_seq, new_data_beg_seq, so_nbio);
+		auto sock_lv2 = make_shared<lv2_socket_p2ps>(native_socket, port, vport, new_op_addr, new_op_port, new_op_vport, new_cur_seq, new_data_beg_seq, so_nbio);
 		const s32 new_sock_id      = idm::import_existing<lv2_socket>(sock_lv2);
 		sock_lv2->set_lv2_id(new_sock_id);
 		const u64 key_connected = (reinterpret_cast<struct sockaddr_in*>(op_addr)->sin_addr.s_addr) | (static_cast<u64>(tcp_header->src_port) << 48) | (static_cast<u64>(tcp_header->dst_port) << 32);
@@ -518,8 +516,9 @@ void lv2_socket_p2ps::send_u2s_packet(std::vector<u8> data, const ::sockaddr_in*
 {
 	char ip_str[16];
 	inet_ntop(AF_INET, &dst->sin_addr, ip_str, sizeof(ip_str));
-	sys_net.trace("[P2PS] Sending U2S packet on socket %d(id:%d): data(%d, seq %d, require_ack %d) to %s:%d", socket, lv2_id, data.size(), seq, require_ack, ip_str, std::bit_cast<u16, be_t<u16>>(dst->sin_port));
-	while (::sendto(socket, reinterpret_cast<char*>(data.data()), ::size32(data), 0, reinterpret_cast<const sockaddr*>(dst), sizeof(sockaddr_in)) == -1)
+	sys_net.trace("[P2PS] Sending U2S packet on socket %d(id:%d): data(%d, seq %d, require_ack %d) to %s:%d", native_socket, lv2_id, data.size(), seq, require_ack, ip_str, std::bit_cast<u16, be_t<u16>>(dst->sin_port));
+
+	while (np::sendto_possibly_ipv6(native_socket, reinterpret_cast<char*>(data.data()), ::size32(data), dst, 0) == -1)
 	{
 		const sys_net_error err = get_last_error(false);
 		// concurrency on the socket can result in EAGAIN error in which case we try again
@@ -625,7 +624,7 @@ std::tuple<bool, s32, shared_ptr<lv2_socket>, sys_net_sockaddr> lv2_socket_p2ps:
 	sys_net_sockaddr ps3_addr{};
 	auto* paddr = reinterpret_cast<sys_net_sockaddr_in_p2p*>(&ps3_addr);
 
-	lv2_socket_p2ps* sock_client = reinterpret_cast<lv2_socket_p2ps*>(idm::check_unlocked<lv2_socket>(p2ps_client));
+	auto sock_client = static_cast<shared_ptr<lv2_socket_p2ps>>(idm::get_unlocked<lv2_socket>(p2ps_client));
 	{
 		std::lock_guard lock(sock_client->mutex);
 		paddr->sin_family = SYS_NET_AF_INET;
@@ -707,7 +706,7 @@ s32 lv2_socket_p2ps::bind(const sys_net_sockaddr& addr)
 
 			port       = p2p_port;
 			vport      = p2p_vport;
-			socket     = real_socket;
+			native_socket = real_socket;
 			bound_addr = psa_in_p2p->sin_addr;
 		}
 	}
@@ -720,7 +719,7 @@ std::pair<s32, sys_net_sockaddr> lv2_socket_p2ps::getsockname()
 	std::lock_guard lock(mutex);
 
 	// Unbound socket
-	if (!socket)
+	if (!native_socket)
 	{
 		return {CELL_OK, {}};
 	}
@@ -783,7 +782,7 @@ std::optional<s32> lv2_socket_p2ps::connect(const sys_net_sockaddr& addr)
 		}
 	}
 
-	socket = real_socket;
+	native_socket = real_socket;
 
 	send_hdr.src_port = vport;
 	send_hdr.dst_port = dst_vport;
@@ -986,7 +985,7 @@ s32 lv2_socket_p2ps::shutdown([[maybe_unused]] s32 how)
 	return CELL_OK;
 }
 
-s32 lv2_socket_p2ps::poll(sys_net_pollfd& sn_pfd, [[maybe_unused]] pollfd& native_pfd)
+void lv2_socket_p2ps::poll(sys_net_pollfd& sn_pfd, [[maybe_unused]] pollfd& native_pfd)
 {
 	std::lock_guard lock(mutex);
 	sys_net.trace("[P2PS] poll checking for 0x%X", sn_pfd.events);
@@ -1003,14 +1002,7 @@ s32 lv2_socket_p2ps::poll(sys_net_pollfd& sn_pfd, [[maybe_unused]] pollfd& nativ
 		{
 			sn_pfd.revents |= SYS_NET_POLLOUT;
 		}
-
-		if (sn_pfd.revents)
-		{
-			return 1;
-		}
 	}
-
-	return 0;
 }
 
 std::tuple<bool, bool, bool> lv2_socket_p2ps::select(bs_t<lv2_socket::poll_t> selected, [[maybe_unused]] pollfd& native_pfd)
