@@ -1,6 +1,5 @@
 #include "stdafx.h"
 #include "PadHandler.h"
-#include "Emu/system_utils.hpp"
 #include "Emu/system_config.h"
 #include "Emu/Cell/timers.hpp"
 #include "Input/pad_thread.h"
@@ -41,7 +40,7 @@ f32 PadHandlerBase::ScaledInput(f32 raw_value, f32 minimum, f32 maximum, f32 dea
 	}
 
 	// convert [min, max] to [0, 1]
-	const f32 val = static_cast<f32>(std::clamp(raw_value, minimum, maximum) - minimum) / (maximum - minimum);
+	const f32 val = static_cast<f32>(std::max(minimum, std::min(raw_value, maximum)) - minimum) / (maximum - minimum);
 
 	// convert [0, 1] to [0, range]
 	return range * val;
@@ -51,7 +50,7 @@ f32 PadHandlerBase::ScaledInput(f32 raw_value, f32 minimum, f32 maximum, f32 dea
 f32 PadHandlerBase::ScaledAxisInput(f32 raw_value, f32 minimum, f32 maximum, f32 deadzone, f32 range)
 {
 	// convert [min, max] to [0, 1]
-	f32 val = static_cast<f32>(std::clamp(raw_value, minimum, maximum) - minimum) / (maximum - minimum);
+	f32 val = static_cast<f32>(std::max(minimum, std::min(raw_value, maximum)) - minimum) / (maximum - minimum);
 
 	if (deadzone > 0)
 	{
@@ -171,7 +170,7 @@ u16 PadHandlerBase::ConvertAxis(f32 value)
 
 // The DS3, (and i think xbox controllers) give a 'square-ish' type response, so that the corners will give (almost)max x/y instead of the ~30x30 from a perfect circle
 // using a simple scale/sensitivity increase would *work* although it eats a chunk of our usable range in exchange
-// this might be the best for now, in practice it seems to push the corners to max of 20x20, with a squircle_factor of 8000
+// this might be the best for now, in practice it seems to push the corners to max of 20x20, with a squircle_factor of ~4000
 // This function assumes inX and inY is already in 0-255
 void PadHandlerBase::ConvertToSquirclePoint(u16& inX, u16& inY, u32 squircle_factor)
 {
@@ -201,10 +200,29 @@ void PadHandlerBase::init_configs()
 {
 	for (u32 i = 0; i < MAX_GAMEPADS; i++)
 	{
+		// We need to restore the original defaults first.
+		m_pad_configs[i].restore_defaults();
+
+		// Set and apply actual defaults depending on pad handler
 		init_config(&m_pad_configs[i]);
 	}
 }
 
+pad_capabilities PadHandlerBase::get_capabilities(const std::string& /*pad_id*/)
+{
+	return pad_capabilities
+	{
+		.has_led = b_has_rgb,
+		.has_mono_led = b_has_led,
+		.has_player_led = b_has_player_led,
+		.has_battery_led = b_has_battery_led,
+		.has_rumble = b_has_rumble,
+		.has_accel = b_has_motion,
+		.has_gyro = b_has_motion,
+		.has_pressure_intensity_button = b_has_pressure_intensity_button
+	};
+}
+
 cfg_pad* PadHandlerBase::get_config(const std::string& pad_id)
 {
 	int index = 0;
@@ -328,12 +346,13 @@ PadHandlerBase::connection PadHandlerBase::get_next_button_press(const std::stri
 	if (callback)
 	{
 		pad_preview_values preview_values = get_preview_values(data);
+		pad_capabilities capabilities = get_capabilities(pad_id);
 		const u32 battery_level = get_battery_level(pad_id);
 
 		if (pressed_button.value > 0)
-			callback(pressed_button.value, pressed_button.name, pad_id, battery_level, std::move(preview_values));
+			callback(pressed_button.value, pressed_button.name, pad_id, battery_level, std::move(preview_values), std::move(capabilities));
 		else
-			callback(0, "", pad_id, battery_level, std::move(preview_values));
+			callback(0, "", pad_id, battery_level, std::move(preview_values), std::move(capabilities));
 	}
 
 	return status;
@@ -539,8 +558,8 @@ bool PadHandlerBase::bindPadToDevice(std::shared_ptr<Pad> pad)
 	pad->m_sensors[2] = AnalogSensor(CELL_PAD_BTN_OFFSET_SENSOR_Z, 0, 0, 0, DEFAULT_MOTION_Z);
 	pad->m_sensors[3] = AnalogSensor(CELL_PAD_BTN_OFFSET_SENSOR_G, 0, 0, 0, DEFAULT_MOTION_G);
 
-	pad->m_vibrateMotors[0] = VibrateMotor(true, 0);
-	pad->m_vibrateMotors[1] = VibrateMotor(false, 0);
+	pad->m_vibrate_motors[0] = VibrateMotor(true);
+	pad->m_vibrate_motors[1] = VibrateMotor(false);
 
 	m_bindings.emplace_back(pad, pad_device, nullptr);
 
@@ -753,6 +772,23 @@ void PadHandlerBase::process()
 
 		pad->move_data.orientation_enabled = b_has_orientation && device->config && device->config->orientation_enabled.get();
 
+		// Disable pad vibration if no new data was sent for 3 seconds
+		if (pad->m_last_rumble_time_us > 0)
+		{
+			std::lock_guard lock(pad::g_pad_mutex);
+
+			if ((get_system_time() - pad->m_last_rumble_time_us) > 3'000'000)
+			{
+				for (VibrateMotor& motor : pad->m_vibrate_motors)
+				{
+					motor.value = 0;
+					motor.adjusted_value = 0;
+				}
+
+				pad->m_last_rumble_time_us = 0;
+			}
+		}
+
 		const connection status = update_connection(device);
 
 		switch (status)
@@ -838,12 +874,12 @@ void PadHandlerBase::set_raw_orientation(ps_move_data& move_data, f32 accel_x, f
 	// The default position is flat on the ground, pointing forward.
 	// The accelerometers constantly measure G forces.
 	// The gyros measure changes in orientation and will reset when the device isn't moved anymore.
-	move_data.accelerometer_x = -accel_x;      // move_data: Increases if the device is rolled to the left
-	move_data.accelerometer_y = accel_z;       // move_data: Increases if the device is pitched upwards
-	move_data.accelerometer_z = accel_y;       // move_data: Increases if the device is moved upwards
-	move_data.gyro_x = degree_to_rad(-gyro_x); // move_data: Increases if the device is pitched upwards
-	move_data.gyro_y = degree_to_rad(gyro_z);  // move_data: Increases if the device is rolled to the right
-	move_data.gyro_z = degree_to_rad(-gyro_y); // move_data: Increases if the device is yawed to the left
+	move_data.accelerometer.x() = -accel_x;      // move_data: Increases if the device is rolled to the left
+	move_data.accelerometer.y() = accel_z;       // move_data: Increases if the device is pitched upwards
+	move_data.accelerometer.z() = accel_y;       // move_data: Increases if the device is moved upwards
+	move_data.gyro.x() = degree_to_rad(-gyro_x); // move_data: Increases if the device is pitched upwards
+	move_data.gyro.y() = degree_to_rad(gyro_z);  // move_data: Increases if the device is rolled to the right
+	move_data.gyro.z() = degree_to_rad(-gyro_y); // move_data: Increases if the device is yawed to the left
 }
 
 void PadHandlerBase::set_raw_orientation(Pad& pad)
@@ -914,7 +950,7 @@ void PadDevice::update_orientation(ps_move_data& move_data)
 
 	// Get elapsed time since last update
 	const u64 now_us = get_system_time();
-	const float elapsed_sec = (last_ahrs_update_time_us == 0) ? 0.0f : ((now_us - last_ahrs_update_time_us) / 1'000'000.0f);
+	const f32 elapsed_sec = (last_ahrs_update_time_us == 0) ? 0.0f : ((now_us - last_ahrs_update_time_us) / 1'000'000.0f);
 	last_ahrs_update_time_us = now_us;
 
 	// The ps move handler's axis may differ from the Fusion axis, so we have to map them correctly.
@@ -923,17 +959,17 @@ void PadDevice::update_orientation(ps_move_data& move_data)
 
 	const FusionVector accelerometer{
 		.axis {
-			.x = -move_data.accelerometer_x,
-			.y = +move_data.accelerometer_y,
-			.z = +move_data.accelerometer_z
+			.x = -move_data.accelerometer.x(),
+			.y = +move_data.accelerometer.y(),
+			.z = +move_data.accelerometer.z()
 		}
 	};
 
 	const FusionVector gyroscope{
 		.axis {
-			.x = +PadHandlerBase::rad_to_degree(move_data.gyro_x),
-			.y = +PadHandlerBase::rad_to_degree(move_data.gyro_z),
-			.z = -PadHandlerBase::rad_to_degree(move_data.gyro_y)
+			.x = +PadHandlerBase::rad_to_degree(move_data.gyro.x()),
+			.y = +PadHandlerBase::rad_to_degree(move_data.gyro.z()),
+			.z = -PadHandlerBase::rad_to_degree(move_data.gyro.y())
 		}
 	};
 
@@ -943,9 +979,9 @@ void PadDevice::update_orientation(ps_move_data& move_data)
 	{
 		magnetometer = FusionVector{
 			.axis {
-				.x = move_data.magnetometer_x,
-				.y = move_data.magnetometer_y,
-				.z = move_data.magnetometer_z
+				.x = move_data.magnetometer.x(),
+				.y = move_data.magnetometer.y(),
+				.z = move_data.magnetometer.z()
 			}
 		};
 	}
@@ -959,4 +995,5 @@ void PadDevice::update_orientation(ps_move_data& move_data)
 	move_data.quaternion[1] = quaternion.array[2];
 	move_data.quaternion[2] = quaternion.array[3];
 	move_data.quaternion[3] = quaternion.array[0];
+	move_data.update_orientation(elapsed_sec);
 }
