@@ -6,9 +6,6 @@
 #include "util/tsc.hpp"
 #include <functional>
 
-extern bool g_use_rtm;
-extern u64 g_rtm_tx_limit2;
-
 #ifdef _MSC_VER
 extern "C"
 {
@@ -35,114 +32,76 @@ namespace vm
 
 	// Update reservation status
 	void reservation_update(u32 addr);
+	std::pair<bool, u64> try_reservation_update(u32 addr);
 
-	struct reservation_waiter_t
+	struct alignas(8) reservation_waiter_t
 	{
 		u32 wait_flag = 0;
-		u8 waiters_count = 0;
-		u8 waiters_index = 0;
+		u32 waiters_count = 0;
 	};
 
-	static inline std::pair<atomic_t<reservation_waiter_t>*, atomic_t<reservation_waiter_t>*> reservation_notifier(u32 raddr)
+	static inline atomic_t<reservation_waiter_t, 128>* reservation_notifier(u32 raddr, u64 rtime)
 	{
-		extern std::array<atomic_t<reservation_waiter_t>, 1024> g_resrv_waiters_count;
+		constexpr u32 wait_vars_for_each = 32;
+		constexpr u32 unique_address_bit_mask = 0b1111;
+		constexpr u32 unique_rtime_bit_mask = 0b1;
+
+		extern std::array<atomic_t<reservation_waiter_t, 128>, wait_vars_for_each * (unique_address_bit_mask + 1) * (unique_rtime_bit_mask + 1)> g_resrv_waiters_count;
 
 		// Storage efficient method to distinguish different nearby addresses (which are likely)
-		constexpr u32 wait_vars_for_each = 8;
-		constexpr u32 unique_address_bit_mask = 0b11;
-		const usz index = std::popcount(raddr & -1024) + ((raddr / 128) & unique_address_bit_mask) * 32;
-		auto& waiter = g_resrv_waiters_count[index * wait_vars_for_each];
-		return { &g_resrv_waiters_count[index * wait_vars_for_each + waiter.load().waiters_index % wait_vars_for_each], &waiter };
-	}
-
-	// Returns waiter count and index
-	static inline std::pair<u32, u32> reservation_notifier_count_index(u32 raddr)
-	{
-		const auto notifiers = reservation_notifier(raddr);
-		return { notifiers.first->load().waiters_count, static_cast<u32>(notifiers.first - notifiers.second) };
+		const usz index = std::min<usz>(std::popcount(raddr & -2048), 31) * (1 << 5) + ((rtime / 128) & unique_rtime_bit_mask) * (1 << 4) + ((raddr / 128) & unique_address_bit_mask);
+		return &g_resrv_waiters_count[index];
 	}
 
 	// Returns waiter count
-	static inline u32 reservation_notifier_count(u32 raddr)
+	static inline u32 reservation_notifier_count(u32 raddr, u64 rtime)
 	{
-		return reservation_notifier(raddr).first->load().waiters_count;
+		reservation_waiter_t v = reservation_notifier(raddr, rtime)->load();
+		return v.wait_flag % 2 == 1 ? v.waiters_count : 0;
 	}
 
-	static inline void reservation_notifier_end_wait(atomic_t<reservation_waiter_t>& waiter)
+	static inline void reservation_notifier_end_wait(atomic_t<reservation_waiter_t, 128>& waiter)
 	{
 		waiter.atomic_op([](reservation_waiter_t& value)
 		{
-			if (value.waiters_count-- == 1)
+			if (value.waiters_count == 1 && value.wait_flag % 2 == 1)
 			{
-				value.wait_flag = 0;
+				// Make wait_flag even (disabling notification on last waiter)
+				value.wait_flag++;
 			}
+
+			value.waiters_count--;
 		});
 	}
 
-	static inline atomic_t<reservation_waiter_t>* reservation_notifier_begin_wait(u32 raddr, u64 rtime)
+	static inline std::pair<atomic_t<reservation_waiter_t, 128>*, u32> reservation_notifier_begin_wait(u32 raddr, u64 rtime)
 	{
-		atomic_t<reservation_waiter_t>& waiter = *reservation_notifier(raddr).first;
+		atomic_t<reservation_waiter_t, 128>& waiter = *reservation_notifier(raddr, rtime);
 
-		waiter.atomic_op([](reservation_waiter_t& value)
+		u32 wait_flag = 0;
+
+		waiter.atomic_op([&](reservation_waiter_t& value)
 		{
-			value.wait_flag = 1;
+			if (value.wait_flag % 2 == 0)
+			{
+				// Make wait_flag odd (for notification deduplication detection)
+				value.wait_flag++;
+			}
+
+			wait_flag = value.wait_flag;
 			value.waiters_count++;
 		});
 
 		if ((reservation_acquire(raddr) & -128) != rtime)
 		{
 			reservation_notifier_end_wait(waiter);
-			return nullptr;
+			return {};
 		}
 
-		return &waiter;
+		return { &waiter, wait_flag };
 	}
 
-	static inline atomic_t<u32>* reservation_notifier_notify(u32 raddr, bool pospone = false)
-	{
-		const auto notifiers = reservation_notifier(raddr);
-
-		if (notifiers.first->load().wait_flag)
-		{
-			if (notifiers.first == notifiers.second)
-			{
-				if (!notifiers.first->fetch_op([](reservation_waiter_t& value)
-				{
-					if (value.waiters_index == 0)
-					{
-						value.wait_flag = 0;
-						value.waiters_count = 0;
-						value.waiters_index++;
-						return true;
-					}
-
-					return false;
-				}).second)
-				{
-					return nullptr;
-				}
-			}
-			else
-			{
-				u8 old_index = static_cast<u8>(notifiers.first - notifiers.second);
-				if (!atomic_storage<u8>::compare_exchange(notifiers.second->raw().waiters_index, old_index, (old_index + 1) % 4))
-				{
-					return nullptr;
-				}
-
-				notifiers.first->release(reservation_waiter_t{});
-			}
-
-			if (pospone)
-			{
-				return utils::bless<atomic_t<u32>>(&notifiers.first->raw().wait_flag);
-			}
-
-			utils::bless<atomic_t<u32>>(&notifiers.first->raw().wait_flag)->notify_all();
-		}
-
-		return nullptr;
-	}
+	atomic_t<u32>* reservation_notifier_notify(u32 raddr, u64 rtime, bool postpone = false);
 
 	u64 reservation_lock_internal(u32, atomic_t<u64>&);
 
@@ -182,7 +141,7 @@ namespace vm
 	void reservation_op_internal(u32 addr, std::function<bool()> func);
 
 	template <bool Ack = false, typename CPU, typename T, typename AT = u32, typename F>
-	inline SAFE_BUFFERS(auto) reservation_op(CPU& cpu, _ptr_base<T, AT> ptr, F op)
+	inline SAFE_BUFFERS(auto) reservation_op(CPU& /*cpu*/, _ptr_base<T, AT> ptr, F op)
 	{
 		// Atomic operation will be performed on aligned 128 bytes of data, so the data size and alignment must comply
 		static_assert(sizeof(T) <= 128 && alignof(T) == sizeof(T), "vm::reservation_op: unsupported type");
@@ -201,201 +160,21 @@ namespace vm
 		auto& res = vm::reservation_acquire(addr);
 		//_m_prefetchw(&res);
 
-#if defined(ARCH_X64)
-		if (g_use_rtm)
-		{
-			// Stage 1: single optimistic transaction attempt
-			unsigned status = -1;
-			u64 _old = 0;
-
-			auto stamp0 = utils::get_tsc(), stamp1 = stamp0, stamp2 = stamp0;
-
-#ifndef _MSC_VER
-			__asm__ goto ("xbegin %l[stage2];" ::: "memory" : stage2);
-#else
-			status = _xbegin();
-			if (status == umax)
-#endif
-			{
-				if (res & rsrv_unique_lock)
-				{
-#ifndef _MSC_VER
-					__asm__ volatile ("xend; mov $-1, %%eax;" ::: "memory");
-#else
-					_xend();
-#endif
-					goto stage2;
-				}
-
-				if constexpr (std::is_void_v<std::invoke_result_t<F, T&>>)
-				{
-					std::invoke(op, *sptr);
-					res += 128;
-#ifndef _MSC_VER
-					__asm__ volatile ("xend;" ::: "memory");
-#else
-					_xend();
-#endif
-					if constexpr (Ack)
-						res.notify_all();
-					return;
-				}
-				else
-				{
-					if (auto result = std::invoke(op, *sptr))
-					{
-						res += 128;
-#ifndef _MSC_VER
-						__asm__ volatile ("xend;" ::: "memory");
-#else
-						_xend();
-#endif
-						if constexpr (Ack)
-							res.notify_all();
-						return result;
-					}
-					else
-					{
-#ifndef _MSC_VER
-						__asm__ volatile ("xend;" ::: "memory");
-#else
-						_xend();
-#endif
-						return result;
-					}
-				}
-			}
-
-			stage2:
-#ifndef _MSC_VER
-			__asm__ volatile ("mov %%eax, %0;" : "=r" (status) :: "memory");
-#endif
-			stamp1 = utils::get_tsc();
-
-			// Stage 2: try to lock reservation first
-			_old = res.fetch_add(1);
-
-			// Compute stamps excluding memory touch
-			stamp2 = utils::get_tsc() - (stamp1 - stamp0);
-
-			// Start lightened transaction
-			for (; !(_old & vm::rsrv_unique_lock) && stamp2 - stamp0 <= g_rtm_tx_limit2; stamp2 = utils::get_tsc())
-			{
-				if (cpu.has_pause_flag())
-				{
-					break;
-				}
-
-#ifndef _MSC_VER
-				__asm__ goto ("xbegin %l[retry];" ::: "memory" : retry);
-#else
-				status = _xbegin();
-
-				if (status != umax) [[unlikely]]
-				{
-					goto retry;
-				}
-#endif
-				if constexpr (std::is_void_v<std::invoke_result_t<F, T&>>)
-				{
-					std::invoke(op, *sptr);
-#ifndef _MSC_VER
-					__asm__ volatile ("xend;" ::: "memory");
-#else
-					_xend();
-#endif
-					res += 127;
-					if (Ack)
-						res.notify_all();
-					return;
-				}
-				else
-				{
-					if (auto result = std::invoke(op, *sptr))
-					{
-#ifndef _MSC_VER
-						__asm__ volatile ("xend;" ::: "memory");
-#else
-						_xend();
-#endif
-						res += 127;
-						if (Ack)
-							res.notify_all();
-						return result;
-					}
-					else
-					{
-#ifndef _MSC_VER
-						__asm__ volatile ("xend;" ::: "memory");
-#else
-						_xend();
-#endif
-						return result;
-					}
-				}
-
-				retry:
-#ifndef _MSC_VER
-				__asm__ volatile ("mov %%eax, %0;" : "=r" (status) :: "memory");
-#endif
-
-				if (!status)
-				{
-					break;
-				}
-			}
-
-			// Stage 3: all failed, heavyweight fallback (see comments at the bottom)
-			if constexpr (std::is_void_v<std::invoke_result_t<F, T&>>)
-			{
-				vm::reservation_op_internal(addr, [&]
-				{
-					std::invoke(op, *sptr);
-					return true;
-				});
-
-				if constexpr (Ack)
-					res.notify_all();
-				return;
-			}
-			else
-			{
-				auto result = std::invoke_result_t<F, T&>();
-
-				vm::reservation_op_internal(addr, [&]
-				{
-					if ((result = std::invoke(op, *sptr)))
-					{
-						return true;
-					}
-					else
-					{
-						return false;
-					}
-				});
-
-				if (Ack && result)
-					res.notify_all();
-				return result;
-			}
-		}
-#else
-		static_cast<void>(cpu);
-#endif /* ARCH_X64 */
-
 		// Lock reservation and perform heavyweight lock
 		reservation_shared_lock_internal(res);
 
+		u64 old_time = umax;
+
 		if constexpr (std::is_void_v<std::invoke_result_t<F, T&>>)
 		{
 			{
 				vm::writer_lock lock(addr);
 				std::invoke(op, *sptr);
-				res += 127;
+				old_time = res.fetch_add(127);
 			}
 
 			if constexpr (Ack)
-				res.notify_all();
+				reservation_notifier_notify(addr, old_time);
 			return;
 		}
 		else
@@ -406,16 +185,16 @@ namespace vm
 
 				if ((result = std::invoke(op, *sptr)))
 				{
-					res += 127;
+					old_time = res.fetch_add(127);
 				}
 				else
 				{
-					res -= 1;
+					old_time = res.fetch_sub(1);
 				}
 			}
 
 			if (Ack && result)
-				res.notify_all();
+				reservation_notifier_notify(addr, old_time);
 			return result;
 		}
 	}
