@@ -1,17 +1,18 @@
 #include "stdafx.h"
 #include "Emu/system_config.h"
 #include "Emu/NP/np_handler.h"
-#include "Emu/Cell/PPUModule.h"
+#include "Emu/Cell/PPUCallback.h"
 #include "Emu/Cell/Modules/sceNp.h"
 #include "Emu/Cell/Modules/sceNp2.h"
 #include "Emu/Cell/Modules/cellNetCtl.h"
+#include "Emu/Cell/timers.hpp"
 #include "Utilities/StrUtil.h"
 #include "Emu/IdManager.h"
-#include "Emu/NP/np_structs_extra.h"
 #include "Emu/System.h"
 #include "Emu/NP/rpcn_config.h"
 #include "Emu/NP/np_contexts.h"
 #include "Emu/NP/np_helpers.h"
+#include "Emu/NP/signaling_handler.h"
 #include "Emu/RSX/Overlays/overlay_message.h"
 #include "Emu/Cell/lv2/sys_net/network_context.h"
 #include "Emu/Cell/lv2/sys_net/sys_net_helpers.h"
@@ -22,8 +23,8 @@
 #include <iphlpapi.h>
 #else
 #ifdef __clang__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wold-style-cast"
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wold-style-cast"
 #endif
 #include <sys/socket.h>
 #include <sys/ioctl.h>
@@ -33,7 +34,7 @@
 #include <netdb.h>
 #include <unistd.h>
 #ifdef __clang__
-#pragma GCC diagnostic pop
+#pragma clang diagnostic pop
 #endif
 #endif
 
@@ -42,7 +43,6 @@
 #include <net/if_dl.h>
 #endif
 
-#include "util/asm.hpp"
 #include "util/yaml.hpp"
 
 #include <span>
@@ -111,7 +111,7 @@ namespace np
 	}
 
 	ticket::ticket(std::vector<u8>&& raw_data)
-		: raw_data(raw_data)
+		: raw_data(std::move(raw_data))
 	{
 		parse();
 	}
@@ -239,6 +239,25 @@ namespace np
 		return true;
 	}
 
+	std::string ticket::get_service_id() const
+	{
+		if (!parse_success)
+		{
+			return "";
+		}
+
+		const auto& node = nodes[0].data.data_nodes[8];
+		if (node.len != SCE_NP_SERVICE_ID_SIZE)
+		{
+			return "";
+		}
+
+		// Trim null characters
+		const auto& vec = node.data.data_vec;
+		const auto it = std::find(vec.begin(), vec.end(), 0);
+		return std::string(vec.begin(), it);
+	}
+
 	std::optional<ticket_data> ticket::parse_node(std::size_t index) const
 	{
 		if ((index + MIN_TICKET_DATA_SIZE) > size())
@@ -368,7 +387,7 @@ namespace np
 			return;
 		}
 
-		if (nodes[0].id != 0x3000 && nodes[1].id != 0x3002)
+		if (nodes[0].id != 0x3000 || nodes[1].id != 0x3002)
 		{
 			ticket_log.error("The 2 blobs ids are incorrect");
 			return;
@@ -412,7 +431,7 @@ namespace np
 			nc.bind_sce_np_port();
 
 			std::lock_guard lock(mutex_rpcn);
-			rpcn = rpcn::rpcn_client::get_instance();
+			rpcn = rpcn::rpcn_client::get_instance(bind_ip);
 		}
 	}
 
@@ -455,19 +474,10 @@ namespace np
 			}
 
 			// Convert bind address
-			conv = {};
-			if (!inet_pton(AF_INET, g_cfg.net.bind_address.to_string().c_str(), &conv))
-			{
-				// Do not set to disconnected on invalid IP just error and continue using default (0.0.0.0)
-				nph_log.error("Provided IP(%s) address for bind is invalid!", g_cfg.net.bind_address.to_string());
-			}
-			else
-			{
-				bind_ip = conv.s_addr;
+			bind_ip = resolve_binding_ip();
 
-				if (bind_ip)
-					local_ip_addr = bind_ip;
-			}
+			if (bind_ip)
+				local_ip_addr = bind_ip;
 
 			if (g_cfg.net.upnp_enabled)
 				upnp.upnp_enable();
@@ -655,7 +665,7 @@ namespace np
 
 		for (; it != end; ++it)
 		{
-			strcpy(ifr.ifr_name, it->ifr_name);
+			strcpy_trunc(ifr.ifr_name, it->ifr_name);
 			if (ioctl(sock, SIOCGIFFLAGS, &ifr) == 0)
 			{
 				if (!(ifr.ifr_flags & IFF_LOOPBACK))
@@ -788,7 +798,14 @@ namespace np
 				break;
 
 			std::lock_guard lock(mutex_rpcn);
-			rpcn = rpcn::rpcn_client::get_instance();
+
+			bool was_already_started = true;
+
+			if (!rpcn)
+			{
+				rpcn = rpcn::rpcn_client::get_instance(bind_ip);
+				was_already_started = false;
+			}
 
 			// Make sure we're connected
 
@@ -808,7 +825,8 @@ namespace np
 				return;
 			}
 
-			rsx::overlays::queue_message(localized_string_id::RPCN_SUCCESS_LOGGED_ON);
+			if (!was_already_started)
+				rsx::overlays::queue_message(localized_string_id::RPCN_SUCCESS_LOGGED_ON);
 
 			string_to_online_name(rpcn->get_online_name(), online_name);
 			string_to_avatar_url(rpcn->get_avatar_url(), avatar_url);
@@ -836,11 +854,24 @@ namespace np
 	{
 		np_memory.release();
 
-		if (g_cfg.net.psn_status == np_psn_status::psn_rpcn)
+		manager_cb = {};
+		manager_cb_arg = {};
+		basic_handler_registered = false;
+		room_event_cb = {};
+		room_event_cb_ctx = 0;
+		room_event_cb_arg = {};
+		room_msg_cb = {};
+		room_msg_cb_ctx = 0;
+		room_msg_cb_arg = {};
+
+		presence_self.pr_status = {};
+		presence_self.pr_data = {};
+		presence_self.advertised = false;
+
+		if (is_connected && is_psn_active && rpcn)
 		{
-			rpcn_log.notice("Disconnecting from RPCN!");
-			std::lock_guard lock(mutex_rpcn);
-			rpcn.reset();
+			rpcn_log.notice("Setting RPCN state to disconnected!");
+			rpcn->reset_state();
 		}
 	}
 
@@ -854,10 +885,13 @@ namespace np
 		auto& data = ::at32(match2_req_results, event_key);
 		data.apply_relocations(dest_addr);
 
-		vm::ptr<void> dest = vm::cast(dest_addr);
+		const u32 size_copied = std::min(size, data.size());
 
-		u32 size_copied = std::min(size, data.size());
-		memcpy(dest.get_ptr(), data.data(), size_copied);
+		if (dest_addr && size_copied)
+		{
+			vm::ptr<void> dest = vm::cast(dest_addr);
+			memcpy(dest.get_ptr(), data.data(), size_copied);
+		}
 
 		np_memory.free(data.addr());
 		match2_req_results.erase(event_key);
@@ -990,7 +1024,7 @@ namespace np
 			}
 		}
 
-		nph_log.notice("basic_event: event:%d, from:%s(%s), size:%d", *event, static_cast<char*>(from->userId.handle.data), static_cast<char*>(from->name.data), *size);
+		nph_log.notice("basic_event: event:%d, from:%s(%s), size:%d", *event, np::npid_to_string(from->userId), static_cast<char*>(from->name.data), *size);
 
 		return CELL_OK;
 	}
@@ -1051,6 +1085,9 @@ namespace np
 
 	void np_handler::send_message(const message_data& msg_data, const std::set<std::string>& npids)
 	{
+		rpcn_log.notice("Sending message to \"%s\":", fmt::merge(npids, "\",\""));
+		msg_data.print();
+
 		get_rpcn()->send_message(msg_data, npids);
 	}
 
@@ -1073,61 +1110,64 @@ namespace np
 				auto replies = rpcn->get_replies();
 				for (auto& reply : replies)
 				{
-					const u16 command     = reply.second.first;
+					const rpcn::CommandType command = static_cast<rpcn::CommandType>(reply.second.first);
 					const u32 req_id      = reply.first;
 					std::vector<u8>& data = reply.second.second;
 
 					// Every reply should at least contain a return value/error code
 					ensure(data.size() >= 1);
+					const auto error = static_cast<rpcn::ErrorType>(data[0]);
+					vec_stream reply_data(data, 1);
 
 					switch (command)
 					{
-					case rpcn::CommandType::GetWorldList: reply_get_world_list(req_id, data); break;
-					case rpcn::CommandType::CreateRoom: reply_create_join_room(req_id, data); break;
-					case rpcn::CommandType::JoinRoom: reply_join_room(req_id, data); break;
-					case rpcn::CommandType::LeaveRoom: reply_leave_room(req_id, data); break;
-					case rpcn::CommandType::SearchRoom: reply_search_room(req_id, data); break;
-					case rpcn::CommandType::GetRoomDataExternalList: reply_get_roomdata_external_list(req_id, data); break;
-					case rpcn::CommandType::SetRoomDataExternal: reply_set_roomdata_external(req_id, data); break;
-					case rpcn::CommandType::GetRoomDataInternal: reply_get_roomdata_internal(req_id, data); break;
-					case rpcn::CommandType::SetRoomDataInternal: reply_set_roomdata_internal(req_id, data); break;
-					case rpcn::CommandType::GetRoomMemberDataInternal: reply_get_roommemberdata_internal(req_id, data); break;
-					case rpcn::CommandType::SetRoomMemberDataInternal: reply_set_roommemberdata_internal(req_id, data); break;
-					case rpcn::CommandType::SetUserInfo: reply_set_userinfo(req_id, data); break;
-					case rpcn::CommandType::PingRoomOwner: reply_get_ping_info(req_id, data); break;
-					case rpcn::CommandType::SendRoomMessage: reply_send_room_message(req_id, data); break;
-					case rpcn::CommandType::RequestSignalingInfos: reply_req_sign_infos(req_id, data); break;
-					case rpcn::CommandType::RequestTicket: reply_req_ticket(req_id, data); break;
-					case rpcn::CommandType::GetBoardInfos: reply_get_board_infos(req_id, data); break;
-					case rpcn::CommandType::RecordScore: reply_record_score(req_id, data); break;
-					case rpcn::CommandType::RecordScoreData: reply_record_score_data(req_id, data); break;
-					case rpcn::CommandType::GetScoreData: reply_get_score_data(req_id, data); break;
-					case rpcn::CommandType::GetScoreRange: reply_get_score_range(req_id, data); break;
-					case rpcn::CommandType::GetScoreFriends: reply_get_score_friends(req_id, data); break;
-					case rpcn::CommandType::GetScoreNpid: reply_get_score_npid(req_id, data); break;
-					case rpcn::CommandType::TusSetMultiSlotVariable: reply_tus_set_multislot_variable(req_id, data); break;
-					case rpcn::CommandType::TusGetMultiSlotVariable: reply_tus_get_multislot_variable(req_id, data); break;
-					case rpcn::CommandType::TusGetMultiUserVariable: reply_tus_get_multiuser_variable(req_id, data); break;
-					case rpcn::CommandType::TusGetFriendsVariable: reply_tus_get_friends_variable(req_id, data); break;
-					case rpcn::CommandType::TusAddAndGetVariable: reply_tus_add_and_get_variable(req_id, data); break;
-					case rpcn::CommandType::TusTryAndSetVariable: reply_tus_try_and_set_variable(req_id, data); break;
-					case rpcn::CommandType::TusDeleteMultiSlotVariable: reply_tus_delete_multislot_variable(req_id, data); break;
-					case rpcn::CommandType::TusSetData: reply_tus_set_data(req_id, data); break;
-					case rpcn::CommandType::TusGetData: reply_tus_get_data(req_id, data); break;
-					case rpcn::CommandType::TusGetMultiSlotDataStatus: reply_tus_get_multislot_data_status(req_id, data); break;
-					case rpcn::CommandType::TusGetMultiUserDataStatus: reply_tus_get_multiuser_data_status(req_id, data); break;
-					case rpcn::CommandType::TusGetFriendsDataStatus: reply_tus_get_friends_data_status(req_id, data); break;
-					case rpcn::CommandType::TusDeleteMultiSlotData: reply_tus_delete_multislot_data(req_id, data); break;
-					case rpcn::CommandType::CreateRoomGUI: reply_create_room_gui(req_id, data); break;
-					case rpcn::CommandType::JoinRoomGUI: reply_join_room_gui(req_id, data); break;
-					case rpcn::CommandType::LeaveRoomGUI: reply_leave_room_gui(req_id, data); break;
-					case rpcn::CommandType::GetRoomListGUI: reply_get_room_list_gui(req_id, data); break;
-					case rpcn::CommandType::SetRoomSearchFlagGUI: reply_set_room_search_flag_gui(req_id, data); break;
-					case rpcn::CommandType::GetRoomSearchFlagGUI: reply_get_room_search_flag_gui(req_id, data); break;
-					case rpcn::CommandType::SetRoomInfoGUI: reply_set_room_info_gui(req_id, data); break;
-					case rpcn::CommandType::GetRoomInfoGUI: reply_get_room_info_gui(req_id, data); break;
-					case rpcn::CommandType::QuickMatchGUI: reply_quickmatch_gui(req_id, data); break;
-					case rpcn::CommandType::SearchJoinRoomGUI: reply_searchjoin_gui(req_id, data); break;
+					case rpcn::CommandType::GetWorldList: reply_get_world_list(req_id, error, reply_data); break;
+					case rpcn::CommandType::CreateRoom: reply_create_join_room(req_id, error, reply_data); break;
+					case rpcn::CommandType::JoinRoom: reply_join_room(req_id, error, reply_data); break;
+					case rpcn::CommandType::LeaveRoom: reply_leave_room(req_id, error, reply_data); break;
+					case rpcn::CommandType::SearchRoom: reply_search_room(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetRoomDataExternalList: reply_get_roomdata_external_list(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetRoomMemberDataExternalList: reply_get_room_member_data_external_list(req_id, error, reply_data); break;
+					case rpcn::CommandType::SetRoomDataExternal: reply_set_roomdata_external(req_id, error); break;
+					case rpcn::CommandType::GetRoomDataInternal: reply_get_roomdata_internal(req_id, error, reply_data); break;
+					case rpcn::CommandType::SetRoomDataInternal: reply_set_roomdata_internal(req_id, error); break;
+					case rpcn::CommandType::GetRoomMemberDataInternal: reply_get_roommemberdata_internal(req_id, error, reply_data); break;
+					case rpcn::CommandType::SetRoomMemberDataInternal: reply_set_roommemberdata_internal(req_id, error); break;
+					case rpcn::CommandType::SetUserInfo: reply_set_userinfo(req_id, error); break;
+					case rpcn::CommandType::PingRoomOwner: reply_get_ping_info(req_id, error, reply_data); break;
+					case rpcn::CommandType::SendRoomMessage: reply_send_room_message(req_id, error); break;
+					case rpcn::CommandType::RequestSignalingInfos: reply_req_sign_infos(req_id, error, reply_data); break;
+					case rpcn::CommandType::RequestTicket: reply_req_ticket(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetBoardInfos: reply_get_board_infos(req_id, error, reply_data); break;
+					case rpcn::CommandType::RecordScore: reply_record_score(req_id, error, reply_data); break;
+					case rpcn::CommandType::RecordScoreData: reply_record_score_data(req_id, error); break;
+					case rpcn::CommandType::GetScoreData: reply_get_score_data(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetScoreRange: reply_get_score_range(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetScoreFriends: reply_get_score_friends(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetScoreNpid: reply_get_score_npid(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusSetMultiSlotVariable: reply_tus_set_multislot_variable(req_id, error); break;
+					case rpcn::CommandType::TusGetMultiSlotVariable: reply_tus_get_multislot_variable(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusGetMultiUserVariable: reply_tus_get_multiuser_variable(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusGetFriendsVariable: reply_tus_get_friends_variable(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusAddAndGetVariable: reply_tus_add_and_get_variable(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusTryAndSetVariable: reply_tus_try_and_set_variable(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusDeleteMultiSlotVariable: reply_tus_delete_multislot_variable(req_id, error); break;
+					case rpcn::CommandType::TusSetData: reply_tus_set_data(req_id, error); break;
+					case rpcn::CommandType::TusGetData: reply_tus_get_data(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusGetMultiSlotDataStatus: reply_tus_get_multislot_data_status(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusGetMultiUserDataStatus: reply_tus_get_multiuser_data_status(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusGetFriendsDataStatus: reply_tus_get_friends_data_status(req_id, error, reply_data); break;
+					case rpcn::CommandType::TusDeleteMultiSlotData: reply_tus_delete_multislot_data(req_id, error); break;
+					case rpcn::CommandType::CreateRoomGUI: reply_create_room_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::JoinRoomGUI: reply_join_room_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::LeaveRoomGUI: reply_leave_room_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::GetRoomListGUI: reply_get_room_list_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::SetRoomSearchFlagGUI: reply_set_room_search_flag_gui(req_id, error); break;
+					case rpcn::CommandType::GetRoomSearchFlagGUI: reply_get_room_search_flag_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::SetRoomInfoGUI: reply_set_room_info_gui(req_id, error); break;
+					case rpcn::CommandType::GetRoomInfoGUI: reply_get_room_info_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::QuickMatchGUI: reply_quickmatch_gui(req_id, error, reply_data); break;
+					case rpcn::CommandType::SearchJoinRoomGUI: reply_searchjoin_gui(req_id, error, reply_data); break;
 					default: fmt::throw_exception("Unknown reply(%d) received!", command); break;
 					}
 				}
@@ -1135,22 +1175,23 @@ namespace np
 				auto notifications = rpcn->get_notifications();
 				for (auto& notif : notifications)
 				{
+					vec_stream noti_data(notif.second);
+
 					switch (notif.first)
 					{
-					case rpcn::NotificationType::UserJoinedRoom: notif_user_joined_room(notif.second); break;
-					case rpcn::NotificationType::UserLeftRoom: notif_user_left_room(notif.second); break;
-					case rpcn::NotificationType::RoomDestroyed: notif_room_destroyed(notif.second); break;
-					case rpcn::NotificationType::UpdatedRoomDataInternal: notif_updated_room_data_internal(notif.second); break;
-					case rpcn::NotificationType::UpdatedRoomMemberDataInternal: notif_updated_room_member_data_internal(notif.second); break;
-					case rpcn::NotificationType::SignalP2PConnect: notif_p2p_connect(notif.second); break;
-					case rpcn::NotificationType::RoomMessageReceived: notif_room_message_received(notif.second); break;
-					case rpcn::NotificationType::SignalingInfo: notif_signaling_info(notif.second); break;
-					case rpcn::NotificationType::MemberJoinedRoomGUI: notif_member_joined_room_gui(notif.second); break;
-					case rpcn::NotificationType::MemberLeftRoomGUI: notif_member_left_room_gui(notif.second); break;
-					case rpcn::NotificationType::RoomDisappearedGUI: notif_room_disappeared_gui(notif.second); break;
-					case rpcn::NotificationType::RoomOwnerChangedGUI: notif_room_owner_changed_gui(notif.second); break;
-					case rpcn::NotificationType::UserKickedGUI: notif_user_kicked_gui(notif.second); break;
-					case rpcn::NotificationType::QuickMatchCompleteGUI: notif_quickmatch_complete_gui(notif.second); break;
+					case rpcn::NotificationType::UserJoinedRoom: notif_user_joined_room(noti_data); break;
+					case rpcn::NotificationType::UserLeftRoom: notif_user_left_room(noti_data); break;
+					case rpcn::NotificationType::RoomDestroyed: notif_room_destroyed(noti_data); break;
+					case rpcn::NotificationType::UpdatedRoomDataInternal: notif_updated_room_data_internal(noti_data); break;
+					case rpcn::NotificationType::UpdatedRoomMemberDataInternal: notif_updated_room_member_data_internal(noti_data); break;
+					case rpcn::NotificationType::RoomMessageReceived: notif_room_message_received(noti_data); break;
+					case rpcn::NotificationType::SignalingHelper: notif_signaling_helper(noti_data); break;
+					case rpcn::NotificationType::MemberJoinedRoomGUI: notif_member_joined_room_gui(noti_data); break;
+					case rpcn::NotificationType::MemberLeftRoomGUI: notif_member_left_room_gui(noti_data); break;
+					case rpcn::NotificationType::RoomDisappearedGUI: notif_room_disappeared_gui(noti_data); break;
+					case rpcn::NotificationType::RoomOwnerChangedGUI: notif_room_owner_changed_gui(noti_data); break;
+					case rpcn::NotificationType::UserKickedGUI: notif_user_kicked_gui(noti_data); break;
+					case rpcn::NotificationType::QuickMatchCompleteGUI: notif_quickmatch_complete_gui(noti_data); break;
 					default: fmt::throw_exception("Unknown notification(%d) received!", notif.first); break;
 					}
 				}
@@ -1255,19 +1296,20 @@ namespace np
 		return false;
 	}
 
-	u32 np_handler::generate_callback_info(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, SceNpMatching2Event event_type)
+	u32 np_handler::generate_callback_info(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, SceNpMatching2Event event_type, bool abortable)
 	{
-		callback_info ret;
-
 		const auto ctx = get_match2_context(ctx_id);
 		ensure(ctx);
 
 		const u32 req_id = get_req_id(optParam ? optParam->appReqId : ctx->default_match2_optparam.appReqId);
 
-		ret.ctx_id = ctx_id;
-		ret.cb_arg = (optParam && optParam->cbFuncArg) ? optParam->cbFuncArg : ctx->default_match2_optparam.cbFuncArg;
-		ret.cb     = (optParam && optParam->cbFunc) ? optParam->cbFunc : ctx->default_match2_optparam.cbFunc;
-		ret.event_type = event_type;
+		callback_info ret{
+			.ctx_id = ctx_id,
+			.cb = (optParam && optParam->cbFunc) ? optParam->cbFunc : ctx->default_match2_optparam.cbFunc,
+			.cb_arg = (optParam && optParam->cbFuncArg) ? optParam->cbFuncArg : ctx->default_match2_optparam.cbFuncArg,
+			.event_type = event_type,
+			.abortable = abortable,
+		};
 
 		nph_log.trace("Callback used is 0x%x with req_id %d", ret.cb, req_id);
 
@@ -1292,16 +1334,22 @@ namespace np
 		return cb_info;
 	}
 
-	bool np_handler::abort_request(u32 req_id)
+	error_code np_handler::abort_request(u32 req_id)
 	{
-		auto cb_info_opt = take_pending_request(req_id);
+		std::lock_guard lock(mutex_pending_requests);
 
-		if (!cb_info_opt)
-			return false;
+		if (!pending_requests.contains(req_id))
+			return SCE_NP_MATCHING2_ERROR_REQUEST_NOT_FOUND;
 
-		cb_info_opt->queue_callback(req_id, 0, SCE_NP_MATCHING2_ERROR_ABORTED, 0);
+		if (!::at32(pending_requests, req_id).abortable)
+			return SCE_NP_MATCHING2_ERROR_CANNOT_ABORT;
 
-		return true;
+		const auto cb_info = std::move(::at32(pending_requests, req_id));
+		pending_requests.erase(req_id);
+
+		cb_info.queue_callback(req_id, 0, SCE_NP_MATCHING2_ERROR_ABORTED, 0);
+
+		return CELL_OK;
 	}
 
 	event_data& np_handler::allocate_req_result(u32 event_key, u32 max_size, u32 initial_size)
@@ -1313,7 +1361,7 @@ namespace np
 
 	player_history& np_handler::get_player_and_set_timestamp(const SceNpId& npid, u64 timestamp)
 	{
-		std::string npid_str = std::string(npid.handle.data);
+		std::string npid_str = np::npid_to_string(npid);
 
 		if (!players_history.contains(npid_str))
 		{
@@ -1327,6 +1375,24 @@ namespace np
 		return history;
 	}
 
+	u32 np_handler::get_clan_ticket_ready() const
+	{
+		return clan_ticket_ready.load();
+	}
+
+	ticket np_handler::get_clan_ticket() const
+	{
+		clan_ticket_ready.wait(0, atomic_wait_timeout{60'000'000'000}); // 60 seconds
+
+		if (!clan_ticket_ready.load())
+		{
+			rpcn_log.error("Failed to get clan ticket within timeout.");
+			return ticket{};
+		}
+
+		return clan_ticket;
+	}
+
 	constexpr usz MAX_HISTORY_ENTRIES = 200;
 
 	void np_handler::add_player_to_history(const SceNpId* npid, const char* description)
@@ -1401,14 +1467,13 @@ namespace np
 
 		if (all_history)
 		{
+			if (index >= players_history.size())
+				return false;
+
 			auto it = players_history.begin();
 			std::advance(it, index);
-
-			if (it != players_history.end())
-			{
-				string_to_npid(it->first, *npid);
-				return true;
-			}
+			string_to_npid(it->first, *npid);
+			return true;
 		}
 		else
 		{
@@ -1515,7 +1580,7 @@ namespace np
 			}
 		}
 
-		if (send_update && is_psn_active)
+		if (is_psn_active && (!presence_self.advertised || send_update))
 		{
 			std::lock_guard lock(mutex_rpcn);
 
@@ -1524,6 +1589,7 @@ namespace np
 				return;
 			}
 
+			presence_self.advertised = true;
 			rpcn->send_presence(presence_self.pr_com_id, presence_self.pr_title, presence_self.pr_status, presence_self.pr_comment, presence_self.pr_data);
 		}
 	}
@@ -1574,7 +1640,7 @@ namespace np
 			return SCE_NP_BASIC_ERROR_NOT_CONNECTED;
 		}
 
-		auto friend_infos = rpcn->get_friend_presence_by_npid(std::string(npid.handle.data));
+		auto friend_infos = rpcn->get_friend_presence_by_npid(np::npid_to_string(npid));
 		if (!friend_infos)
 		{
 			return SCE_NP_BASIC_ERROR_INVALID_ARGUMENT;
@@ -1609,6 +1675,11 @@ namespace np
 		return np_cache.get_memberids(room_id, sort_method);
 	}
 
+	std::pair<error_code, std::optional<SceNpMatching2SignalingOptParam>> np_handler::local_get_signaling_opt_param(SceNpMatching2RoomId room_id)
+	{
+		return np_cache.get_opt_param(room_id);
+	}
+
 	error_code np_handler::local_get_room_member_data(SceNpMatching2RoomId room_id, SceNpMatching2RoomMemberId member_id, const std::vector<SceNpMatching2AttributeId>& binattrs_list, SceNpMatching2RoomMemberDataInternal* ptr_member, u32 addr_data, u32 size_data, u32 ctx_id)
 	{
 		auto [include_onlinename, include_avatarurl] = get_match2_context_options(ctx_id);
@@ -1683,4 +1754,29 @@ namespace np
 		return ctx;
 	}
 
+	void np_handler::callback_info::queue_callback(u32 req_id, u32 event_key, s32 error_code, u32 data_size) const
+	{
+		if (cb)
+		{
+			sysutil_register_cb([=, ctx_id = this->ctx_id, event_type = this->event_type, cb = this->cb, cb_arg = this->cb_arg](ppu_thread& cb_ppu) -> s32
+				{
+					sceNp2.trace("Calling callback 0x%x with req_id %d, event_type: 0x%x, error_code: 0x%x", cb, req_id, event_type, error_code);
+					cb(cb_ppu, ctx_id, req_id, event_type, event_key, error_code, data_size, cb_arg);
+					return 0;
+				});
+		}
+	}
+
+	SceNpMatching2MemoryInfo np_handler::get_memory_info() const
+	{
+		auto [m_size, m_usage, m_max_usage] = np_memory.get_stats();
+
+		SceNpMatching2MemoryInfo mem_info{};
+		mem_info.totalMemSize = m_size;
+		mem_info.curMemUsage = m_usage;
+		mem_info.maxMemUsage = m_max_usage;
+
+		return mem_info;
+	}
+
 } // namespace np
