@@ -3,6 +3,7 @@
 #include <queue>
 #include <map>
 #include <unordered_map>
+#include <condition_variable>
 
 #include "Emu/Memory/vm_ptr.h"
 #include "Emu/Cell/Modules/sceNp.h"
@@ -10,8 +11,6 @@
 #include "Emu/Cell/Modules/cellSysutil.h"
 
 #include "Emu/NP/rpcn_client.h"
-#include "Emu/NP/generated/np2_structs_generated.h"
-#include "Emu/NP/signaling_handler.h"
 #include "Emu/NP/np_allocator.h"
 #include "Emu/NP/np_cache.h"
 #include "Emu/NP/np_gui_cache.h"
@@ -71,6 +70,7 @@ namespace np
 		bool empty() const;
 
 		bool get_value(s32 param_id, vm::ptr<SceNpTicketParam> param) const;
+		std::string get_service_id() const;
 
 	private:
 		std::optional<ticket_data> parse_node(std::size_t index) const;
@@ -177,6 +177,7 @@ namespace np
 		u32 leave_room(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2LeaveRoomRequest* req);
 		u32 search_room(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2SearchRoomRequest* req);
 		u32 get_roomdata_external_list(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2GetRoomDataExternalListRequest* req);
+		u32 get_room_member_data_external_list(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2GetRoomMemberDataExternalListRequest* req);
 		u32 set_roomdata_external(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2SetRoomDataExternalRequest* req);
 		u32 get_roomdata_internal(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2GetRoomDataInternalRequest* req);
 		u32 set_roomdata_internal(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, const SceNpMatching2SetRoomDataInternalRequest* req);
@@ -235,6 +236,7 @@ namespace np
 		std::pair<error_code, std::optional<SceNpMatching2RoomSlotInfo>> local_get_room_slots(SceNpMatching2RoomId room_id);
 		std::pair<error_code, std::optional<SceNpMatching2SessionPassword>> local_get_room_password(SceNpMatching2RoomId room_id);
 		std::pair<error_code, std::vector<SceNpMatching2RoomMemberId>> local_get_room_memberids(SceNpMatching2RoomId room_id, s32 sort_method);
+		std::pair<error_code, std::optional<SceNpMatching2SignalingOptParam>> local_get_signaling_opt_param(SceNpMatching2RoomId room_id);
 		error_code local_get_room_member_data(SceNpMatching2RoomId room_id, SceNpMatching2RoomMemberId member_id, const std::vector<SceNpMatching2AttributeId>& binattrs_list, SceNpMatching2RoomMemberDataInternal* ptr_member, u32 addr_data, u32 size_data, u32 ctx_id);
 
 		// Local GUI functions
@@ -255,11 +257,14 @@ namespace np
 		// Misc stuff
 		void req_ticket(u32 version, const SceNpId* npid, const char* service_id, const u8* cookie, u32 cookie_size, const char* entitlement_id, u32 consumed_count);
 		const ticket& get_ticket() const;
+		u32 get_clan_ticket_ready() const;
+		ticket get_clan_ticket() const;
 		void add_player_to_history(const SceNpId* npid, const char* description);
 		u32 add_players_to_history(const SceNpId* npids, const char* description, u32 count);
 		u32 get_players_history_count(u32 options);
 		bool get_player_history_entry(u32 options, u32 index, SceNpId* npid);
-		bool abort_request(u32 req_id);
+		SceNpMatching2MemoryInfo get_memory_info() const;
+		error_code abort_request(u32 req_id);
 
 		// For signaling
 		void req_sign_infos(const std::string& npid, u32 conn_id);
@@ -295,79 +300,79 @@ namespace np
 		bool error_and_disconnect(const std::string& error_msg);
 
 		// Notification handlers
-		void notif_user_joined_room(std::vector<u8>& data);
-		void notif_user_left_room(std::vector<u8>& data);
-		void notif_room_destroyed(std::vector<u8>& data);
-		void notif_updated_room_data_internal(std::vector<u8>& data);
-		void notif_updated_room_member_data_internal(std::vector<u8>& data);
-		void notif_p2p_connect(std::vector<u8>& data);
-		void notif_signaling_info(std::vector<u8>& data);
-		void notif_room_message_received(std::vector<u8>& data);
+		void notif_user_joined_room(vec_stream& noti);
+		void notif_user_left_room(vec_stream& noti);
+		void notif_room_destroyed(vec_stream& noti);
+		void notif_updated_room_data_internal(vec_stream& noti);
+		void notif_updated_room_member_data_internal(vec_stream& noti);
+		void notif_signaling_helper(vec_stream& noti);
+		void notif_room_message_received(vec_stream& noti);
 
-		void generic_gui_notification_handler(std::vector<u8>& data, std::string_view name, s32 notification_type);
+		void generic_gui_notification_handler(vec_stream& noti, std::string_view name, s32 notification_type);
 
-		void notif_member_joined_room_gui(std::vector<u8>& data);
-		void notif_member_left_room_gui(std::vector<u8>& data);
-		void notif_room_disappeared_gui(std::vector<u8>& data);
-		void notif_room_owner_changed_gui(std::vector<u8>& data);
-		void notif_user_kicked_gui(std::vector<u8>& data);
-		void notif_quickmatch_complete_gui(std::vector<u8>& data);
+		void notif_member_joined_room_gui(vec_stream& noti);
+		void notif_member_left_room_gui(vec_stream& noti);
+		void notif_room_disappeared_gui(vec_stream& noti);
+		void notif_room_owner_changed_gui(vec_stream& noti);
+		void notif_user_kicked_gui(vec_stream& noti);
+		void notif_quickmatch_complete_gui(vec_stream& noti);
 
 		// Reply handlers
-		bool reply_get_world_list(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_create_join_room(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_join_room(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_leave_room(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_search_room(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_roomdata_external_list(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_set_roomdata_external(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_roomdata_internal(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_set_roomdata_internal(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_set_roommemberdata_internal(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_roommemberdata_internal(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_set_userinfo(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_ping_info(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_send_room_message(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_req_sign_infos(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_req_ticket(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_board_infos(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_record_score(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_record_score_data(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_score_data(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_score_range(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_score_friends(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_score_npid(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_set_multislot_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_multislot_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_multiuser_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_friends_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_add_and_get_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_try_and_set_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_delete_multislot_variable(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_set_data(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_data(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_multislot_data_status(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_multiuser_data_status(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_get_friends_data_status(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_tus_delete_multislot_data(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_create_room_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_join_room_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_leave_room_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_room_list_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_set_room_search_flag_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_room_search_flag_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_set_room_info_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_get_room_info_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_quickmatch_gui(u32 req_id, std::vector<u8>& reply_data);
-		bool reply_searchjoin_gui(u32 req_id, std::vector<u8>& reply_data);
+		void reply_get_world_list(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_create_join_room(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_join_room(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_leave_room(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_search_room(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_roomdata_external_list(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_room_member_data_external_list(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_set_roomdata_external(u32 req_id, rpcn::ErrorType error);
+		void reply_get_roomdata_internal(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_set_roomdata_internal(u32 req_id, rpcn::ErrorType error);
+		void reply_set_roommemberdata_internal(u32 req_id, rpcn::ErrorType error);
+		void reply_get_roommemberdata_internal(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_set_userinfo(u32 req_id, rpcn::ErrorType error);
+		void reply_get_ping_info(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_send_room_message(u32 req_id, rpcn::ErrorType error);
+		void reply_req_sign_infos(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_req_ticket(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_board_infos(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_record_score(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_record_score_data(u32 req_id, rpcn::ErrorType error);
+		void reply_get_score_data(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_score_range(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_score_friends(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_score_npid(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_set_multislot_variable(u32 req_id, rpcn::ErrorType error);
+		void reply_tus_get_multislot_variable(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_get_multiuser_variable(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_get_friends_variable(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_add_and_get_variable(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_try_and_set_variable(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_delete_multislot_variable(u32 req_id, rpcn::ErrorType error);
+		void reply_tus_set_data(u32 req_id, rpcn::ErrorType error);
+		void reply_tus_get_data(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_get_multislot_data_status(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_get_multiuser_data_status(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_get_friends_data_status(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_tus_delete_multislot_data(u32 req_id, rpcn::ErrorType error);
+		void reply_create_room_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_join_room_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_leave_room_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_get_room_list_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_set_room_search_flag_gui(u32 req_id, rpcn::ErrorType error);
+		void reply_get_room_search_flag_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_set_room_info_gui(u32 req_id, rpcn::ErrorType error);
+		void reply_get_room_info_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_quickmatch_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void reply_searchjoin_gui(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
 
 		// Helper functions
 		std::pair<bool, bool> get_match2_context_options(u32 ctx_id);
-		bool handle_GetScoreResponse(u32 req_id, std::vector<u8>& reply_data, bool simple_result = false);
-		bool handle_tus_no_data(u32 req_id, std::vector<u8>& reply_data);
-		bool handle_TusVarResponse(u32 req_id, std::vector<u8>& reply_data);
-		bool handle_TusVariable(u32 req_id, std::vector<u8>& reply_data);
-		bool handle_TusDataStatusResponse(u32 req_id, std::vector<u8>& reply_data);
+		void handle_GetScoreResponse(u32 req_id, rpcn::ErrorType error, vec_stream& reply, bool simple_result = false);
+		void handle_tus_no_data(u32 req_id, rpcn::ErrorType error);
+		void handle_TusVarResponse(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void handle_TusVariable(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
+		void handle_TusDataStatusResponse(u32 req_id, rpcn::ErrorType error, vec_stream& reply);
 
 		struct callback_info
 		{
@@ -375,21 +380,12 @@ namespace np
 			vm::ptr<SceNpMatching2RequestCallback> cb;
 			vm::ptr<void> cb_arg;
 			SceNpMatching2Event event_type;
+			bool abortable;
 
-			void queue_callback(u32 req_id, u32 event_key, s32 error_code, u32 data_size) const
-			{
-				if (cb)
-				{
-					sysutil_register_cb([=, ctx_id = this->ctx_id, event_type = this->event_type, cb = this->cb, cb_arg = this->cb_arg](ppu_thread& cb_ppu) -> s32
-					{
-						cb(cb_ppu, ctx_id, req_id, event_type, event_key, error_code, data_size, cb_arg);
-						return 0;
-					});
-				}
-			}
+			void queue_callback(u32 req_id, u32 event_key, s32 error_code, u32 data_size) const;
 		};
 
-		u32 generate_callback_info(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, SceNpMatching2Event event_type);
+		u32 generate_callback_info(SceNpMatching2ContextId ctx_id, vm::cptr<SceNpMatching2RequestOptParam> optParam, SceNpMatching2Event event_type, bool abortable);
 		std::optional<callback_info> take_pending_request(u32 req_id);
 
 	private:
@@ -418,6 +414,10 @@ namespace np
 
 		ticket current_ticket;
 
+		// Clan ticket
+		atomic_t<u32> clan_ticket_ready = 0;
+		ticket clan_ticket;
+
 		// IP & DNS info
 		std::string hostname = "localhost";
 		std::array<u8, 6> ether_address{};
@@ -512,6 +512,7 @@ namespace np
 			std::string pr_status;
 			std::string pr_comment;
 			std::vector<u8> pr_data;
+			atomic_t<bool> advertised = false;
 		} presence_self;
 
 		player_history& get_player_and_set_timestamp(const SceNpId& npid, u64 timestamp);
