@@ -1,21 +1,21 @@
 #include "stdafx.h"
-#include "Emu/Cell/PPUModule.h"
+#include "Emu/Cell/PPUCallback.h"
 #include "Emu/Cell/Modules/cellSysutil.h"
 #include "Emu/IdManager.h"
 #include "Emu/NP/np_handler.h"
 #include "Emu/NP/np_helpers.h"
 #include "Emu/NP/np_structs_extra.h"
-#include "Emu/NP/fb_helpers.h"
+#include "Emu/NP/pb_helpers.h"
+#include "Emu/NP/signaling_handler.h"
+#include "Emu/NP/ip_address.h"
 
 LOG_CHANNEL(rpcn_log, "rpcn");
 
 namespace np
 {
-	void np_handler::notif_user_joined_room(std::vector<u8>& data)
+	void np_handler::notif_user_joined_room(vec_stream& noti)
 	{
-		vec_stream noti(data);
-		u64 room_id       = noti.get<u64>();
-		const auto* update_info = noti.get_flatbuffer<RoomMemberUpdateInfo>();
+		auto notification = noti.get_protobuf<np2_structs::NotificationUserJoinedRoom>();
 
 		if (noti.is_error())
 		{
@@ -23,20 +23,45 @@ namespace np
 			return;
 		}
 
+		ensure(notification->has_update_info());
+
 		const u32 event_key = get_event_key();
-		auto [include_onlinename, include_avatarurl] = get_match2_context_options(room_event_cb_ctx);
+		const auto [include_onlinename, include_avatarurl] = get_match2_context_options(room_event_cb_ctx);
+		const auto room_id = notification->room_id();
 
 		auto& edata      = allocate_req_result(event_key, SCE_NP_MATCHING2_EVENT_DATA_MAX_SIZE_RoomMemberUpdateInfo, sizeof(SceNpMatching2RoomMemberUpdateInfo));
 		auto* notif_data = reinterpret_cast<SceNpMatching2RoomMemberUpdateInfo*>(edata.data());
-		RoomMemberUpdateInfo_to_SceNpMatching2RoomMemberUpdateInfo(edata, update_info, notif_data, include_onlinename, include_avatarurl);
+		RoomMemberUpdateInfo_to_SceNpMatching2RoomMemberUpdateInfo(edata, notification->update_info(), notif_data, include_onlinename, include_avatarurl);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
+		// Ensures we do not call the callback if the room is not in the cache(ie we left the room already)
 		if (!np_cache.add_member(room_id, notif_data->roomMemberDataInternal.get_ptr()))
+		{
+			get_match2_event(event_key, 0, 0);
 			return;
+		}
 
-		rpcn_log.notice("Received notification that user %s(%d) joined the room(%d)", notif_data->roomMemberDataInternal->userInfo.npId.handle.data, notif_data->roomMemberDataInternal->memberId, room_id);
+		rpcn_log.notice("Received notification that user %s(%d) joined the room(%d)", np::npid_to_string(notif_data->roomMemberDataInternal->userInfo.npId), notif_data->roomMemberDataInternal->memberId, room_id);
 		extra_nps::print_SceNpMatching2RoomMemberDataInternal(notif_data->roomMemberDataInternal.get_ptr());
 
+		// We initiate signaling if necessary
+		if (notification->has_signaling())
+		{
+			const auto& signaling_info = notification->signaling();
+			const u32 addr_p2p = register_ip(signaling_info.ip());
+			const u16 port_p2p = signaling_info.port().value();
+
+			const u16 member_id = notif_data->roomMemberDataInternal->memberId;
+			const SceNpId& npid = notif_data->roomMemberDataInternal->userInfo.npId;
+
+			rpcn_log.notice("Join notification told to connect to member(%d=%s) of room(%d): %s:%d", member_id, np::npid_to_string(npid), room_id, ip_to_string(addr_p2p), port_p2p);
+
+			// Attempt Signaling
+			auto& sigh = g_fxo->get<named_thread<signaling_handler>>();
+			const u32 conn_id = sigh.init_sig2(npid, room_id, member_id);
+			sigh.start_sig(conn_id, addr_p2p, port_p2p);
+		}
+
 		if (room_event_cb)
 		{
 			sysutil_register_cb([room_event_cb = this->room_event_cb, room_id, event_key, room_event_cb_ctx = this->room_event_cb_ctx, room_event_cb_arg = this->room_event_cb_arg, size = edata.size()](ppu_thread& cb_ppu) -> s32
@@ -47,11 +72,10 @@ namespace np
 		}
 	}
 
-	void np_handler::notif_user_left_room(std::vector<u8>& data)
+	void np_handler::notif_user_left_room(vec_stream& noti)
 	{
-		vec_stream noti(data);
 		u64 room_id       = noti.get<u64>();
-		const auto* update_info = noti.get_flatbuffer<RoomMemberUpdateInfo>();
+		auto update_info = noti.get_protobuf<np2_structs::RoomMemberUpdateInfo>();
 
 		if (noti.is_error())
 		{
@@ -64,13 +88,17 @@ namespace np
 
 		auto& edata      = allocate_req_result(event_key, SCE_NP_MATCHING2_EVENT_DATA_MAX_SIZE_RoomMemberUpdateInfo, sizeof(SceNpMatching2RoomMemberUpdateInfo));
 		auto* notif_data = reinterpret_cast<SceNpMatching2RoomMemberUpdateInfo*>(edata.data());
-		RoomMemberUpdateInfo_to_SceNpMatching2RoomMemberUpdateInfo(edata, update_info, notif_data, include_onlinename, include_avatarurl);
+		RoomMemberUpdateInfo_to_SceNpMatching2RoomMemberUpdateInfo(edata, *update_info, notif_data, include_onlinename, include_avatarurl);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
+		// Ensures we do not call the callback if the room is not in the cache(ie we left the room already)
 		if (!np_cache.del_member(room_id, notif_data->roomMemberDataInternal->memberId))
+		{
+			get_match2_event(event_key, 0, 0);
 			return;
+		}
 
-		rpcn_log.notice("Received notification that user %s(%d) left the room(%d)", notif_data->roomMemberDataInternal->userInfo.npId.handle.data, notif_data->roomMemberDataInternal->memberId, room_id);
+		rpcn_log.notice("Received notification that user %s(%d) left the room(%d)", np::npid_to_string(notif_data->roomMemberDataInternal->userInfo.npId), notif_data->roomMemberDataInternal->memberId, room_id);
 		extra_nps::print_SceNpMatching2RoomMemberDataInternal(notif_data->roomMemberDataInternal.get_ptr());
 
 		if (room_event_cb)
@@ -83,11 +111,10 @@ namespace np
 		}
 	}
 
-	void np_handler::notif_room_destroyed(std::vector<u8>& data)
+	void np_handler::notif_room_destroyed(vec_stream& noti)
 	{
-		vec_stream noti(data);
 		u64 room_id       = noti.get<u64>();
-		const auto* update_info = noti.get_flatbuffer<RoomUpdateInfo>();
+		auto update_info = noti.get_protobuf<np2_structs::RoomUpdateInfo>();
 
 		if (noti.is_error())
 		{
@@ -99,7 +126,7 @@ namespace np
 
 		auto& edata      = allocate_req_result(event_key, SCE_NP_MATCHING2_EVENT_DATA_MAX_SIZE_RoomUpdateInfo, sizeof(SceNpMatching2RoomUpdateInfo));
 		auto* notif_data = reinterpret_cast<SceNpMatching2RoomUpdateInfo*>(edata.data());
-		RoomUpdateInfo_to_SceNpMatching2RoomUpdateInfo(update_info, notif_data);
+		RoomUpdateInfo_to_SceNpMatching2RoomUpdateInfo(*update_info, notif_data);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
 		rpcn_log.notice("Received notification that room(%d) was destroyed", room_id);
@@ -117,11 +144,10 @@ namespace np
 		}
 	}
 
-	void np_handler::notif_updated_room_data_internal(std::vector<u8>& data)
+	void np_handler::notif_updated_room_data_internal(vec_stream& noti)
 	{
-		vec_stream noti(data);
 		SceNpMatching2RoomId room_id = noti.get<u64>();
-		const auto* update_info = noti.get_flatbuffer<RoomDataInternalUpdateInfo>();
+		auto update_info = noti.get_protobuf<np2_structs::RoomDataInternalUpdateInfo>();
 
 		if (noti.is_error())
 		{
@@ -134,7 +160,7 @@ namespace np
 
 		auto& edata      = allocate_req_result(event_key, SCE_NP_MATCHING2_EVENT_DATA_MAX_SIZE_RoomDataInternalUpdateInfo, sizeof(SceNpMatching2RoomDataInternalUpdateInfo));
 		auto* notif_data = reinterpret_cast<SceNpMatching2RoomDataInternalUpdateInfo*>(edata.data());
-		RoomDataInternalUpdateInfo_to_SceNpMatching2RoomDataInternalUpdateInfo(edata, update_info, notif_data, npid, include_onlinename, include_avatarurl);
+		RoomDataInternalUpdateInfo_to_SceNpMatching2RoomDataInternalUpdateInfo(edata, *update_info, notif_data, npid, include_onlinename, include_avatarurl);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
 		np_cache.insert_room(notif_data->newRoomDataInternal.get_ptr());
@@ -153,11 +179,10 @@ namespace np
 		}
 	}
 
-	void np_handler::notif_updated_room_member_data_internal(std::vector<u8>& data)
+	void np_handler::notif_updated_room_member_data_internal(vec_stream& noti)
 	{
-		vec_stream noti(data);
 		SceNpMatching2RoomId room_id = noti.get<u64>();
-		const auto* update_info = noti.get_flatbuffer<RoomMemberDataInternalUpdateInfo>();
+		auto update_info = noti.get_protobuf<np2_structs::RoomMemberDataInternalUpdateInfo>();
 
 		if (noti.is_error())
 		{
@@ -170,13 +195,16 @@ namespace np
 
 		auto& edata      = allocate_req_result(event_key, SCE_NP_MATCHING2_EVENT_DATA_MAX_SIZE_RoomMemberDataInternalUpdateInfo, sizeof(SceNpMatching2RoomMemberDataInternalUpdateInfo));
 		auto* notif_data = reinterpret_cast<SceNpMatching2RoomMemberDataInternalUpdateInfo*>(edata.data());
-		RoomMemberDataInternalUpdateInfo_to_SceNpMatching2RoomMemberDataInternalUpdateInfo(edata, update_info, notif_data, include_onlinename, include_avatarurl);
+		RoomMemberDataInternalUpdateInfo_to_SceNpMatching2RoomMemberDataInternalUpdateInfo(edata, *update_info, notif_data, include_onlinename, include_avatarurl);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
 		if (!np_cache.add_member(room_id, notif_data->newRoomMemberDataInternal.get_ptr()))
+		{
+			get_match2_event(event_key, 0, 0);
 			return;
+		}
 
-		rpcn_log.notice("Received notification that user's %s(%d) room (%d) data was updated", notif_data->newRoomMemberDataInternal->userInfo.npId.handle.data, notif_data->newRoomMemberDataInternal->memberId, room_id);
+		rpcn_log.notice("Received notification that user's %s(%d) room (%d) data was updated", np::npid_to_string(notif_data->newRoomMemberDataInternal->userInfo.npId), notif_data->newRoomMemberDataInternal->memberId, room_id);
 		extra_nps::print_SceNpMatching2RoomMemberDataInternal(notif_data->newRoomMemberDataInternal.get_ptr());
 
 		if (room_event_cb)
@@ -189,12 +217,11 @@ namespace np
 		}
 	}
 
-	void np_handler::notif_room_message_received(std::vector<u8>& data)
+	void np_handler::notif_room_message_received(vec_stream& noti)
 	{
-		vec_stream noti(data);
 		u64 room_id        = noti.get<u64>();
 		u16 member_id      = noti.get<u16>();
-		const auto* message_info = noti.get_flatbuffer<RoomMessageInfo>();
+		auto message_info = noti.get_protobuf<np2_structs::RoomMessageInfo>();
 
 		if (noti.is_error())
 		{
@@ -207,7 +234,7 @@ namespace np
 
 		auto& edata      = allocate_req_result(event_key, SCE_NP_MATCHING2_EVENT_DATA_MAX_SIZE_RoomMessageInfo, sizeof(SceNpMatching2RoomMessageInfo));
 		auto* notif_data = reinterpret_cast<SceNpMatching2RoomMessageInfo*>(edata.data());
-		RoomMessageInfo_to_SceNpMatching2RoomMessageInfo(edata, message_info, notif_data, include_onlinename, include_avatarurl);
+		RoomMessageInfo_to_SceNpMatching2RoomMessageInfo(edata, *message_info, notif_data, include_onlinename, include_avatarurl);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
 		rpcn_log.notice("Received notification of a room message from member(%d) in room(%d)", member_id, room_id);
@@ -222,56 +249,29 @@ namespace np
 		}
 	}
 
-	void np_handler::notif_p2p_connect(std::vector<u8>& data)
+	void np_handler::notif_signaling_helper(vec_stream& noti)
 	{
-		vec_stream noti(data);
-		const u64 room_id = noti.get<u64>();
-		const u16 member_id = noti.get<u16>();
-		const u16 port_p2p = noti.get<u16>();
-		const u32 addr_p2p = noti.get<u32>();
+		auto matching_info = noti.get_protobuf<np2_structs::MatchingSignalingInfo>();
 
-		if (noti.is_error())
+		if (noti.is_error() || !matching_info->has_addr() || matching_info->npid().empty() || matching_info->addr().ip().empty())
 		{
-			rpcn_log.error("Received faulty SignalP2PConnect notification");
-			return;
-		}
-
-		auto [res, npid] = np_cache.get_npid(room_id, member_id);
-		if (!npid)
-			return;
-
-		rpcn_log.notice("Received notification to connect to member(%d=%s) of room(%d): %s:%d", member_id, reinterpret_cast<const char*>((*npid).handle.data), room_id, ip_to_string(addr_p2p), port_p2p);
-
-		// Attempt Signaling
-		auto& sigh = g_fxo->get<named_thread<signaling_handler>>();
-		const u32 conn_id = sigh.init_sig2(*npid, room_id, member_id);
-		sigh.start_sig(conn_id, addr_p2p, port_p2p);
-	}
-
-	void np_handler::notif_signaling_info(std::vector<u8>& data)
-	{
-		vec_stream noti(data);
-		const u32 addr_p2p = noti.get<u32>();
-		const u32 port_p2p = noti.get<u16>();
-		const std::string str_npid = noti.get_string(false);
-
-		if (noti.is_error())
-		{
-			rpcn_log.error("Received faulty SignalingInfo notification");
+			rpcn_log.error("Received faulty SignalingHelper notification");
 			return;
 		}
 
 		SceNpId npid_p2p;
-		string_to_npid(str_npid, npid_p2p);
+		string_to_npid(matching_info->npid(), npid_p2p);
+
+		const u32 addr_p2p = register_ip(matching_info->addr().ip());
+		const u16 port_p2p = matching_info->addr().port().value();
 
 		auto& sigh = g_fxo->get<named_thread<signaling_handler>>();
 		sigh.send_information_packets(addr_p2p, port_p2p, npid_p2p);
 	}
 
-	void np_handler::generic_gui_notification_handler(std::vector<u8>& data, std::string_view name, s32 notification_type)
+	void np_handler::generic_gui_notification_handler(vec_stream& noti, std::string_view name, s32 notification_type)
 	{
-		vec_stream noti(data);
-		const auto* update_info = noti.get_flatbuffer<MatchingRoomStatus>();
+		auto update_info = noti.get_protobuf<np2_structs::MatchingRoomStatus>();
 
 		if (noti.is_error())
 		{
@@ -291,7 +291,7 @@ namespace np
 
 		event_data edata(np_memory.allocate(MAX_SceNpMatchingRoomStatus_SIZE), sizeof(SceNpMatchingRoomStatus), MAX_SceNpMatchingRoomStatus_SIZE);
 		auto* room_status = reinterpret_cast<SceNpMatchingRoomStatus*>(edata.data());
-		MatchingRoomStatus_to_SceNpMatchingRoomStatus(edata, update_info, room_status);
+		MatchingRoomStatus_to_SceNpMatchingRoomStatus(edata, *update_info, room_status);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
 		extra_nps::print_SceNpMatchingRoomStatus(room_status);
@@ -323,37 +323,36 @@ namespace np
 		ctx->queue_callback(req_id, notification_type, 0);
 	}
 
-	void np_handler::notif_member_joined_room_gui(std::vector<u8>& data)
+	void np_handler::notif_member_joined_room_gui(vec_stream& noti)
 	{
-		return generic_gui_notification_handler(data, "MemberJoinedRoomGUI", SCE_NP_MATCHING_EVENT_ROOM_UPDATE_NEW_MEMBER);
+		return generic_gui_notification_handler(noti, "MemberJoinedRoomGUI", SCE_NP_MATCHING_EVENT_ROOM_UPDATE_NEW_MEMBER);
 	}
 
-	void np_handler::notif_member_left_room_gui(std::vector<u8>& data)
+	void np_handler::notif_member_left_room_gui(vec_stream& noti)
 	{
-		return generic_gui_notification_handler(data, "MemberLeftRoomGUI", SCE_NP_MATCHING_EVENT_ROOM_UPDATE_MEMBER_LEAVE);
+		return generic_gui_notification_handler(noti, "MemberLeftRoomGUI", SCE_NP_MATCHING_EVENT_ROOM_UPDATE_MEMBER_LEAVE);
 	}
 
-	void np_handler::notif_room_disappeared_gui(std::vector<u8>& data)
+	void np_handler::notif_room_disappeared_gui(vec_stream& noti)
 	{
-		return generic_gui_notification_handler(data, "RoomDisappearedGUI", SCE_NP_MATCHING_EVENT_ROOM_DISAPPEARED);
+		return generic_gui_notification_handler(noti, "RoomDisappearedGUI", SCE_NP_MATCHING_EVENT_ROOM_DISAPPEARED);
 	}
 
-	void np_handler::notif_room_owner_changed_gui(std::vector<u8>& data)
+	void np_handler::notif_room_owner_changed_gui(vec_stream& noti)
 	{
-		return generic_gui_notification_handler(data, "RoomOwnerChangedGUI", SCE_NP_MATCHING_EVENT_ROOM_UPDATE_OWNER_CHANGE);
+		return generic_gui_notification_handler(noti, "RoomOwnerChangedGUI", SCE_NP_MATCHING_EVENT_ROOM_UPDATE_OWNER_CHANGE);
 	}
 
-	void np_handler::notif_user_kicked_gui(std::vector<u8>& data)
+	void np_handler::notif_user_kicked_gui(vec_stream& noti)
 	{
-		return generic_gui_notification_handler(data, "UserKickedGUI", SCE_NP_MATCHING_EVENT_ROOM_KICKED);
+		return generic_gui_notification_handler(noti, "UserKickedGUI", SCE_NP_MATCHING_EVENT_ROOM_KICKED);
 	}
 
 	void gui_epilog(const shared_ptr<matching_ctx>& ctx);
 
-	void np_handler::notif_quickmatch_complete_gui(std::vector<u8>& data)
+	void np_handler::notif_quickmatch_complete_gui(vec_stream& noti)
 	{
-		vec_stream noti(data);
-		const auto* update_info = noti.get_flatbuffer<MatchingRoomStatus>();
+		auto update_info = noti.get_protobuf<np2_structs::MatchingRoomStatus>();
 
 		if (noti.is_error())
 		{
@@ -365,7 +364,7 @@ namespace np
 
 		event_data edata(np_memory.allocate(MAX_SceNpMatchingJoinedRoomInfo_SIZE), sizeof(SceNpMatchingJoinedRoomInfo), MAX_SceNpMatchingJoinedRoomInfo_SIZE);
 		auto* room_info = reinterpret_cast<SceNpMatchingJoinedRoomInfo*>(edata.data());
-		MatchingRoomStatus_to_SceNpMatchingJoinedRoomInfo(edata, update_info, room_info);
+		MatchingRoomStatus_to_SceNpMatchingJoinedRoomInfo(edata, *update_info, room_info);
 		np_memory.shrink_allocation(edata.addr(), edata.size());
 
 		extra_nps::print_SceNpMatchingJoinedRoomInfo(room_info);
@@ -382,7 +381,10 @@ namespace np
 		auto ctx = get_matching_context(ctx_id);
 
 		if (!ctx)
+		{
+			np_memory.free(edata.addr());
 			return;
+		}
 
 		gui_cache.add_room(room_info->room_status.id);
 
