@@ -2,7 +2,6 @@
 
 #include "util/types.hpp"
 #include "Utilities/geometry.h"
-#include "Utilities/address_range.h"
 #include "TextureUtils.h"
 #include "../rsx_utils.h"
 #include "Emu/Memory/vm.h"
@@ -147,7 +146,7 @@ namespace rsx
 		u8  samples_x = 1;
 		u8  samples_y = 1;
 
-		rsx::address_range memory_range;
+		rsx::address_range32 memory_range;
 
 		std::unique_ptr<typename std::remove_pointer_t<image_storage_type>> resolve_surface;
 		surface_sample_layout sample_layout = surface_sample_layout::null;
@@ -368,7 +367,7 @@ namespace rsx
 
 			const u32 internal_height = get_surface_height<rsx::surface_metrics::samples>();
 			const u32 excess = (rsx_pitch - native_pitch);
-			memory_range = rsx::address_range::start_length(base_addr, internal_height * rsx_pitch - excess);
+			memory_range = rsx::address_range32::start_length(base_addr, internal_height * rsx_pitch - excess);
 		}
 
 		void sync_tag()
@@ -420,7 +419,7 @@ namespace rsx
 
 			const u32 internal_height = get_surface_height<rsx::surface_metrics::samples>();
 			const u32 excess = (rsx_pitch - native_pitch);
-			memory_range = rsx::address_range::start_length(base_addr, internal_height * rsx_pitch - excess);
+			memory_range = rsx::address_range32::start_length(base_addr, internal_height * rsx_pitch - excess);
 		}
 
 		void sync_tag()
@@ -436,9 +435,9 @@ namespace rsx
 			memory_tag_samples[0].second = ~memory_tag_samples[0].second;
 		}
 
-		bool test()
+		bool test() const
 		{
-			for (auto &e : memory_tag_samples)
+			for (const auto& e : memory_tag_samples)
 			{
 				if (e.second != *reinterpret_cast<nse_t<u64, 1>*>(vm::g_sudo_addr + e.first))
 					return false;
@@ -472,10 +471,10 @@ namespace rsx
 				return 0;
 
 			// Sort here before doing transfers since surfaces may have been updated in the meantime
-			std::sort(old_contents.begin(), old_contents.end(), [](auto& a, auto &b)
+			std::sort(old_contents.begin(), old_contents.end(), [](const auto& a, const auto &b)
 			{
-				auto _a = static_cast<T*>(a.source);
-				auto _b = static_cast<T*>(b.source);
+				const auto _a = static_cast<const T*>(a.source);
+				const auto _b = static_cast<const T*>(b.source);
 				return (_a->last_use_tag < _b->last_use_tag);
 			});
 
@@ -659,7 +658,7 @@ namespace rsx
 			return { 0, 0, internal_width, internal_height };
 		}
 
-		inline rsx::address_range get_memory_range() const
+		inline rsx::address_range32 get_memory_range() const
 		{
 			return memory_range;
 		}
