@@ -1,16 +1,13 @@
 #include "stdafx.h"
 #include "GLTexture.h"
 #include "GLCompute.h"
-#include "GLRenderTargets.h"
 #include "GLOverlays.h"
 #include "GLGSRender.h"
 
 #include "glutils/blitter.h"
 #include "glutils/ring_buffer.h"
 
-#include "../GCM.h"
 #include "../RSXThread.h"
-#include "../RSXTexture.h"
 
 #include "util/asm.hpp"
 
@@ -39,6 +36,16 @@ namespace gl
 	{
 		switch (block_size)
 		{
+		case 1:
+			gl::get_compute_task<gl::cs_deswizzle_3d<u8, WordType, SwapBytes>>()->run(
+				cmd, dst, dst_offset, src, src_offset,
+				data_length, width, height, depth, 1);
+			break;
+		case 2:
+			gl::get_compute_task<gl::cs_deswizzle_3d<u16, WordType, SwapBytes>>()->run(
+				cmd, dst, dst_offset, src, src_offset,
+				data_length, width, height, depth, 1);
+			break;
 		case 4:
 			gl::get_compute_task<gl::cs_deswizzle_3d<u32, WordType, SwapBytes>>()->run(
 				cmd, dst, dst_offset, src, src_offset,
@@ -73,6 +80,7 @@ namespace gl
 
 	GLenum get_sized_internal_format(u32 texture_format)
 	{
+		const bool supports_dxt = get_driver_caps().EXT_texture_compression_s3tc_supported;
 		switch (texture_format)
 		{
 		case CELL_GCM_TEXTURE_B8: return GL_R8;
@@ -95,9 +103,9 @@ namespace gl
 		case CELL_GCM_TEXTURE_D1R5G5B5: return GL_BGR5_A1;
 		case CELL_GCM_TEXTURE_D8R8G8B8: return GL_BGRA8;
 		case CELL_GCM_TEXTURE_Y16_X16_FLOAT: return GL_RG16F;
-		case CELL_GCM_TEXTURE_COMPRESSED_DXT1: return GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
-		case CELL_GCM_TEXTURE_COMPRESSED_DXT23: return GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
-		case CELL_GCM_TEXTURE_COMPRESSED_DXT45: return GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
+		case CELL_GCM_TEXTURE_COMPRESSED_DXT1: return supports_dxt ? GL_COMPRESSED_RGBA_S3TC_DXT1_EXT : GL_BGRA8;
+		case CELL_GCM_TEXTURE_COMPRESSED_DXT23: return supports_dxt ? GL_COMPRESSED_RGBA_S3TC_DXT3_EXT : GL_BGRA8;
+		case CELL_GCM_TEXTURE_COMPRESSED_DXT45: return supports_dxt ? GL_COMPRESSED_RGBA_S3TC_DXT5_EXT : GL_BGRA8;
 		case CELL_GCM_TEXTURE_COMPRESSED_HILO8: return GL_RG8;
 		case CELL_GCM_TEXTURE_COMPRESSED_HILO_S8: return GL_RG8_SNORM;
 		case CELL_GCM_TEXTURE_COMPRESSED_B8R8_G8R8: return GL_BGRA8;
@@ -108,6 +116,7 @@ namespace gl
 
 	std::tuple<GLenum, GLenum> get_format_type(u32 texture_format)
 	{
+		const bool supports_dxt = get_driver_caps().EXT_texture_compression_s3tc_supported;
 		switch (texture_format)
 		{
 		case CELL_GCM_TEXTURE_B8: return std::make_tuple(GL_RED, GL_UNSIGNED_BYTE);
@@ -130,9 +139,9 @@ namespace gl
 		case CELL_GCM_TEXTURE_D1R5G5B5: return std::make_tuple(GL_BGRA, GL_UNSIGNED_SHORT_1_5_5_5_REV);
 		case CELL_GCM_TEXTURE_D8R8G8B8: return std::make_tuple(GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV);
 		case CELL_GCM_TEXTURE_Y16_X16_FLOAT: return std::make_tuple(GL_RG, GL_HALF_FLOAT);
-		case CELL_GCM_TEXTURE_COMPRESSED_DXT1: return std::make_tuple(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT, GL_UNSIGNED_BYTE);
-		case CELL_GCM_TEXTURE_COMPRESSED_DXT23: return std::make_tuple(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT, GL_UNSIGNED_BYTE);
-		case CELL_GCM_TEXTURE_COMPRESSED_DXT45: return std::make_tuple(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, GL_UNSIGNED_BYTE);
+		case CELL_GCM_TEXTURE_COMPRESSED_DXT1: return std::make_tuple(supports_dxt ? GL_COMPRESSED_RGBA_S3TC_DXT1_EXT : GL_BGRA, GL_UNSIGNED_BYTE);
+		case CELL_GCM_TEXTURE_COMPRESSED_DXT23: return std::make_tuple(supports_dxt ? GL_COMPRESSED_RGBA_S3TC_DXT3_EXT : GL_BGRA, GL_UNSIGNED_BYTE);
+		case CELL_GCM_TEXTURE_COMPRESSED_DXT45: return std::make_tuple(supports_dxt ? GL_COMPRESSED_RGBA_S3TC_DXT5_EXT : GL_BGRA, GL_UNSIGNED_BYTE);
 		case CELL_GCM_TEXTURE_COMPRESSED_HILO8: return std::make_tuple(GL_RG, GL_UNSIGNED_BYTE);
 		case CELL_GCM_TEXTURE_COMPRESSED_HILO_S8: return std::make_tuple(GL_RG, GL_BYTE);
 		case CELL_GCM_TEXTURE_COMPRESSED_B8R8_G8R8: return std::make_tuple(GL_BGRA, GL_UNSIGNED_BYTE);
@@ -257,6 +266,44 @@ namespace gl
 		fmt::throw_exception("Unknown format 0x%x", texture_format);
 	}
 
+	GLenum get_compatible_snorm_format(GLenum base_format)
+	{
+		switch (base_format)
+		{
+		case GL_R8:
+			return GL_R8_SNORM;
+		case GL_RG8:
+			return GL_RG8_SNORM;
+		case GL_RGBA8:
+			return GL_RGBA8_SNORM;
+		case GL_R16:
+			return GL_R16_SNORM;
+		case GL_RG16:
+			return GL_RG16_SNORM;
+		case GL_RGBA16:
+			return GL_RGBA16_SNORM;
+		default:
+			return GL_NONE;
+		}
+	}
+
+	GLenum get_compatible_srgb_format(GLenum base_format)
+	{
+		switch (base_format)
+		{
+		case GL_RGBA8:
+			return GL_SRGB8_ALPHA8_EXT;
+		case GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
+			return GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT;
+		case GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
+			return GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT;
+		case GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
+			return GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT;
+		default:
+			return GL_NONE;
+		}
+	}
+
 	cs_shuffle_base* get_trivial_transform_job(const pixel_buffer_layout& pack_info)
 	{
 		if (!pack_info.swap_bytes)
@@ -280,6 +327,8 @@ namespace gl
 	void* copy_image_to_buffer(gl::command_context& cmd, const pixel_buffer_layout& pack_info, const gl::texture* src, gl::buffer* dst,
 		u32 dst_offset, const int src_level, const coord3u& src_region,  image_memory_requirements* mem_info)
 	{
+		ensure(src && dst);
+
 		auto initialize_scratch_mem = [&]() -> bool // skip_transform
 		{
 			const u64 max_mem = (mem_info->memory_required) ? mem_info->memory_required : mem_info->image_size_in_bytes;
@@ -429,7 +478,7 @@ namespace gl
 				image_region.height *= dst_region.depth;
 				scratch = std::make_unique<gl::texture>(
 					GL_TEXTURE_2D,
-					image_region.x + image_region.width, image_region.y + image_region.height, 1, 1,
+					image_region.x + image_region.width, image_region.y + image_region.height, 1, 1, 1,
 					static_cast<GLenum>(dst->get_internal_format()), dst->format_class());
 
 				scratch_view = std::make_unique<gl::nil_texture_view>(scratch.get());
@@ -445,7 +494,7 @@ namespace gl
 			{
 				scratch = std::make_unique<gl::texture>(
 					GL_TEXTURE_2D,
-					image_region.x + image_region.width, 1, 1, 1,
+					image_region.x + image_region.width, 1, 1, 1, 1,
 					static_cast<GLenum>(dst->get_internal_format()), dst->format_class());
 
 				scratch_view = std::make_unique<gl::nil_texture_view>(scratch.get());
@@ -576,20 +625,21 @@ namespace gl
 		const GLenum internal_format = get_sized_internal_format(gcm_format);
 		const auto format_class = rsx::classify_format(gcm_format);
 
-		return new gl::viewable_image(target, width, height, depth, mipmaps, internal_format, format_class);
+		return new gl::viewable_image(target, width, height, depth, mipmaps, 1, internal_format, format_class);
 	}
 
 	void fill_texture(gl::command_context& cmd, texture* dst, int format,
 			const std::vector<rsx::subresource_layout> &input_layouts,
-			bool is_swizzled, GLenum gl_format, GLenum gl_type, rsx::simple_array<std::byte>& staging_buffer)
+			bool is_swizzled, GLenum gl_format, GLenum gl_type, std::span<std::byte> staging_buffer)
 	{
-		const auto driver_caps = gl::get_driver_caps();
+		const auto& driver_caps = gl::get_driver_caps();
 		rsx::texture_uploader_capabilities caps
 		{
 			.supports_byteswap = true,
 			.supports_vtc_decoding = false,
 			.supports_hw_deswizzle = driver_caps.ARB_compute_shader_supported,
 			.supports_zero_copy = false,
+			.supports_dxt = driver_caps.EXT_texture_compression_s3tc_supported,
 			.alignment = 4
 		};
 
@@ -599,7 +649,7 @@ namespace gl
 		glBindBuffer(GL_PIXEL_UNPACK_BUFFER, GL_NONE);
 		glBindBuffer(GL_PIXEL_PACK_BUFFER, GL_NONE);
 
-		if (rsx::is_compressed_host_format(format)) [[likely]]
+		if (rsx::is_compressed_host_format(caps, format)) [[likely]]
 		{
 			caps.supports_vtc_decoding = driver_caps.vendor_NVIDIA;
 			unpack_settings.apply();
@@ -631,7 +681,7 @@ namespace gl
 				{
 					const GLsizei size = layout.width_in_block * layout.height_in_block * format_block_size;
 					ensure(usz(size) <= staging_buffer.size());
-					if (gl::get_driver_caps().ARB_dsa_supported)
+					if (gl::get_driver_caps().ARB_direct_state_access_supported)
 					{
 						glCompressedTextureSubImage3D(dst->id(), layout.level, 0, 0, layout.layer, layout.width_in_texel, layout.height_in_texel, 1, gl_format, size, staging_buffer.data());
 					}
@@ -690,13 +740,13 @@ namespace gl
 				if (driver_caps.ARB_compute_shader_supported)
 				{
 					u64 row_pitch = rsx::align2<u64, u64>(layout.width_in_block * block_size_in_bytes, caps.alignment);
-					if (!rsx::is_compressed_host_format(format))
-					{
-						// Handle emulated compressed formats with host unpack (R8G8 compressed)
-						row_pitch = std::max<u64>(row_pitch, dst->pitch());
-					}
 
-					image_linear_size = row_pitch * layout.height_in_block * layout.depth;
+					// We're in the "else" branch, so "is_compressed_host_format()" is always false.
+					// Handle emulated compressed formats with host unpack (R8G8 compressed)
+					row_pitch = std::max<u64>(row_pitch, dst->pitch());
+
+					// FIXME: Double-check this logic; it seems like we should always use texels both here and for row_pitch.
+					image_linear_size = row_pitch * layout.height_in_texel * layout.depth;
 
 					compute_scratch_mem = { nullptr, g_compute_decode_buffer.alloc(static_cast<u32>(image_linear_size), 256) };
 					compute_scratch_mem.first = reinterpret_cast<void*>(static_cast<uintptr_t>(compute_scratch_mem.second));
@@ -707,7 +757,7 @@ namespace gl
 				}
 
 				rsx::io_buffer io_buf = dst_buffer;
-				caps.supports_hw_deswizzle = (is_swizzled && driver_caps.ARB_compute_shader_supported && image_linear_size > 4096);
+				caps.supports_hw_deswizzle = (is_swizzled && driver_caps.ARB_compute_shader_supported && image_linear_size > 1024);
 				auto op = upload_texture_subresource(io_buf, layout, format, is_swizzled, caps);
 
 				// Define upload region
@@ -748,39 +798,54 @@ namespace gl
 						g_upload_transfer_buffer.copy_to(&g_deswizzle_scratch_buffer.get(), upload_scratch_mem.second, deswizzle_data_offset, static_cast<u32>(image_linear_size));
 
 						// 2.2 Apply compute transform to deswizzle input and dump it in compute_scratch_mem
-						ensure(op.element_size == 2 || op.element_size == 4);
 						const auto block_size = op.element_size * op.block_length;
 
 						if (op.require_swap)
 						{
 							mem_layout.swap_bytes = false;
 
-							if (op.element_size == 4) [[ likely ]]
+							switch (op.element_size)
 							{
-								do_deswizzle_transformation<u32, true>(cmd, block_size,
+							case 1:
+								do_deswizzle_transformation<u8, true>(cmd, block_size,
 									&g_compute_decode_buffer.get(), compute_scratch_mem.second, &g_deswizzle_scratch_buffer.get(), deswizzle_data_offset,
 									static_cast<u32>(image_linear_size), layout.width_in_texel, layout.height_in_texel, layout.depth);
-							}
-							else
-							{
+								break;
+							case 2:
 								do_deswizzle_transformation<u16, true>(cmd, block_size,
 									&g_compute_decode_buffer.get(), compute_scratch_mem.second, &g_deswizzle_scratch_buffer.get(), deswizzle_data_offset,
 									static_cast<u32>(image_linear_size), layout.width_in_texel, layout.height_in_texel, layout.depth);
+								break;
+							case 4:
+								do_deswizzle_transformation<u32, true>(cmd, block_size,
+									&g_compute_decode_buffer.get(), compute_scratch_mem.second, &g_deswizzle_scratch_buffer.get(), deswizzle_data_offset,
+									static_cast<u32>(image_linear_size), layout.width_in_texel, layout.height_in_texel, layout.depth);
+								break;
+							default:
+								fmt::throw_exception("Unimplemented element size deswizzle");
 							}
 						}
 						else
 						{
-							if (op.element_size == 4) [[ likely ]]
+							switch (op.element_size)
 							{
-								do_deswizzle_transformation<u32, false>(cmd, block_size,
+							case 1:
+								do_deswizzle_transformation<u8, false>(cmd, block_size,
 									&g_compute_decode_buffer.get(), compute_scratch_mem.second, &g_deswizzle_scratch_buffer.get(), deswizzle_data_offset,
 									static_cast<u32>(image_linear_size), layout.width_in_texel, layout.height_in_texel, layout.depth);
-							}
-							else
-							{
+								break;
+							case 2:
 								do_deswizzle_transformation<u16, false>(cmd, block_size,
 									&g_compute_decode_buffer.get(), compute_scratch_mem.second, &g_deswizzle_scratch_buffer.get(), deswizzle_data_offset,
 									static_cast<u32>(image_linear_size), layout.width_in_texel, layout.height_in_texel, layout.depth);
+								break;
+							case 4:
+								do_deswizzle_transformation<u32, false>(cmd, block_size,
+									&g_compute_decode_buffer.get(), compute_scratch_mem.second, &g_deswizzle_scratch_buffer.get(), deswizzle_data_offset,
+									static_cast<u32>(image_linear_size), layout.width_in_texel, layout.height_in_texel, layout.depth);
+								break;
+							default:
+								fmt::throw_exception("Unimplemented element size deswizzle");
 							}
 						}
 
@@ -816,9 +881,10 @@ namespace gl
 	void upload_texture(gl::command_context& cmd, texture* dst, u32 gcm_format, bool is_swizzled, const std::vector<rsx::subresource_layout>& subresources_layout)
 	{
 		// Calculate staging buffer size
-		rsx::simple_array<std::byte> data_upload_buf;
+		rsx::simple_array<std::byte, sizeof(u128)> data_upload_buf;
 
-		if (rsx::is_compressed_host_format(gcm_format))
+		rsx::texture_uploader_capabilities caps { .supports_dxt = gl::get_driver_caps().EXT_texture_compression_s3tc_supported };
+		if (rsx::is_compressed_host_format(caps, gcm_format))
 		{
 			const auto& desc = subresources_layout[0];
 			const u32 texture_data_sz = desc.width_in_block * desc.height_in_block * desc.depth * rsx::get_format_block_size_in_bytes(gcm_format);
