@@ -1,12 +1,11 @@
 #include "stdafx.h"
 #include "overlays.h"
 #include "overlay_manager.h"
-#include "overlay_message_dialog.h"
 #include "Input/pad_thread.h"
 #include "Emu/Io/interception.h"
 #include "Emu/Io/KeyboardHandler.h"
 #include "Emu/RSX/RSXThread.h"
-#include "Emu/RSX/Common/time.hpp"
+#include "Emu/System.h"
 
 LOG_CHANNEL(overlays);
 
@@ -16,6 +15,33 @@ namespace rsx
 {
 	namespace overlays
 	{
+		std::string get_sound_filepath(sound_effect sound)
+		{
+			const auto get_sound_filename = [sound]()
+			{
+				switch (sound)
+				{
+				case sound_effect::cursor:       return "snd_cursor"sv;
+				case sound_effect::accept:       return "snd_decide"sv;
+				case sound_effect::cancel:       return "snd_cancel"sv;
+				case sound_effect::osk_accept:   return "snd_oskenter"sv;
+				case sound_effect::osk_cancel:   return "snd_oskcancel"sv;
+				case sound_effect::dialog_ok:    return "snd_system_ok"sv;
+				case sound_effect::dialog_error: return "snd_system_ng"sv;
+				case sound_effect::trophy:       return "snd_trophy"sv;
+				}
+
+				fmt::throw_exception("Unreachable (sound=%d)", static_cast<u32>(sound));
+			};
+
+			return fmt::format("%ssounds/%s.wav", fs::get_config_dir(), get_sound_filename());
+		}
+
+		void play_sound(sound_effect sound, std::optional<f32> volume)
+		{
+			Emu.GetCallbacks().play_sound(get_sound_filepath(sound), volume);
+		}
+
 		thread_local DECLARE(user_interface::g_thread_bit) = 0;
 
 		u32 user_interface::alloc_thread_bit()
@@ -226,7 +252,7 @@ namespace rsx
 						continue;
 					}
 
-					if (!(pad->m_port_status & CELL_PAD_STATUS_CONNECTED))
+					if (!pad->is_connected() || pad->is_copilot())
 					{
 						continue;
 					}
@@ -315,7 +341,7 @@ namespace rsx
 						continue;
 					}
 
-					for (const Button& button : pad->m_buttons)
+					for (const Button& button : pad->m_buttons_external)
 					{
 						pad_button button_id = pad_button::pad_button_max_enum;
 						if (button.m_offset == CELL_PAD_BTN_OFFSET_DIGITAL1)
@@ -392,7 +418,7 @@ namespace rsx
 							break;
 					}
 
-					for (const AnalogStick& stick : pad->m_sticks)
+					for (const AnalogStick& stick : pad->m_sticks_external)
 					{
 						pad_button button_id = pad_button::pad_button_max_enum;
 						pad_button release_id = pad_button::pad_button_max_enum;
