@@ -1,9 +1,9 @@
 #include "stdafx.h"
 
+#include "Emu/System.h"
 #include "RSXFIFO.h"
 #include "RSXThread.h"
 #include "Capture/rsx_capture.h"
-#include "Common/time.hpp"
 #include "Core/RSXReservationLock.hpp"
 #include "Emu/Memory/vm_reservation.h"
 #include "Emu/Cell/lv2/sys_rsx.h"
@@ -11,6 +11,7 @@
 
 #include "util/asm.hpp"
 
+#include <thread>
 #include <bitset>
 
 using spu_rdata_t = std::byte[128];
@@ -138,7 +139,7 @@ namespace rsx
 				u32 bytes_read = 0;
 
 				// Find the next set bit after every iteration
-				for (int i = 0;; i = (std::countr_zero<u32>(utils::rol8(to_fetch, 0 - i - 1)) + i + 1) % 8)
+				for (int i = 0;; i = (std::countr_zero<u32>(std::rotl<u8>(to_fetch, 0 - i - 1)) + i + 1) % 8)
 				{
 					// If a reservation is being updated, try to load another
 					const auto& res = vm::reservation_acquire(addr1 + i * 128);
@@ -222,18 +223,49 @@ namespace rsx
 			m_remaining_commands = 0;
 		}
 
-		std::span<const u32> FIFO_control::get_current_arg_ptr() const
+		std::span<const u32> FIFO_control::get_current_arg_ptr(u32 length_in_words) const
 		{
+			if (!length_in_words)
+			{
+				// This means the caller is doing something stupid
+				rsx_log.error("Invalid access to FIFO args data, requested length = 0");
+				return {};
+			}
+
 			if (g_cfg.core.rsx_fifo_accuracy)
 			{
 				// Return a pointer to the cache storage with confined access
-				return {reinterpret_cast<const u32*>(&m_cache) + (m_internal_get - m_cache_addr) / 4, (m_cache_size - (m_internal_get - m_cache_addr)) / 4};
+				const u32 cache_offset_in_words = (m_internal_get - m_cache_addr) / 4;
+				const u32 cache_size_in_words = m_cache_size / 4;
+				return {reinterpret_cast<const u32*>(&m_cache) + cache_offset_in_words, cache_size_in_words - cache_offset_in_words};
 			}
-			else
+
+			// Return a raw pointer to contiguous memory
+			constexpr u32 _1M = 0x100000;
+			const u32 size = length_in_words * sizeof(u32);
+			const u32 from = m_iotable->get_addr(m_internal_get);
+
+			for (u32 remaining = size, addr = m_internal_get, ptr = from; remaining > 0;)
 			{
-				// Return a raw pointer with no limited access
-				return {static_cast<const u32*>(vm::base(m_iotable->get_addr(m_internal_get))), 0x10000};
+				const u32 next_block = utils::align(addr + 1, _1M);
+				const u32 available = (next_block - addr);
+				if (remaining <= available)
+				{
+					return { static_cast<const u32*>(vm::base(from)), length_in_words };
+				}
+
+				remaining -= available;
+				const u32 next_ptr = m_iotable->get_addr(next_block);
+				if (next_ptr != (ptr + available))
+				{
+					return { static_cast<const u32*>(vm::base(from)), (size - remaining) / sizeof(u32)};
+				}
+
+				ptr = next_ptr;
+				addr = next_block;
 			}
+
+			fmt::throw_exception("Unreachable");
 		}
 
 		bool FIFO_control::read_unsafe(register_pair& data)
