@@ -1,7 +1,8 @@
 #include "stdafx.h"
-#include "VKGSRender.h"
+#include "Emu/RSX/VK/VKGSRenderTypes.hpp"
 #include "VKTextureCache.h"
 #include "VKCompute.h"
+#include "VKAsyncScheduler.h"
 
 #include "util/asm.hpp"
 
@@ -61,7 +62,7 @@ namespace vk
 		}
 	}
 
-	void cached_texture_section::dma_transfer(vk::command_buffer& cmd, vk::image* src, const areai& src_area, const utils::address_range& valid_range, u32 pitch)
+	void cached_texture_section::dma_transfer(vk::command_buffer& cmd, vk::image* src, const areai& src_area, const utils::address_range32& valid_range, u32 pitch)
 	{
 		ensure(src->samples() == 1);
 
@@ -301,7 +302,7 @@ namespace vk
 			{
 				dma_sync(true);
 
-				std::vector<VkBufferCopy> copy;
+				rsx::simple_array<VkBufferCopy> copy;
 				copy.reserve(transfer_height);
 
 				u32 dst_offset = dma_mapping.first;
@@ -397,11 +398,13 @@ namespace vk
 		m_cached_memory_size = 0;
 	}
 
-	void texture_cache::copy_transfer_regions_impl(vk::command_buffer& cmd, vk::image* dst, const std::vector<copy_region_descriptor>& sections_to_transfer) const
+	void texture_cache::copy_transfer_regions_impl(vk::command_buffer& cmd, vk::image* dst, const rsx::simple_array<copy_region_descriptor>& sections_to_transfer) const
 	{
 		const auto dst_aspect = dst->aspect();
 		const auto dst_bpp = vk::get_format_texel_width(dst->format());
 
+		std::unordered_set<decltype(sections_to_transfer.front().src)> processed_input_images;
+
 		for (const auto& section : sections_to_transfer)
 		{
 			if (!section.src)
@@ -435,11 +438,15 @@ namespace vk
 			const bool typeless = section.src->aspect() != dst_aspect ||
 				!formats_are_bitcast_compatible(dst, section.src);
 
-			// Avoid inserting unnecessary barrier GENERAL->TRANSFER_SRC->GENERAL in active render targets
-			const auto preferred_layout = (section.src->current_layout != VK_IMAGE_LAYOUT_GENERAL) ?
-				VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL : VK_IMAGE_LAYOUT_GENERAL;
+			if (!processed_input_images.contains(section.src))
+			{
+				// Avoid inserting unnecessary barrier GENERAL->TRANSFER_SRC->GENERAL in active render targets
+				const auto preferred_layout = (section.src->current_layout != VK_IMAGE_LAYOUT_GENERAL) ?
+					VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL : VK_IMAGE_LAYOUT_GENERAL;
 
-			section.src->push_layout(cmd, preferred_layout);
+				section.src->push_layout(cmd, preferred_layout);
+				processed_input_images.insert(section.src);
+			}
 
 			auto src_image = section.src;
 			auto src_x = section.src_x;
@@ -478,8 +485,6 @@ namespace vk
 					const areai src_rect = coordi{{ src_x, src_y }, { src_w, src_h }};
 					const areai dst_rect = coordi{{ section.dst_x, section.dst_y }, { section.dst_w, section.dst_h }};
 					vk::copy_image_typeless(cmd, section.src, dst, src_rect, dst_rect, 1);
-
-					section.src->pop_layout(cmd);
 					continue;
 				}
 
@@ -540,8 +545,12 @@ namespace vk
 					vkCmdCopyImage(cmd, _dst->value, _dst->current_layout, dst->value, dst->current_layout, 1, &copy_rgn);
 				}
 			}
+		}
 
-			section.src->pop_layout(cmd);
+		// Pop unique image layouts here
+		for (auto& image : processed_input_images)
+		{
+			image->pop_layout(cmd);
 		}
 	}
 
@@ -584,7 +593,7 @@ namespace vk
 		return mapping;
 	}
 
-	vk::image* texture_cache::get_template_from_collection_impl(const std::vector<copy_region_descriptor>& sections_to_transfer) const
+	vk::image* texture_cache::get_template_from_collection_impl(const rsx::simple_array<copy_region_descriptor>& sections_to_transfer) const
 	{
 		if (sections_to_transfer.size() == 1) [[likely]]
 		{
@@ -707,13 +716,13 @@ namespace vk
 			view_swizzle = source->native_component_map;
 		}
 
-		image->set_debug_name("Temp view");
+		image->set_debug_name(fmt::format("Temp view, fmt=0x%x", gcm_format));
 		image->set_native_component_layout(view_swizzle);
 		auto view = image->get_view(remap_vector);
 
 		if (copy)
 		{
-			std::vector<copy_region_descriptor> region =
+			rsx::simple_array<copy_region_descriptor> region =
 			{ {
 				.src = source,
 				.xform = rsx::surface_transform::coordinate_transform,
@@ -749,7 +758,7 @@ namespace vk
 	}
 
 	vk::image_view* texture_cache::generate_cubemap_from_images(vk::command_buffer& cmd, u32 gcm_format, u16 size,
-		const std::vector<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
+		const rsx::simple_array<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
 	{
 		auto _template = get_template_from_collection_impl(sections_to_copy);
 		auto result = create_temporary_subresource_view_impl(cmd, _template, VK_IMAGE_TYPE_2D,
@@ -784,7 +793,7 @@ namespace vk
 	}
 
 	vk::image_view* texture_cache::generate_3d_from_2d_images(vk::command_buffer& cmd, u32 gcm_format, u16 width, u16 height, u16 depth,
-		const std::vector<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
+		const rsx::simple_array<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
 	{
 		auto _template = get_template_from_collection_impl(sections_to_copy);
 		auto result = create_temporary_subresource_view_impl(cmd, _template, VK_IMAGE_TYPE_3D,
@@ -819,7 +828,7 @@ namespace vk
 	}
 
 	vk::image_view* texture_cache::generate_atlas_from_images(vk::command_buffer& cmd, u32 gcm_format, u16 width, u16 height,
-		const std::vector<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
+		const rsx::simple_array<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
 	{
 		auto _template = get_template_from_collection_impl(sections_to_copy);
 		auto result = create_temporary_subresource_view_impl(cmd, _template, VK_IMAGE_TYPE_2D,
@@ -857,7 +866,7 @@ namespace vk
 	}
 
 	vk::image_view* texture_cache::generate_2d_mipmaps_from_images(vk::command_buffer& cmd, u32 gcm_format, u16 width, u16 height,
-		const std::vector<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
+		const rsx::simple_array<copy_region_descriptor>& sections_to_copy, const rsx::texture_channel_remap_t& remap_vector)
 	{
 		const auto mipmaps = ::narrow<u8>(sections_to_copy.size());
 		auto _template = get_template_from_collection_impl(sections_to_copy);
@@ -904,7 +913,7 @@ namespace vk
 
 	void texture_cache::update_image_contents(vk::command_buffer& cmd, vk::image_view* dst_view, vk::image* src, u16 width, u16 height)
 	{
-		std::vector<copy_region_descriptor> region =
+		rsx::simple_array<copy_region_descriptor> region =
 		{ {
 			.src = src,
 			.xform = rsx::surface_transform::identity,
@@ -920,7 +929,7 @@ namespace vk
 		dst->pop_layout(cmd);
 	}
 
-	cached_texture_section* texture_cache::create_new_texture(vk::command_buffer& cmd, const utils::address_range& rsx_range, u16 width, u16 height, u16 depth, u16 mipmaps, u32 pitch,
+	cached_texture_section* texture_cache::create_new_texture(vk::command_buffer& cmd, const utils::address_range32& rsx_range, u16 width, u16 height, u16 depth, u16 mipmaps, u32 pitch,
 		u32 gcm_format, rsx::texture_upload_context context, rsx::texture_dimension_extended type, bool swizzled, rsx::component_order swizzle_flags, rsx::flags32_t flags)
 	{
 		const auto section_depth = depth;
@@ -1016,6 +1025,29 @@ namespace vk
 			const VkFormat vk_format = get_compatible_sampler_format(m_formats_support, gcm_format);
 			VkImageCreateFlags create_flags = is_cubemap ? VK_IMAGE_CREATE_CUBE_COMPATIBLE_BIT : 0;
 			VkSharingMode sharing_mode = (flags & texture_create_flags::shareable) ? VK_SHARING_MODE_CONCURRENT : VK_SHARING_MODE_EXCLUSIVE;
+			rsx::simple_array<VkFormat> mutable_format_list;
+
+			if (flags & texture_create_flags::mutable_format)
+			{
+				const VkFormat snorm_fmt = get_compatible_snorm_format(vk_format);
+				const VkFormat srgb_fmt = get_compatible_srgb_format(vk_format);
+
+				if (snorm_fmt != VK_FORMAT_UNDEFINED)
+				{
+					mutable_format_list.push_back(snorm_fmt);
+				}
+
+				if (srgb_fmt != VK_FORMAT_UNDEFINED)
+				{
+					mutable_format_list.push_back(srgb_fmt);
+				}
+
+				if (!mutable_format_list.empty())
+				{
+					create_flags |= VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT;
+					mutable_format_list.push_back(vk_format);
+				}
+			}
 
 			if (auto found = find_cached_image(vk_format, width, height, depth, mipmaps, image_type, create_flags, usage_flags, sharing_mode))
 			{
@@ -1028,9 +1060,16 @@ namespace vk
 					create_flags |= VK_IMAGE_CREATE_SHAREABLE_RPCS3;
 				}
 
+				VkFormatEx create_format = vk_format;
+				if (!mutable_format_list.empty())
+				{
+					create_format.pViewFormats = mutable_format_list.data();
+					create_format.viewFormatCount = mutable_format_list.size();
+				}
+
 				image = new vk::viewable_image(*m_device,
 					m_memory_types.device_local, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
-					image_type, vk_format,
+					image_type, create_format,
 					width, height, depth, mipmaps, layer, VK_SAMPLE_COUNT_1_BIT, VK_IMAGE_LAYOUT_UNDEFINED,
 					VK_IMAGE_TILING_OPTIMAL, usage_flags, create_flags,
 					VMM_ALLOCATION_POOL_TEXTURE_CACHE, rsx::classify_format(gcm_format));
@@ -1075,7 +1114,7 @@ namespace vk
 
 	cached_texture_section* texture_cache::create_nul_section(
 		vk::command_buffer& /*cmd*/,
-		const utils::address_range& rsx_range,
+		const utils::address_range32& rsx_range,
 		const rsx::image_section_attributes_t& attrs,
 		const rsx::GCM_tile_reference& tile,
 		bool memory_load)
@@ -1100,7 +1139,7 @@ namespace vk
 		return &region;
 	}
 
-	cached_texture_section* texture_cache::upload_image_from_cpu(vk::command_buffer& cmd, const utils::address_range& rsx_range, u16 width, u16 height, u16 depth, u16 mipmaps, u32 pitch, u32 gcm_format,
+	cached_texture_section* texture_cache::upload_image_from_cpu(vk::command_buffer& cmd, const utils::address_range32& rsx_range, u16 width, u16 height, u16 depth, u16 mipmaps, u32 pitch, u32 gcm_format,
 		rsx::texture_upload_context context, const std::vector<rsx::subresource_layout>& subresource_layout, rsx::texture_dimension_extended type, bool swizzled)
 	{
 		if (context != rsx::texture_upload_context::shader_read)
@@ -1114,7 +1153,13 @@ namespace vk
 		const bool upload_async = rsx::get_current_renderer()->get_backend_config().supports_asynchronous_compute;
 		rsx::flags32_t create_flags = 0;
 
-		if (upload_async && g_fxo->get<AsyncTaskScheduler>().is_host_mode())
+		if (context == rsx::texture_upload_context::shader_read &&
+			!g_cfg.video.disable_hardware_texel_remapping)
+		{
+			create_flags |= texture_create_flags::mutable_format;
+		}
+
+		if (upload_async && ensure(g_fxo->try_get<AsyncTaskScheduler>())->is_host_mode())
 		{
 			create_flags |= texture_create_flags::do_not_reuse;
 			if (m_device->get_graphics_queue() != m_device->get_transfer_queue())
@@ -1249,7 +1294,7 @@ namespace vk
 		{
 		default:
 			//TODO
-			err_once("Format incompatibility detected, reporting failure to force data copy (VK_FORMAT=0x%X, GCM_FORMAT=0x%X)", static_cast<u32>(vk_format), gcm_format);
+			warn_once("Format incompatibility detected, reporting failure to force data copy (VK_FORMAT=0x%X, GCM_FORMAT=0x%X)", static_cast<u32>(vk_format), gcm_format);
 			return false;
 #ifndef __APPLE__
 		case CELL_GCM_TEXTURE_R5G6B5:
@@ -1338,7 +1383,7 @@ namespace vk
 			cmd.submit(submit_info, VK_TRUE);
 			vk::wait_for_fence(&submit_fence, GENERAL_WAIT_TIMEOUT);
 
-			CHECK_RESULT(vkResetCommandBuffer(cmd, 0));
+			cmd.reset();
 			cmd.begin();
 		}
 		else
