@@ -163,6 +163,7 @@ enum class localized_string_id
 	RPCN_ERROR_INVALID_INPUT,
 	RPCN_ERROR_WOLFSSL,
 	RPCN_ERROR_RESOLVE,
+	RPCN_ERROR_BINDING,
 	RPCN_ERROR_CONNECT,
 	RPCN_ERROR_LOGIN_ERROR,
 	RPCN_ERROR_ALREADY_LOGGED,
@@ -216,6 +217,7 @@ enum class localized_string_id
 	HOME_MENU_SETTINGS_VIDEO_OUTPUT_SCALING,
 	HOME_MENU_SETTINGS_VIDEO_RCAS_SHARPENING,
 	HOME_MENU_SETTINGS_VIDEO_STRETCH_TO_DISPLAY,
+	HOME_MENU_SETTINGS_VIDEO_STEREO_MODE,
 	HOME_MENU_SETTINGS_INPUT,
 	HOME_MENU_SETTINGS_INPUT_BACKGROUND_INPUT,
 	HOME_MENU_SETTINGS_INPUT_KEEP_PADS_CONNECTED,
@@ -243,6 +245,7 @@ enum class localized_string_id
 	HOME_MENU_SETTINGS_OVERLAYS_SHOW_PRESSURE_INTENSITY_TOGGLE_HINT,
 	HOME_MENU_SETTINGS_OVERLAYS_SHOW_ANALOG_LIMITER_TOGGLE_HINT,
 	HOME_MENU_SETTINGS_OVERLAYS_SHOW_MOUSE_AND_KB_TOGGLE_HINT,
+	HOME_MENU_SETTINGS_OVERLAYS_RECORD_WITH_OVERLAYS,
 	HOME_MENU_SETTINGS_PERFORMANCE_OVERLAY,
 	HOME_MENU_SETTINGS_PERFORMANCE_OVERLAY_ENABLE,
 	HOME_MENU_SETTINGS_PERFORMANCE_OVERLAY_ENABLE_FRAMERATE_GRAPH,
@@ -263,6 +266,7 @@ enum class localized_string_id
 	HOME_MENU_SETTINGS_DEBUG,
 	HOME_MENU_SETTINGS_DEBUG_OVERLAY,
 	HOME_MENU_SETTINGS_DEBUG_INPUT_OVERLAY,
+	HOME_MENU_SETTINGS_MOUSE_DEBUG_INPUT_OVERLAY,
 	HOME_MENU_SETTINGS_DEBUG_DISABLE_VIDEO_OUTPUT,
 	HOME_MENU_SETTINGS_DEBUG_TEXTURE_LOD_BIAS,
 	HOME_MENU_SCREENSHOT,
@@ -270,12 +274,18 @@ enum class localized_string_id
 	HOME_MENU_SAVESTATE_SAVE,
 	HOME_MENU_SAVESTATE_AND_EXIT,
 	HOME_MENU_RELOAD_SAVESTATE,
+	HOME_MENU_RELOAD_SECOND_SAVESTATE,
+	HOME_MENU_RELOAD_THIRD_SAVESTATE,
+	HOME_MENU_RELOAD_FOURTH_SAVESTATE,
+	HOME_MENU_TOGGLE_FULLSCREEN,
 	HOME_MENU_RECORDING,
 	HOME_MENU_TROPHIES,
 	HOME_MENU_TROPHY_LIST_TITLE,
 	HOME_MENU_TROPHY_LOCKED_TITLE,
 	HOME_MENU_TROPHY_HIDDEN_TITLE,
 	HOME_MENU_TROPHY_HIDDEN_DESCRIPTION,
+	HOME_MENU_TROPHY_SHOW_HIDDEN_TROPHIES,
+	HOME_MENU_TROPHY_HIDE_HIDDEN_TROPHIES,
 	HOME_MENU_TROPHY_PLATINUM_RELEVANT,
 	HOME_MENU_TROPHY_GRADE_BRONZE,
 	HOME_MENU_TROPHY_GRADE_SILVER,
