@@ -196,22 +196,6 @@ void fmt_class_string<screen_quadrant>::format(std::string& out, u64 arg)
 	});
 }
 
-template <>
-void fmt_class_string<tsx_usage>::format(std::string& out, u64 arg)
-{
-	format_enum(out, arg, [](tsx_usage value)
-	{
-		switch (value)
-		{
-		case tsx_usage::disabled: return "Disabled";
-		case tsx_usage::enabled: return "Enabled";
-		case tsx_usage::forced: return "Forced";
-		}
-
-		return unknown;
-	});
-}
-
 template <>
 void fmt_class_string<rsx_fifo_mode>::format(std::string& out, u64 arg)
 {
@@ -374,6 +358,9 @@ void fmt_class_string<camera_handler>::format(std::string& out, u64 arg)
 		case camera_handler::null: return "Null";
 		case camera_handler::fake: return "Fake";
 		case camera_handler::qt: return "Qt";
+#ifdef HAVE_SDL3
+		case camera_handler::sdl: return "SDL";
+#endif
 		}
 
 		return unknown;
@@ -716,3 +703,34 @@ void fmt_class_string<xfloat_accuracy>::format(std::string& out, u64 arg)
 		return unknown;
 	});
 }
+
+template <>
+void fmt_class_string<date_format>::format(std::string& out, u64 arg)
+{
+	format_enum(out, arg, [](date_format value)
+	{
+		switch (value)
+		{
+		case date_format::yyyymmdd: return "yyyymmdd";
+		case date_format::ddmmyyyy: return "ddmmyyyy";
+		case date_format::mmddyyyy: return "mmddyyyy";
+		}
+
+		return unknown;
+	});
+}
+
+template <>
+void fmt_class_string<time_format>::format(std::string& out, u64 arg)
+{
+	format_enum(out, arg, [](time_format value)
+	{
+		switch (value)
+		{
+		case time_format::clock12: return "clock12";
+		case time_format::clock24: return "clock24";
+		}
+
+		return unknown;
+	});
+}
