@@ -8,7 +8,6 @@
 #include "evdev_joystick_handler.h"
 #include "util/logs.hpp"
 
-#include <functional>
 #include <algorithm>
 #include <unistd.h>
 #include <fcntl.h>
@@ -19,6 +18,33 @@
 
 LOG_CHANNEL(evdev_log, "evdev");
 
+bool positive_axis::load()
+{
+	if (fs::file cfg_file{ cfg_name, fs::read })
+	{
+		return from_string(cfg_file.to_string());
+	}
+
+	from_default();
+	return false;
+}
+
+void positive_axis::save() const
+{
+	fs::pending_file file(cfg_name);
+
+	if (file.file)
+	{
+		file.file.write(to_string());
+		file.commit();
+	}
+}
+
+bool positive_axis::exist() const
+{
+	return fs::is_file(cfg_name);
+}
+
 evdev_joystick_handler::evdev_joystick_handler()
     : PadHandlerBase(pad_handler::evdev)
 {
@@ -94,8 +120,6 @@ void evdev_joystick_handler::init_config(cfg_pad* cfg)
 	cfg->rstickdeadzone.def    = 30; // between 0 and 255
 	cfg->ltriggerthreshold.def = 0;  // between 0 and 255
 	cfg->rtriggerthreshold.def = 0;  // between 0 and 255
-	cfg->lpadsquircling.def    = 8000;
-	cfg->rpadsquircling.def    = 8000;
 
 	// apply defaults
 	cfg->from_default();
@@ -111,7 +135,12 @@ bool evdev_joystick_handler::Init()
 	if (m_is_init)
 		return true;
 
-	m_pos_axis_config.load();
+	if (!m_pos_axis_config.load())
+	{
+		evdev_log.notice("positive_axis config missing. Using defaults");
+	}
+
+	evdev_log.notice("positive_axis config=\n%s", m_pos_axis_config.to_string());
 
 	if (!m_pos_axis_config.exist())
 		m_pos_axis_config.save();
@@ -390,7 +419,7 @@ PadHandlerBase::connection evdev_joystick_handler::get_next_button_press(const s
 	if (call_type != gui_call_type::blacklist && call_type != gui_call_type::reset_input && !has_new_event)
 	{
 		if (callback)
-			callback(0, "", padId, 0, preview_values);
+			callback(0, "", padId, 0, preview_values, get_capabilities(padId));
 		return connection::no_data;
 	}
 
@@ -512,10 +541,12 @@ PadHandlerBase::connection evdev_joystick_handler::get_next_button_press(const s
 
 	if (callback)
 	{
+		pad_capabilities capabilities = get_capabilities(padId);
+
 		if (pressed_button.value > 0)
-			callback(pressed_button.value, pressed_button.name, padId, 0, std::move(preview_values));
+			callback(pressed_button.value, pressed_button.name, padId, 0, std::move(preview_values), std::move(capabilities));
 		else
-			callback(0, "", padId, 0, std::move(preview_values));
+			callback(0, "", padId, 0, std::move(preview_values), std::move(capabilities));
 	}
 
 	return connection::connected;
@@ -784,7 +815,7 @@ std::shared_ptr<evdev_joystick_handler::EvdevDevice> evdev_joystick_handler::add
 					// Let's log axis information while we are in the settings in order to identify problems more easily.
 					for (const auto& [code, axis_name] : axis_list)
 					{
-						if (const input_absinfo *info = libevdev_get_abs_info(dev, code))
+						if (const input_absinfo* info = libevdev_get_abs_info(dev, code))
 						{
 							const char* code_name = libevdev_event_code_get_name(EV_ABS, code);
 							evdev_log.notice("Axis info for %s: %s (%s) => minimum=%d, maximum=%d, fuzz=%d, flat=%d, resolution=%d",
@@ -858,7 +889,7 @@ std::shared_ptr<evdev_joystick_handler::EvdevDevice> evdev_joystick_handler::add
 				// A device must not mix regular directional axes and accelerometer axes on the same event node.
 				for (const auto& [code, axis_name] : axis_list)
 				{
-					if (const input_absinfo *info = libevdev_get_abs_info(dev, code))
+					if (const input_absinfo* info = libevdev_get_abs_info(dev, code))
 					{
 						const bool is_accel = code == ABS_X || code == ABS_Y || code == ABS_Z;
 						const char* code_name = libevdev_event_code_get_name(EV_ABS, code);
@@ -1271,8 +1302,8 @@ void evdev_joystick_handler::apply_pad_data(const pad_ensemble& binding)
 		return;
 
 	// Handle vibration
-	const u8 force_large = cfg->get_large_motor_speed(pad->m_vibrateMotors);
-	const u8 force_small = cfg->get_small_motor_speed(pad->m_vibrateMotors);
+	const u8 force_large = cfg->get_large_motor_speed(pad->m_vibrate_motors);
+	const u8 force_small = cfg->get_small_motor_speed(pad->m_vibrate_motors);
 	SetRumble(evdev_device, force_large, force_small);
 }
 
@@ -1315,7 +1346,7 @@ bool evdev_joystick_handler::bindPadToDevice(std::shared_ptr<Pad> pad)
 
 	const auto find_buttons = [&](const cfg::string& name) -> std::set<u32>
 	{
-		const std::vector<std::string> names = cfg_pad::get_buttons(name);
+		const std::vector<std::string> names = cfg_pad::get_buttons(name.to_string());
 
 		// In evdev we store indices to an EvdevButton vector in our pad objects instead of the usual key codes.
 		std::set<u32> indices;
@@ -1447,8 +1478,8 @@ bool evdev_joystick_handler::bindPadToDevice(std::shared_ptr<Pad> pad)
 	pad->m_sensors[2] = AnalogSensor(CELL_PAD_BTN_OFFSET_SENSOR_Z, m_dev->axis_motion[2].code, m_dev->axis_motion[2].mirrored, m_dev->axis_motion[2].shift, DEFAULT_MOTION_Z);
 	pad->m_sensors[3] = AnalogSensor(CELL_PAD_BTN_OFFSET_SENSOR_G, m_dev->axis_motion[3].code, m_dev->axis_motion[3].mirrored, m_dev->axis_motion[3].shift, DEFAULT_MOTION_G);
 
-	pad->m_vibrateMotors[0] = VibrateMotor(true, 0);
-	pad->m_vibrateMotors[1] = VibrateMotor(false, 0);
+	pad->m_vibrate_motors[0] = VibrateMotor(true);
+	pad->m_vibrate_motors[1] = VibrateMotor(false);
 
 	if (std::shared_ptr<EvdevDevice> evdev_device = add_device(player_config->device, false))
 	{
