@@ -9,16 +9,17 @@
 #include "Emu/System.h"
 #include "Emu/system_config.h"
 #include "Emu/system_progress.hpp"
+#include "Emu/savestate_utils.hpp"
 #include "Emu/IdManager.h"
 #include "Emu/Audio/audio_utils.h"
 #include "Emu/Cell/Modules/cellScreenshot.h"
-#include "Emu/Cell/Modules/cellVideoOut.h"
 #include "Emu/Cell/Modules/cellAudio.h"
 #include "Emu/Cell/lv2/sys_rsxaudio.h"
 #include "Emu/RSX/rsx_utils.h"
 #include "Emu/RSX/Overlays/overlay_message.h"
 #include "Emu/Io/interception.h"
 #include "Emu/Io/recording_config.h"
+#include "Input/pad_thread.h"
 
 #include <QApplication>
 #include <QDateTime>
@@ -50,24 +51,19 @@ LOG_CHANNEL(screenshot_log, "SCREENSHOT");
 LOG_CHANNEL(mark_log, "MARK");
 LOG_CHANNEL(gui_log, "GUI");
 
+extern atomic_t<bool> g_user_asked_for_fullscreen;
 extern atomic_t<bool> g_user_asked_for_recording;
 extern atomic_t<bool> g_user_asked_for_screenshot;
 extern atomic_t<bool> g_user_asked_for_frame_capture;
 extern atomic_t<bool> g_disable_frame_limit;
+extern atomic_t<bool> g_game_window_focused;
 extern atomic_t<recording_mode> g_recording_mode;
 
-atomic_t<bool> g_game_window_focused = false;
-
 namespace pad
 {
 	extern atomic_t<bool> g_home_menu_requested;
 }
 
-bool is_input_allowed()
-{
-	return g_game_window_focused || g_cfg.io.background_input_enabled;
-}
-
 gs_frame::gs_frame(QScreen* screen, const QRect& geometry, const QIcon& appIcon, std::shared_ptr<gui_settings> gui_settings, bool force_fullscreen)
 	: QWindow()
 	, m_initial_geometry(geometry)
@@ -75,8 +71,6 @@ gs_frame::gs_frame(QScreen* screen, const QRect& geometry, const QIcon& appIcon,
 	, m_start_games_fullscreen(force_fullscreen)
 	, m_renderer(g_cfg.video.renderer)
 {
-	load_gui_settings();
-
 	m_window_title = Emu.GetFormattedTitle(0);
 
 	if (!g_cfg_recording.load())
@@ -128,12 +122,15 @@ gs_frame::gs_frame(QScreen* screen, const QRect& geometry, const QIcon& appIcon,
 		create();
 	}
 
-	m_shortcut_handler = new shortcut_handler(gui::shortcuts::shortcut_handler_id::game_window, this, m_gui_settings);
-	connect(m_shortcut_handler, &shortcut_handler::shortcut_activated, this, &gs_frame::handle_shortcut);
+	load_gui_settings();
 
 	// Change cursor when in fullscreen.
-	connect(this, &QWindow::visibilityChanged, this, [this](QWindow::Visibility visibility)
+	connect(this, &QWindow::visibilityChanged, this, [this](Visibility visibility)
 	{
+		if (visibility != Visibility::Minimized && visibility != Visibility::Hidden)
+		{
+			m_visibility = visibility; // Only save "visible" visibility
+		}
 		handle_cursor(visibility, true, false, true);
 	});
 
@@ -154,6 +151,9 @@ gs_frame::gs_frame(QScreen* screen, const QRect& geometry, const QIcon& appIcon,
 gs_frame::~gs_frame()
 {
 	g_user_asked_for_screenshot = false;
+	g_user_asked_for_recording = false;
+	g_user_asked_for_frame_capture = false;
+	g_user_asked_for_fullscreen = false;
 	pad::g_home_menu_requested = false;
 
 	// Save active screen to gui settings
@@ -181,6 +181,20 @@ void gs_frame::load_gui_settings()
 	m_lock_mouse_in_fullscreen  = m_gui_settings->GetValue(gui::gs_lockMouseFs).toBool();
 	m_hide_mouse_after_idletime = m_gui_settings->GetValue(gui::gs_hideMouseIdle).toBool();
 	m_hide_mouse_idletime = m_gui_settings->GetValue(gui::gs_hideMouseIdleTime).toUInt();
+
+	if (m_disable_kb_hotkeys)
+	{
+		if (m_shortcut_handler)
+		{
+			m_shortcut_handler->deleteLater();
+			m_shortcut_handler = nullptr;
+		}
+	}
+	else if (!m_shortcut_handler)
+	{
+		m_shortcut_handler = new shortcut_handler(gui::shortcuts::shortcut_handler_id::game_window, this, m_gui_settings);
+		connect(m_shortcut_handler, &shortcut_handler::shortcut_activated, this, &gs_frame::handle_shortcut);
+	}
 }
 
 void gs_frame::update_shortcuts()
@@ -191,12 +205,12 @@ void gs_frame::update_shortcuts()
 	}
 }
 
-void gs_frame::paintEvent(QPaintEvent *event)
+void gs_frame::paintEvent(QPaintEvent* event)
 {
 	Q_UNUSED(event)
 }
 
-void gs_frame::showEvent(QShowEvent *event)
+void gs_frame::showEvent(QShowEvent* event)
 {
 	// We have to calculate new window positions, since the frame is only known once the window was created.
 	// We will try to find the originally requested dimensions if possible by moving the frame.
@@ -307,6 +321,10 @@ void gs_frame::handle_shortcut(gui::shortcuts::shortcut shortcut_key, const QKey
 		break;
 	}
 	case gui::shortcuts::shortcut::gw_restart:
+	case gui::shortcuts::shortcut::gw_savestate_1:
+	case gui::shortcuts::shortcut::gw_savestate_2:
+	case gui::shortcuts::shortcut::gw_savestate_3:
+	case gui::shortcuts::shortcut::gw_savestate_4:
 	{
 		if (Emu.IsStopped())
 		{
@@ -314,8 +332,27 @@ void gs_frame::handle_shortcut(gui::shortcuts::shortcut shortcut_key, const QKey
 			return;
 		}
 
-		extern bool boot_last_savestate(bool testing);
-		boot_last_savestate(false);
+		u32 index = 1;
+
+		switch (shortcut_key)
+		{
+		case gui::shortcuts::shortcut::gw_restart: index = 1; break;
+		case gui::shortcuts::shortcut::gw_savestate_1: index = 1; break;
+		case gui::shortcuts::shortcut::gw_savestate_2: index = 2; break;
+		case gui::shortcuts::shortcut::gw_savestate_3: index = 3; break;
+		case gui::shortcuts::shortcut::gw_savestate_4: index = 4; break;
+		default: break; // unreachable
+		}
+
+		if (shortcut_key == gui::shortcuts::shortcut::gw_restart && !boot_current_game_savestate(true, index))
+		{
+			// Normal restart if there is no savestate
+			Emu.Restart();
+			break;
+		}
+
+		// Reboot with savestate
+		boot_current_game_savestate(false, index);
 		break;
 	}
 	case gui::shortcuts::shortcut::gw_savestate:
@@ -370,6 +407,15 @@ void gs_frame::handle_shortcut(gui::shortcuts::shortcut shortcut_key, const QKey
 		audio::change_volume(-5);
 		break;
 	}
+	case gui::shortcuts::shortcut::gw_toggle_mouse_gyro:
+	{
+		if (auto* pad_thr = pad::get_pad_thread(true))
+		{
+			const bool mouse_gyro_enabled = pad_thr->get_mouse_gyro().toggle_enabled();
+			gui_log.notice("Mouse-based gyro emulation %s", mouse_gyro_enabled ? "enabled" : "disabled");
+		}
+		break;
+	}
 	default:
 	{
 		break;
@@ -423,16 +469,19 @@ void gs_frame::toggle_recording()
 		// Play a sound
 		if (const std::string sound_path = fs::get_config_dir() + "sounds/snd_recording.wav"; fs::is_file(sound_path))
 		{
-			Emu.GetCallbacks().play_sound(sound_path);
+			Emu.GetCallbacks().play_sound(sound_path, std::nullopt);
 		}
 		else
 		{
 			QApplication::beep();
 		}
 
-		ensure(m_video_encoder->path().starts_with(fs::get_config_dir()));
-		const std::string shortpath = m_video_encoder->path().substr(fs::get_config_dir().size() - 1); // -1 for /
-		rsx::overlays::queue_message(tr("Recording saved: %0").arg(QString::fromStdString(shortpath)).toStdString());
+		if (g_cfg.misc.show_capture_hints)
+		{
+			ensure(m_video_encoder->path().starts_with(fs::get_config_dir()));
+			const std::string shortpath = m_video_encoder->path().substr(fs::get_config_dir().size() - 1); // -1 for /
+			rsx::overlays::queue_message(tr("Recording saved: %0").arg(QString::fromStdString(shortpath)).toStdString());
+		}
 	}
 	else
 	{
@@ -504,7 +553,11 @@ void gs_frame::toggle_recording()
 
 		if (m_video_encoder->has_error)
 		{
-			rsx::overlays::queue_message(tr("Recording not possible").toStdString());
+			if (g_cfg.misc.show_capture_hints)
+			{
+				rsx::overlays::queue_message(tr("Recording not possible").toStdString());
+			}
+
 			m_video_encoder->stop();
 			return;
 		}
@@ -512,7 +565,12 @@ void gs_frame::toggle_recording()
 		if (!video_provider.set_video_sink(m_video_encoder, recording_mode::rpcs3))
 		{
 			gui_log.warning("The video provider could not set the video sink. A sink with higher priority must have been set.");
-			rsx::overlays::queue_message(tr("Recording not possible").toStdString());
+
+			if (g_cfg.misc.show_capture_hints)
+			{
+				rsx::overlays::queue_message(tr("Recording not possible").toStdString());
+			}
+
 			m_video_encoder->stop();
 			return;
 		}
@@ -521,7 +579,10 @@ void gs_frame::toggle_recording()
 
 		g_recording_mode = recording_mode::rpcs3;
 
-		rsx::overlays::queue_message(tr("Recording started").toStdString());
+		if (g_cfg.misc.show_capture_hints)
+		{
+			rsx::overlays::queue_message(tr("Recording started").toStdString());
+		}
 	}
 }
 
@@ -548,7 +609,7 @@ void gs_frame::update_cursor()
 		// Hide the mouse if the idle timeout was reached (which means that the timer isn't running)
 		show_mouse = false;
 	}
-	else if (visibility() == QWindow::Visibility::FullScreen)
+	else if (visibility() == Visibility::FullScreen)
 	{
 		// Fullscreen: Show or hide the mouse depending on the settings.
 		show_mouse = m_show_mouse_in_fullscreen;
@@ -577,7 +638,7 @@ void gs_frame::hide_on_close()
 {
 	// Make sure not to save the hidden state, which is useless to us.
 	const Visibility current_visibility = visibility();
-	m_gui_settings->SetValue(gui::gs_visibility, current_visibility == Visibility::Hidden ? Visibility::AutomaticVisibility : current_visibility, false);
+	m_gui_settings->SetValue(gui::gs_visibility, current_visibility == Visibility::Hidden ? m_visibility : current_visibility, false);
 	m_gui_settings->SetValue(gui::gs_geometry, geometry(), true);
 
 	if (!g_progr_text)
@@ -657,9 +718,6 @@ void gs_frame::show()
 			}
 		}
 	});
-
-	// if we do this before show, the QWinTaskbarProgress won't show
-	m_progress_indicator->show(this);
 }
 
 display_handle_t gs_frame::handle() const
@@ -747,7 +805,7 @@ f64 gs_frame::client_display_rate()
 	return rate;
 }
 
-void gs_frame::flip(draw_context_t, bool /*skip_frame*/)
+void gs_frame::flip(draw_context_t /*context*/, bool /*skip_frame*/)
 {
 	static Timer fps_t;
 
@@ -786,6 +844,14 @@ void gs_frame::flip(draw_context_t, bool /*skip_frame*/)
 			toggle_recording();
 		});
 	}
+
+	if (g_user_asked_for_fullscreen.exchange(false))
+	{
+		Emu.CallFromMainThread([this]()
+		{
+			toggle_fullscreen();
+		});
+	}
 }
 
 bool gs_frame::can_consume_frame() const
@@ -794,13 +860,13 @@ bool gs_frame::can_consume_frame() const
 	return video_provider.can_consume_frame();
 }
 
-void gs_frame::present_frame(std::vector<u8>& data, u32 pitch, u32 width, u32 height, bool is_bgra) const
+void gs_frame::present_frame(std::vector<u8>&& data, u32 pitch, u32 width, u32 height, bool is_bgra) const
 {
 	utils::video_provider& video_provider = g_fxo->get<utils::video_provider>();
-	video_provider.present_frame(data, pitch, width, height, is_bgra);
+	video_provider.present_frame(std::move(data), pitch, width, height, is_bgra);
 }
 
-void gs_frame::take_screenshot(std::vector<u8> data, u32 sshot_width, u32 sshot_height, bool is_bgra)
+void gs_frame::take_screenshot(std::vector<u8>&& data, u32 sshot_width, u32 sshot_height, bool is_bgra)
 {
 	std::thread(
 		[sshot_width, sshot_height, is_bgra](std::vector<u8> sshot_data)
@@ -1004,7 +1070,7 @@ void gs_frame::take_screenshot(std::vector<u8> data, u32 sshot_width, u32 sshot_
 					}
 				}
 
-				const std::string cell_sshot_filename = manager.get_screenshot_path(date_time.toString("yyyy/MM/dd").toStdString());
+				const std::string cell_sshot_filename = Emu.GetCallbacks().get_photo_path(manager.get_photo_title() + ".png");
 				const std::string cell_sshot_dir      = fs::get_parent_dir(cell_sshot_filename);
 
 				screenshot_log.notice("Saving cell screenshot to %s", cell_sshot_filename);
@@ -1034,7 +1100,7 @@ void gs_frame::take_screenshot(std::vector<u8> data, u32 sshot_width, u32 sshot_
 			{
 				if (const std::string sound_path = fs::get_config_dir() + "sounds/snd_screenshot.wav"; fs::is_file(sound_path))
 				{
-					Emu.GetCallbacks().play_sound(sound_path);
+					Emu.GetCallbacks().play_sound(sound_path, std::nullopt);
 				}
 				else
 				{
@@ -1042,9 +1108,12 @@ void gs_frame::take_screenshot(std::vector<u8> data, u32 sshot_width, u32 sshot_
 				}
 			});
 
-			ensure(filename.starts_with(fs::get_config_dir()));
-			const std::string shortpath = filename.substr(fs::get_config_dir().size() - 1); // -1 for /
-			rsx::overlays::queue_message(tr("Screenshot saved: %0").arg(QString::fromStdString(shortpath)).toStdString());
+			if (g_cfg.misc.show_capture_hints)
+			{
+				ensure(filename.starts_with(fs::get_config_dir()));
+				const std::string shortpath = filename.substr(fs::get_config_dir().size() - 1); // -1 for /
+				rsx::overlays::queue_message(tr("Screenshot saved: %0").arg(QString::fromStdString(shortpath)).toStdString());
+			}
 
 			return;
 		},
@@ -1077,7 +1146,7 @@ void gs_frame::mouseDoubleClickEvent(QMouseEvent* ev)
 	}
 }
 
-void gs_frame::handle_cursor(QWindow::Visibility visibility, bool visibility_changed, bool active_changed, bool start_idle_timer)
+void gs_frame::handle_cursor(Visibility visibility, bool visibility_changed, bool active_changed, bool start_idle_timer)
 {
 	// Let's reload the gui settings when the visibility or the active window changes.
 	if (visibility_changed || active_changed)
@@ -1088,7 +1157,7 @@ void gs_frame::handle_cursor(QWindow::Visibility visibility, bool visibility_cha
 		if (visibility_changed)
 		{
 			// In fullscreen we default to hiding and locking. In windowed mode we do not want the lock by default.
-			m_mouse_hide_and_lock = (visibility == QWindow::Visibility::FullScreen) && m_lock_mouse_in_fullscreen;
+			m_mouse_hide_and_lock = (visibility == Visibility::FullScreen) && m_lock_mouse_in_fullscreen;
 		}
 	}
 
@@ -1108,7 +1177,7 @@ void gs_frame::handle_cursor(QWindow::Visibility visibility, bool visibility_cha
 
 void gs_frame::mouse_hide_timeout()
 {
-	// Our idle timeout occured, so we update the cursor
+	// Our idle timeout occurred, so we update the cursor
 	if (m_hide_mouse_after_idletime && m_show_mouse)
 	{
 		handle_cursor(visibility(), false, false, false);
@@ -1169,6 +1238,16 @@ bool gs_frame::event(QEvent* ev)
 		// This will make the cursor visible again if it was hidden by the mouse idle timeout
 		handle_cursor(visibility(), false, false, true);
 	}
+
+	// Handle events for mouse-based gyro emulation.
+	if (Emu.IsRunning())
+	{
+		if (auto* pad_thr = pad::get_pad_thread(true))
+		{
+			pad_thr->get_mouse_gyro().handle_event(ev, *this);
+		}
+	}
+
 	return QWindow::event(ev);
 }
 
