@@ -12,12 +12,11 @@
 
 #include "Emu/system_utils.hpp"
 #include "Utilities/File.h"
+#include "Loader/ISO.h"
 #include <cmath>
 
 LOG_CHANNEL(gui_log, "GUI");
 
-constexpr auto qstr = QString::fromStdString;
-
 namespace gui
 {
 	namespace utils
@@ -174,21 +173,45 @@ namespace gui
 			}
 			return res;
 		}
-		
-		QColor get_foreground_color()
+
+		QColor get_foreground_color(QWidget* widget)
 		{
+			if (widget)
+			{
+				widget->ensurePolished();
+				return widget->palette().color(QPalette::ColorRole::WindowText);
+			}
+
 			QLabel dummy_color;
 			dummy_color.ensurePolished();
 			return dummy_color.palette().color(QPalette::ColorRole::WindowText);
 		}
 
-		QColor get_background_color()
+		QColor get_background_color(QWidget* widget)
 		{
+			if (widget)
+			{
+				widget->ensurePolished();
+				return widget->palette().color(QPalette::ColorRole::Window);
+			}
+
 			QLabel dummy_color;
 			dummy_color.ensurePolished();
 			return dummy_color.palette().color(QPalette::ColorRole::Window);
 		}
 
+		QColor adjust_color_for_background(const QColor& fg, const QColor& bg)
+		{
+			const int diff = fg.lightness() - bg.lightness();
+
+			if (std::abs(diff) >= 40)
+			{
+				return fg;
+			}
+
+			return (bg.lightness() < 128) ? fg.lighter(180) : fg.darker(180);
+		}
+
 		QColor get_label_color(const QString& object_name, const QColor& fallback_light, const QColor& fallback_dark, QPalette::ColorRole color_role)
 		{
 			if (!gui::custom_stylesheet_active || !gui::stylesheet.contains(object_name))
@@ -232,6 +255,23 @@ namespace gui
 			return QString("style=\"color: %0;\"").arg(get_link_color_string(name));
 		}
 
+		QString make_link(const QString& text, const QString& url)
+		{
+			return QString("<a %0 href=\"%1\">%2</a>").arg(get_link_style()).arg(url).arg(text);
+		}
+
+		QString make_bold(const QString& text)
+		{
+			return QString("<span style=\"font-weight:600;\">%0</span>").arg(text);
+		}
+
+		QString make_paragraph(QString text, const QString& white_space_style)
+		{
+			return QString(R"(<p%0>%1</p>)")
+				.arg(white_space_style.isEmpty() ? "" : "style=\"white-space: nowrap;\"")
+				.arg(text.replace("\n", "<br>"));
+		}
+
 		QPixmap get_centered_pixmap(QPixmap pixmap, const QSize& icon_size, int offset_x, int offset_y, qreal device_pixel_ratio, Qt::TransformationMode mode)
 		{
 			// Create empty canvas for expanded image
@@ -377,27 +417,46 @@ namespace gui
 			std::string icon_path = fs::get_config_dir() + "/Icons/game_icons/" + title_id + "/ICON0.PNG";
 			bool found_file       = fs::is_file(icon_path);
 
+			std::unique_ptr<QPixmap> pixmap;
+
 			if (!found_file)
 			{
 				// Get Icon for the gs_frame from path. this handles presumably all possible use cases
-				const QString qpath = qstr(path);
-				const std::string path_list[] = { path, qpath.section("/", 0, -2, QString::SectionIncludeTrailingSep).toStdString(),
-					                              qpath.section("/", 0, -3, QString::SectionIncludeTrailingSep).toStdString() };
+				std::vector<std::string> path_list;
 
-				for (const std::string& pth : path_list)
+				const bool is_archive = is_file_iso(path);
+				if (is_archive)
 				{
-					if (!fs::is_dir(pth))
+					icon_path = "PS3_GAME/ICON0.PNG";
+
+					QPixmap px;
+					if (load_iso_icon(px, icon_path, path))
 					{
-						continue;
+						found_file = true;
+						pixmap = std::make_unique<QPixmap>(std::move(px));
 					}
+				}
+				else
+				{
+					const QString qpath = QString::fromStdString(path);
+					path_list = { path, qpath.section("/", 0, -2, QString::SectionIncludeTrailingSep).toStdString(),
+					                    qpath.section("/", 0, -3, QString::SectionIncludeTrailingSep).toStdString() };
 
-					const std::string sfo_dir = rpcs3::utils::get_sfo_dir_from_game_path(pth, title_id);
-					icon_path = sfo_dir + "/ICON0.PNG";
-					found_file = fs::is_file(icon_path);
-
-					if (found_file)
+					for (const std::string& pth : path_list)
 					{
-						break;
+						if (!fs::is_dir(pth))
+						{
+							continue;
+						}
+
+						const std::string sfo_dir = rpcs3::utils::get_sfo_dir_from_game_path(pth, title_id);
+						icon_path = sfo_dir + "/ICON0.PNG";
+						found_file = fs::is_file(icon_path);
+
+						if (found_file)
+						{
+							break;
+						}
 					}
 				}
 			}
@@ -405,7 +464,7 @@ namespace gui
 			if (found_file)
 			{
 				// load the image from path. It will most likely be a rectangle
-				const QImage source = QImage(qstr(icon_path));
+				const QImage source = pixmap ? pixmap->toImage() : QImage(QString::fromStdString(icon_path));
 				const int edge_max = std::max(source.width(), source.height());
 
 				// create a new transparent image with square size and same format as source (maybe handle other formats than RGB32 as well?)
@@ -431,7 +490,7 @@ namespace gui
 
 		void open_dir(const std::string& spath)
 		{
-			QString path = qstr(spath);
+			QString path = QString::fromStdString(spath);
 
 			if (fs::is_file(spath))
 			{
@@ -453,7 +512,7 @@ namespace gui
 				QProcess::execute("/usr/bin/osascript", { "-e", "tell application \"Finder\" to activate" });
 #else
 				// open parent directory
-				const QUrl url = QUrl::fromLocalFile(qstr(fs::get_parent_dir(spath)));
+				const QUrl url = QUrl::fromLocalFile(QString::fromStdString(fs::get_parent_dir(spath)));
 				const std::string url_path = url.toString().toStdString();
 				gui_log.notice("gui::utils::open_dir: About to open parent dir url '%s' for path '%s'", url_path, spath);
 
@@ -626,7 +685,7 @@ namespace gui
 			usz byte_unit = 0;
 			usz divisor = 1;
 #if defined(__APPLE__)
-			constexpr usz multiplier = 1000; 
+			constexpr usz multiplier = 1000;
 			static const QString s_units[]{"B", "kB", "MB", "GB", "TB", "PB"};
 #else
 			constexpr usz multiplier = 1024;
@@ -641,5 +700,65 @@ namespace gui
 
 			return QStringLiteral("%0 %1").arg(QString::number((size + 0.) / divisor, 'f', 2)).arg(s_units[byte_unit]);
 		}
+
+		QDateTime datetime(s64 time)
+		{
+			QDateTime dateTime;
+			dateTime.setSecsSinceEpoch(time);
+			return dateTime;
+		}
+
+		QString format_datetime(const QDateTime& date, const QString& fmt, bool is_relative, const QString& fmt_relative)
+		{
+			const qint64 exctrated_date = date.date().toJulianDay();
+			const qint64 current_date = QDate::currentDate().toJulianDay();
+
+			if (!is_relative || exctrated_date > current_date || current_date - exctrated_date >= 3)
+			{
+				return date.toString(fmt);
+			}
+
+			if (current_date == exctrated_date)
+			{
+				return QString("Today %1").arg(date.toString(fmt_relative));
+			}
+
+			return QString("%1 days ago %2").arg(current_date - exctrated_date).arg(date.toString(fmt_relative));
+		}
+
+		bool load_iso_icon(QPixmap& icon, const std::string& icon_path, const std::string& archive_path)
+		{
+			if (icon_path.empty() || archive_path.empty()) return false;
+			if (!is_file_iso(archive_path)) return false;
+
+			iso_archive archive(archive_path);
+			if (!archive.exists(icon_path)) return false;
+
+			auto icon_file = archive.open(icon_path);
+			const auto icon_size = icon_file.size();
+			if (icon_size == 0) return false;
+
+			QByteArray data(icon_size, 0);
+			icon_file.read(data.data(), icon_size);
+
+			return icon.loadFromData(data);
+		}
+
+		bool load_icon(QPixmap& icon, const std::string& icon_path, const std::string& archive_path)
+		{
+			if (icon_path.empty()) return false;
+
+			if (archive_path.empty())
+			{
+				return icon.load(QString::fromStdString(icon_path));
+			}
+
+			return load_iso_icon(icon, icon_path, archive_path);
+		}
+
+		QString format_timestamp(s64 time, const QString& fmt)
+		{
+			return format_datetime(datetime(time), fmt);
+		}
 	} // utils
 } // gui
