@@ -16,9 +16,15 @@
 #else
 #include <unistd.h>
 #include <sys/resource.h>
-#ifndef __APPLE__
+#ifdef __APPLE__
+#include <sys/sysctl.h>
+#else
 #include <sys/utsname.h>
 #include <errno.h>
+#if defined(ARCH_ARM64) && defined(__linux__)
+#include <sys/auxv.h>
+#include <asm/hwcap.h>
+#endif
 #endif
 #endif
 
@@ -71,9 +77,10 @@ namespace Darwin_ProcessInfo
 }
 #endif
 
+#ifdef _WIN32
+#if !defined(ARCH_X64)
 namespace utils
 {
-#ifdef _WIN32
 	// Some helpers for sanity
 	const auto read_reg_dword = [](HKEY hKey, std::string_view value_name) -> std::pair<bool, DWORD>
 	{
@@ -151,8 +158,9 @@ namespace utils
 
 		return fmt::format("Operating system: %s, Version %s", product_name, version_id);
 	}
-#endif
 }
+#endif
+#endif
 
 bool utils::has_ssse3()
 {
@@ -278,17 +286,6 @@ bool utils::has_avx10()
 #endif
 }
 
-bool utils::has_avx10_512()
-{
-#if defined(ARCH_X64)
-	// AVX10 with 512 wide vectors
-	static const bool g_value = has_avx10() && get_cpuid(24, 0)[2] & 0x40000;
-	return g_value;
-#else
-	return false;
-#endif
-}
-
 u32 utils::avx10_isa_version()
 {
 #if defined(ARCH_X64)
@@ -310,28 +307,6 @@ u32 utils::avx10_isa_version()
 #endif
 }
 
-bool utils::has_avx512_256()
-{
-#if defined(ARCH_X64)
-	// Either AVX10 or AVX512 implies support for 256-bit length AVX-512 SKL-X tier instructions
-	static const bool g_value = (has_avx512() || has_avx10());
-	return g_value;
-#else
-	return false;
-#endif
-}
-
-bool utils::has_avx512_icl_256()
-{
-#if defined(ARCH_X64)
-	// Check for AVX512_ICL or check for AVX10, together with GFNI, VAES, and VPCLMULQDQ, implies support for the same instructions that AVX-512_icl does at 256 bit length
-	static const bool g_value = (has_avx512_icl() || (has_avx10() && get_cpuid(7, 0)[2] & 0x00000700));
-	return g_value;
-#else
-	return false;
-#endif
-}
-
 bool utils::has_xop()
 {
 #if defined(ARCH_X64)
@@ -475,6 +450,103 @@ u32 utils::get_rep_movsb_threshold()
 	return g_value;
 }
 
+#ifdef ARCH_ARM64
+
+bool utils::has_neon()
+{
+	static const bool g_value = []() -> bool
+	{
+#if defined(__linux__)
+		return (getauxval(AT_HWCAP) & HWCAP_ASIMD) != 0;
+#elif defined(__APPLE__)
+		int val = 0;
+		size_t len = sizeof(val);
+		sysctlbyname("hw.optional.AdvSIMD", &val, &len, nullptr, 0);
+		int val_legacy = 0;
+		size_t len_legacy = sizeof(val_legacy);
+		sysctlbyname("hw.optional.neon", &val_legacy, &len_legacy, nullptr, 0);
+		return val != 0 || val_legacy != 0;
+#elif defined(_WIN32)
+		return IsProcessorFeaturePresent(PF_ARM_VFP_32_REGISTERS_AVAILABLE) != 0;
+#endif
+	}();
+	return g_value;
+}
+
+bool utils::has_sha3()
+{
+	static const bool g_value = []() -> bool
+	{
+#if defined(__linux__)
+		return (getauxval(AT_HWCAP) & HWCAP_SHA3) != 0;
+#elif defined(__APPLE__)
+		int val = 0;
+		size_t len = sizeof(val);
+		sysctlbyname("hw.optional.arm.FEAT_SHA3", &val, &len, nullptr, 0);
+		return val != 0;
+#elif defined(_WIN32)
+		return IsProcessorFeaturePresent(PF_ARM_SHA3_INSTRUCTIONS_AVAILABLE) != 0;
+#endif
+	}();
+	return g_value;
+}
+
+bool utils::has_dotprod()
+{
+	static const bool g_value = []() -> bool
+	{
+#if defined(__linux__)
+		return (getauxval(AT_HWCAP) & HWCAP_ASIMDDP) != 0;
+#elif defined(__APPLE__)
+		int val = 0;
+		size_t len = sizeof(val);
+		sysctlbyname("hw.optional.arm.FEAT_DotProd", &val, &len, nullptr, 0);
+		return val != 0;
+#elif defined(_WIN32)
+		return IsProcessorFeaturePresent(PF_ARM_V82_DP_INSTRUCTIONS_AVAILABLE) != 0;
+#endif
+	}();
+	return g_value;
+}
+
+bool utils::has_sve()
+{
+	static const bool g_value = []() -> bool
+	{
+#if defined(__linux__)
+		return (getauxval(AT_HWCAP) & HWCAP_SVE) != 0;
+#elif defined(__APPLE__)
+		int val = 0;
+		size_t len = sizeof(val);
+		sysctlbyname("hw.optional.arm.FEAT_SVE", &val, &len, nullptr, 0);
+		return val != 0;
+#elif defined(_WIN32)
+		return IsProcessorFeaturePresent(PF_ARM_SVE_INSTRUCTIONS_AVAILABLE) != 0;
+#endif
+	}();
+	return g_value;
+}
+
+bool utils::has_sve2()
+{
+	static const bool g_value = []() -> bool
+	{
+#if defined(__linux__)
+		return (getauxval(AT_HWCAP2) & HWCAP2_SVE2) != 0;
+#elif defined(__APPLE__)
+		int val = 0;
+		size_t len = sizeof(val);
+		sysctlbyname("hw.optional.arm.FEAT_SVE2", &val, &len, nullptr, 0);
+		return val != 0;
+#elif defined(_WIN32)
+		return IsProcessorFeaturePresent(PF_ARM_SVE2_INSTRUCTIONS_AVAILABLE) != 0;
+#endif
+	}();
+	return g_value;
+}
+
+#endif
+
 std::string utils::get_cpu_brand()
 {
 #if defined(ARCH_X64)
@@ -527,6 +599,17 @@ std::string utils::get_system_info()
 	{
 		fmt::append(result, " | TSC: Disabled");
 	}
+#ifdef ARCH_ARM64
+
+	if (has_neon())
+	{
+		result += " | Neon";
+	}
+	else
+	{
+		fmt::throw_exception("Neon support not present");
+	}
+#else
 
 	if (has_avx())
 	{
@@ -536,15 +619,6 @@ std::string utils::get_system_info()
 		{
 			const u32 avx10_version = avx10_isa_version();
 			fmt::append(result, "10.%d", avx10_version);
-
-			if (has_avx10_512())
-			{
-				result += "-512";
-			}
-			else
-			{
-				result += "-256";
-			}
 		}
 		else if (has_avx512())
 		{
@@ -602,6 +676,7 @@ std::string utils::get_system_info()
 	{
 		result += " | TSX disabled via microcode";
 	}
+#endif
 
 	return result;
 }
@@ -890,7 +965,7 @@ static const bool s_tsc_freq_evaluated = []() -> bool
 		printf("[TSC calibration] Available clock sources: '%s'\n", clock_sources.c_str());
 
 		// Check if the Kernel has blacklisted the TSC
-		const auto available_clocks = fmt::split(clock_sources, { " " });
+		const auto available_clocks = fmt::split_sv(clock_sources, { " " });
 		const bool tsc_reliable = std::find(available_clocks.begin(), available_clocks.end(), "tsc") != available_clocks.end();
 
 		if (!tsc_reliable)
